Package: shinystan
Title: Interactive Visual and Numerical Diagnostics and Posterior
        Analysis for Bayesian Models
Version: 2.7.0
Date: 2025-12-11
Authors@R: c(person("Jonah", "Gabry", role = c("aut", "cre"),
    email = "jgabry@gmail.com"),
      person("Duco", "Veen", role = "aut"),
      person(family = "Stan Development Team", role = "ctb"),
      person("Michael", "Andreae", role = "ctb"),
      person("Michael", "Betancourt", role = "ctb"),
      person("Bob", "Carpenter", role = "ctb"),
      person("Yuanjun", "Gao", role = "ctb"),
      person("Andrew", "Gelman", role = "ctb"),
      person("Ben", "Goodrich", role = "ctb"),
      person("Daniel", "Lee", role = "ctb"),
      person("Dongying", "Song", role = "ctb"),
      person("Rob", "Trangucci", role = "ctb"), 
      person("Visruth", "Srimath Kandali", role = "ctb"))
Maintainer: Jonah Gabry <jgabry@gmail.com>
Description: A graphical user interface for interactive Markov chain Monte
    Carlo (MCMC) diagnostics and plots and tables helpful for analyzing a
    posterior sample. The interface is powered by the 'Shiny' web
    application framework from 'RStudio' and works with the output of MCMC 
    programs written in any programming language (and has extended 
    functionality for 'Stan' models fit using the 'rstan' and 'rstanarm' 
    packages).
URL: https://mc-stan.org/shinystan/, https://discourse.mc-stan.org
BugReports: https://github.com/stan-dev/shinystan/issues/
License: GPL (>= 3)
Depends: R (>= 3.1.0), shiny (>= 1.0.3)
Imports: bayesplot (>= 1.5.0), colourpicker, DT (>= 0.2), dygraphs (>=
        1.1.1.2), ggplot2 (>= 3.4.0), gridExtra, gtools, markdown (>=
        0.7.4), methods, reshape2, rstan (>= 2.17.1), stats, shinyjs
        (>= 0.6.0), shinythemes (>= 1.0.1), threejs (>= 0.2.1), utils,
        xtable, xts (>= 0.9-7)
Suggests: cmdstanr (>= 0.9.0), coda, knitr (>= 1.9), posterior (>=
        1.0.0), rmarkdown (>= 0.8.1), rsconnect (>= 0.4.2), rstanarm
        (>= 2.17.4), testthat
Additional_repositories: https://stan-dev.r-universe.dev/
LazyData: true
Encoding: UTF-8
VignetteBuilder: knitr
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2025-12-11 18:24:13 UTC; jgabry
Author: Jonah Gabry [aut, cre],
  Duco Veen [aut],
  Stan Development Team [ctb],
  Michael Andreae [ctb],
  Michael Betancourt [ctb],
  Bob Carpenter [ctb],
  Yuanjun Gao [ctb],
  Andrew Gelman [ctb],
  Ben Goodrich [ctb],
  Daniel Lee [ctb],
  Dongying Song [ctb],
  Rob Trangucci [ctb],
  Visruth Srimath Kandali [ctb]
Repository: CRAN
Date/Publication: 2025-12-12 07:10:18 UTC
Built: R 4.6.0; ; 2026-04-25 12:41:30 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(launch_shinystan,default)
S3method(launch_shinystan,shinystan)
export(as.shinystan)
export(deploy_shinystan)
export(drop_parameters)
export(generate_quantity)
export(is.shinystan)
export(launch_shinystan)
export(launch_shinystan_demo)
export(model_code)
export(model_name)
export(notes)
export(rename_model)
export(retrieve)
export(sso_info)
export(update_sso)
import(bayesplot)
import(dygraphs)
import(ggplot2)
import(methods)
import(shiny)
import(threejs)
import(utils)
import(xts)
importFrom(DT,datatable)
importFrom(colourpicker,colourInput)
importFrom(gridExtra,arrangeGrob)
importFrom(gridExtra,grid.arrange)
importFrom(gtools,mixedsort)
importFrom(markdown,markdownToHTML)
importFrom(reshape2,melt)
importFrom(shinyjs,useShinyjs)
importFrom(shinythemes,shinytheme)
importFrom(stats,acf)
importFrom(stats,na.omit)
importFrom(stats,quantile)
importFrom(stats,time)
importFrom(stats,var)
importFrom(xtable,print.xtable)
importFrom(xtable,xtable)

navlistPanel(
  id = "pp_navlist",
  widths = c(3, 9),
  well = TRUE,
  tabPanel(
    "Distribution of observed data vs replications",
    div(
      br(),
      h4(withMathJax(plot_descriptions["plot_hists_rep_vs_obs"])),
      br(),
      # actionButton("resample_go", label = "Show different replications", icon = icon("refresh")),
      fluidRow(
        column(
          width = 5,
          radioButtons(
            "pp_rep_vs_obs_overlay_rstanarm",
            label = "",
            choices = list(Histograms = "histograms", "Overlaid Densities" = "density"),
            inline = TRUE
          )
        )
      ), 
      plotOutput("pp_rep_vs_obs_out_rstanarm"),
      br()
    )
  ),
  tabPanel(
    "Distributions of test statistics",
    div(
      br(),
      h4(withMathJax(plot_descriptions["plot_test_statistics"])),
      helpText(
        "The blue lines show \\(T(y)\\), the value of the statistic computed from the observed data."
      ),
      fluidRow(
        column(
          width = 6,
          plotOutput("pp_hists_test_statistics_mean_out_rstanarm", height = "200px")
        ), 
        column(
          width = 6,
          plotOutput("pp_hists_test_statistics_sd_out_rstanarm", height = "200px")
        )
      ), 
      br(),
      fluidRow(
        column(
          width = 6,
          plotOutput("pp_hists_test_statistics_min_out_rstanarm", height = "200px")
        ),
        column(
          width = 6,
          plotOutput("pp_hists_test_statistics_max_out_rstanarm", height = "200px")
        )
      ), 
      br()
    )
  ),
  tabPanel(
    "Scatterplots",
    div(
      br(),
      h4(withMathJax(plot_descriptions["plot_obs_vs_avg_y_rep"])),
      plotOutput("pp_y_vs_avg_rep_out_rstanarm"),
      br()
    )
  ), 
  tabPanel(
    "Histograms of residuals",
    div(
      br(),
      h4(withMathJax(plot_descriptions["plot_hist_resids"])),
      # br(),
      # actionButton("resample_resids_go", label = "Show a different replication", icon = icon("refresh")),
      # br(),br(),
      plotOutput("pp_hist_resids_out_rstanarm")
    )
  ), 
  "About",
  tabPanel(
    "About graphical posterior predictive checking",
    source(file.path("ui_files", "pp_about.R"), local = TRUE)$value
  )
)

# 
# # multiparameter traceplots -----------------------------------------------
# calc_height_trace_plot <- reactive({
#   params <- input$multitrace_params
#   grid <- FALSE
#   if (!is.null(input$multitrace_layout)) {
#     if (input$multitrace_layout == "Grid") grid <- TRUE
#   }
#   params <- .update_params_with_groups(params, param_names)
#   LL <- length(params)
#   if (LL == 0) LL <- 4
#   if (LL == 1) LL <- 2
#   if (grid) {
#     if (LL > 5) return(30*LL)
#     if (LL < 5) return(60*LL)
#   }
#   round(100*LL)
# })
# 
# # multitrace_plot
# multitrace_plot <- reactive({
#   validate(need(!is.null(input$multitrace_rect), message = "Loading..."))
#   x1 <- input$multi_xzoom[1]
#   x2 <- input$multi_xzoom[2]
#   dat <- samps_all[x1:x2,,,drop=FALSE]
#   # zoom <- "On"
#   do.call(".param_trace_multi", args = list(
#     params      = input$multitrace_params,
#     all_param_names = param_names,
#     dat         = dat,
#     chain       = input$multitrace_chain,
#     warmup_val  = warmup_val,
#     palette     = input$multitrace_palette ,
#     rect        = input$multitrace_rect,
#     rect_color  = "skyblue",
#     rect_alpha  = input$multitrace_rect_alpha,
#     layout      = input$multitrace_layout,
#     x1          = x1,
#     x2          = x2
#   ))
# })
# 
# output$multitrace_plot_out <- renderPlot({
#   x <- multitrace_plot()
#   suppressWarnings(print(x)) # this avoids warnings about removing rows when using tracezoom feature
# }, height = calc_height_trace_plot, bg = "transparent")
# 
# # download the plot
# output$download_multitrace <- downloadHandler(
#   filename = paste0('shinystan_multitrace.RData'),
#   content = function(file) {
#     shinystan_multitrace <- multitrace_plot()
#     save(shinystan_multitrace, file = file)
#   }
# )

summary_stats <- reactive({
  `%>%` <- DT::`%>%`
  validate(need(input$table_digits, "loading"))
  DT::datatable(data = round(TABLE_STATS, digits = input$table_digits),
                colnames = c('mcse' = 'se_mean'),
                options = list(
                  colReorder = list(realtime = TRUE),
                  # dom = 'RBfClrTtip',
                  dom = "Bflrtip",
                  buttons = list('copy', 'print', list(
                    extend = 'collection',
                    buttons = c('csv', 'pdf'),
                    text = 'Download'
                  ), 
                  list(extend = 'colvis', columns = 1:ncol(TABLE_STATS))),
                  pageLength = 10,
                  pagingType = "full",
                  processing = TRUE,
                  deferRender = TRUE,
                  scrollY = 400,
                  scrollX = TRUE,
                  scrollCollapse = FALSE,
                  language = list(search = "_INPUT_", searchPlaceholder = "Regex searching"),
                  search = list(regex = TRUE)
                ),
                extensions =
                  c("Buttons", "ColReorder", "FixedColumns", "Scroller")) %>%
    DT::formatStyle(columns = "Rhat",
                    color = DT::styleInterval(1.1, c("blue", "red"))) %>%
    DT::formatRound(columns = "n_eff", digits = 0) # %>%
  #     DT::formatRound(columns = c("Rhat", "mean", "mcse", "sd", "2.5%", "25%",
  #                                 "50%", "75%", "97.5%"), digits = input$table_digits)
})
output$all_summary_out <- DT::renderDataTable({
  summary_stats()
})

absolutePanel(
  id = "controls_multiparam",
  class = "draggable_controls",
  fixed = TRUE,
  top = 190,
  right = 20,
  width = 200,
  draggable = TRUE,
  shinyjs::hidden(div(
    id = "multiparam_options",
    wellPanel(
      class = "optionswell",
      strongBig("Parameter estimates"),
      hr(class = "hroptions"),
      selectInput(
        "multiparam_options_display",
        label = strongBig("Control"),
        choices = c("Options", "Aesthetics", "Sorting"),
        selected = "Options",
        width = "100%"
      ),
      conditionalPanel(
        condition = "input.multiparam_options_display == 'Options'",
        checkboxInput(
          "param_plot_show_density",
          label = "Kernel density estimates",
          value = FALSE
        ),
        checkboxInput(
          "param_plot_show_ci_line",
          label = "95% interval line",
          value = TRUE
        ),
        radioButtons(
          "param_plot_point_est",
          label = "Point estimate",
          choices = c("Median", "Mean"),
          selected = "Median",
          inline = TRUE
        ),
        hr(class = "hroptions"),
        downloadButton("download_multiparam_plot", "ggplot2", class = "plot-download"),
        downloadButton('save_pdf_multiparam', "pdf", class = "plot-download pdf-download")
      ),
      conditionalPanel(
        condition = "input.multiparam_options_display == 'Aesthetics'",
        withMathJax(),
        checkboxInput(
          "param_plot_color_by_rhat",
          label = "Color point est. by \\(\\hat{R}\\)",
          value = FALSE
        ),
        colourpicker::colourInput(
          "param_plot_fill_color",
          span(style = "font-size: 12px", "Density/CI color"),
          "#590815"
        ),
        colourpicker::colourInput(
          "param_plot_outline_color",
          span(style = "font-size: 12px", "Outline color"),
          "#487575"
        ),
        conditionalPanel(
          condition = "input.param_plot_color_by_rhat == false",
          colourpicker::colourInput(
            "param_plot_est_color",
            span(style = "font-size: 12px", "Point estimate color"),
            base_fill
          )
        ),
        conditionalPanel(
          condition = "input.param_plot_color_by_rhat == true",
          selectInput(
            "param_plot_rhat_palette",
            span(style = "font-size: 12px", "Rhat palette"),
            choices = c("Blues", "Grays", "Greens", "Oranges", "Purples", "Reds"),
            selected = "Blues",
            selectize = TRUE
          )
        )
      ),
      conditionalPanel(
        condition = "input.multiparam_options_display == 'Sorting'",
        radioButtons(
          "param_plot_sort_j",
          label = "Sort parameters in select list by",
          choices = c(Row = TRUE, Column = FALSE),
          selected = TRUE,
          inline = TRUE
        ),
        helpText(
          style = "font-size: 12px;",
          "If applicable, sort with x[1,2] before x[2,1] or vice-versa"
        )
      )
    )
  ))
)

pp_hists_test_statistics_mean <- reactive({
  pp_tests()
  y <- get_y()
  yrep <- get_yrep()
  mean_y <- mean(y)
  mean_yrep <- apply(yrep, 1, mean)
  do.call(".pp_hists_test_statistics", args = list(
    stat_y = mean_y,
    stat_yrep = mean_yrep,
    which = "mean",
    geom = input$pp_hists_test_statistics_type
  ))
})
pp_hists_test_statistics_sd <- reactive({
  pp_tests()
  y <- get_y()
  yrep <- get_yrep()
  sd_y <- sd(y)
  sd_yrep <- apply(yrep, 1, sd)
  do.call(".pp_hists_test_statistics", args = list(
    stat_y = sd_y,
    stat_yrep = sd_yrep,
    which = "sd",
    geom = input$pp_hists_test_statistics_type
  ))
})
pp_hists_test_statistics_min <- reactive({
  pp_tests()
  y <- get_y()
  yrep <- get_yrep()  
  min_y <- min(y)
  min_yrep <- apply(yrep, 1, min)
  do.call(".pp_hists_test_statistics", args = list(
    stat_y = min_y,
    stat_yrep = min_yrep,
    which = "min",
    geom = input$pp_hists_test_statistics_type
  ))
})
pp_hists_test_statistics_max <- reactive({
  pp_tests()
  y <- get_y()
  yrep <- get_yrep()
  max_y <- max(y)
  max_yrep <- apply(yrep, 1, max)
  do.call(".pp_hists_test_statistics", args = list(
    stat_y = max_y,
    stat_yrep = max_yrep,
    which = "max",
    geom = input$pp_hists_test_statistics_type
  ))
})

pp_test_stats <- c("mean", "sd", "min", "max")
for (i in seq_along(pp_test_stats)) {
  local({
    fn <- paste0("pp_hists_test_statistics_", pp_test_stats[i])
    output[[paste0(fn,"_out")]] <- renderPlot({
      x <- suppressMessages(do.call(fn, list()))
      suppress_and_print(x)
    }, bg = "transparent")
  })
}

# pp_hists_test_statistics_custom1 <- reactive({
#   tests()
#   validate(need(input$pp_test_statistics_fun1, message = ""))
#   y <- get_y()
#   yrep <- get_yrep()
#   
#   fun <- input$pp_test_statistics_fun1
#   if (grepl("function", fun)) {
#     f <- eval(parse(text = fun))
#     stat_y <- f(y)
#     stat_yrep <- apply(yrep, 1, FUN = f)
#   } else {
#     stat_y <- do.call(fun, args = list(y))
#     stat_yrep <- apply(yrep, 1, paste(fun))
#   }
#   
#   do.call(".pp_hists_test_statistics", args = list(
#     stat_y = stat_y,
#     stat_yrep = stat_yrep,
#     which = "f",
#     geom = input$pp_hists_test_statistics_type
#   ))
# })
# pp_hists_test_statistics_custom2 <- reactive({
#   tests()
#   if (is.null(input$pp_test_statistics_fun2) | is.na(input$pp_test_statistics_fun2)) {
#     return(last_plot())
#   }
#   y <- get_y()
#   yrep <- get_yrep()
#   stat_y <- do.call(input$pp_test_statistics_fun2, args = list(y))
#   stat_yrep <- apply(yrep, 1, paste(input$pp_test_statistics_fun2))
#   
#   do.call(".pp_hists_test_statistics", args = list(
#     stat_y = stat_y,
#     stat_yrep = stat_yrep,
#     which = paste(input$pp_test_statistics_fun2),
#     geom = input$pp_hists_test_statistics_type
#   ))
# })

shinyjs::hidden(
  div(id = "bivariate_options",
      wellPanel(
        class = "optionswell",
        hr(class = "hroptions"),
        strongBig("Transformation"),
        transform_helpText("x,y"),
        fluidRow(
          column(width = 3, transformation_selectInput("bivariate_transform_x")),
          column(width = 3, transformation_selectInput("bivariate_transform_y")),
          column(
            width = 2,
            actionButton("bivariate_transform_go", label = "Transform", class = "transform-go")
          )
        ),
        hr(class = "hroptions"),
        selectInput(
          "bivariate_options_display",
          label = strongBig("Control"),
          choices = c("Points", "Ellipse", "Lines"),
          selected = "Points",
          width = "50%"
        ),
        conditionalPanel(
          condition = "input.bivariate_options_display == 'Points'",
          fluidRow(
            column(
              width = 3,
              colourpicker::colourInput("bivariate_pt_color", strongMed("Color"), base_fill)
            ),
            column(
              width = 2,
              numericInput(
                "bivariate_pt_size",
                strongMed("Size"),
                value = 3.5,
                min = 0,
                max = 10,
                step = 0.5
              )
            ),
            column(
              width = 2,
              numericInput(
                "bivariate_pt_shape",
                strongMed("Shape"),
                value = 10,
                min = 1,
                max = 10,
                step = 1
              )
            ),
            column(
              width = 2,
              sliderInput(
                "bivariate_pt_alpha",
                strongMed("Opacity"),
                value = alpha_calc_pt(.nIter),
                min = 0,
                max = 1,
                step = 0.01,
                ticks = FALSE
              )
            )
          )),
        conditionalPanel(
          condition = "input.bivariate_options_display == 'Ellipse'",
          fluidRow(
            column(
              width = 2,
              selectizeInput(
                inputId = "bivariate_ellipse_lev",
                label = strongMed("Type"),
                selected = "None",
                choices = list(
                  "None" = "None",
                  "50%" = 0.5,
                  "80%" = 0.8,
                  "95%" = 0.95,
                  "99%" = 0.99
                )
              )
            ),
            column(
              width = 3,
              colourpicker::colourInput(
                "bivariate_ellipse_color",
                strongMed("Color"),
                vline_base_clr
              )
            ),
            column(
              width = 2,
              numericInput(
                "bivariate_ellipse_lwd",
                strongMed("Size"),
                value = 1,
                min = 0,
                max = 5,
                step = 0.5
              )
            ),
            column(
              width = 2,
              numericInput(
                "bivariate_ellipse_lty",
                strongMed("Shape"),
                value = 1,
                min = 1,
                max = 6,
                step = 1
              )
            ),
            column(
              width = 2,
              sliderInput(
                "bivariate_ellipse_alpha",
                strongMed("Opacity"),
                value = 1,
                min = 0,
                max = 1,
                step = 0.01,
                ticks = FALSE
              )
            )
          )
        ),
        conditionalPanel(
          condition = "input.bivariate_options_display == 'Lines'",
          fluidRow(
            column(
              width = 2,
              selectizeInput(
                inputId = "bivariate_lines",
                label = strongMed("Position"),
                choices = c(Hide = "hide", Back = "back", Front = "front"),
                selected = "back"
              )
            ),
            column(
              width = 3,
              colourpicker::colourInput("bivariate_lines_color", strongMed("Color"), "gray")
            ),
            column(
              width = 2,
              sliderInput(
                "bivariate_lines_alpha",
                label = strongMed("Opacity"),
                value = alpha_calc_lines(.nIter),
                min = 0,
                max = 1,
                step = 0.01,
                ticks = FALSE
              )
            )
          )
        )
      )
  )
)

calc_height_param_plot <- reactive({
  if (!isTRUE(input$param_plot_show_density)) {
    "auto"
  } else {
    params <- input$params_to_plot
    params <- .update_params_with_groups(params, PARAM_NAMES)
    LL <- length(params)
    LL <- ifelse(LL < 8, 8, LL)
    if (!is.null(input$param_plot_color_by_rhat)) {
      # delay until input is ready
      if (input$param_plot_color_by_rhat == TRUE) {
        LL <- LL + 1
      }
    }
    round(50 * LL)
  }
})

multiparam_plot <- reactive({
  validate(need(input$param_plot_fill_color, message = "Loading..."))
  if (is.null(input$param_plot_ci_level)) {
    # delay until input is ready
    return()
  }
  
  customize <- !is.null(input$param_plot_show_density)
  do.call(
    ".multiparam_plot",
    args = list(
      samps           = SAMPS_post_warmup,
      params          = input$params_to_plot,
      all_param_names = PARAM_NAMES,
      CI.level        = input$param_plot_ci_level / 100,
      rhat_values     = SUMMARY[, "Rhat"],
      show_density    = ifelse(customize, input$param_plot_show_density, FALSE), # == "yes", FALSE),
      show_ci_line    = ifelse(customize, input$param_plot_show_ci_line, TRUE), # == "yes", TRUE),
      color_by_rhat   = ifelse(customize, input$param_plot_color_by_rhat, FALSE), # == "yes", FALSE),
      rhat_palette    = ifelse(customize, input$param_plot_rhat_palette, "Oranges"),
      point_est       = ifelse(customize, input$param_plot_point_est, "Median"),
      fill_color      = ifelse(customize, input$param_plot_fill_color, "gray35"),
      outline_color   = ifelse(customize, input$param_plot_outline_color, "black"),
      est_color       = ifelse(customize, input$param_plot_est_color, "black")
    )
  )
})


output$multiparam_plot_out <- renderPlot({
  multiparam_plot()
}, height = calc_height_param_plot, bg = "transparent")

# download the plot
output$download_multiparam_plot <- downloadHandler(
  filename = 'shinystan-multiparam-gg.RData',
  content = function(file) {
    shinystan_multiparam_gg <- multiparam_plot()
    save(shinystan_multiparam_gg, file = file)
  }
)
output$save_pdf_multiparam = downloadHandler(
  filename = "shinstan-multiparam.pdf",
  content = function(file) {
    ggsave(file, plot = multiparam_plot(), device = pdf)
  }
)

# Extract the contents of the shiny_stan_object slots and do some additional
# processing

MODEL_NAME <- slot(object, "model_name")
PARAM_NAMES <- slot(object, "param_names")
PARAM_DIMS <- slot(object, "param_dims")
SAMPS_all <- slot(object, "posterior_sample")
SAMPLER_PARAMS <- slot(object, "sampler_params")
N_ITER <- slot(object, "n_iter")
N_CHAIN <- slot(object, "n_chain")
N_WARMUP <- slot(object, "n_warmup")
SAMPS_post_warmup <-
  SAMPS_all[seq(from = N_WARMUP + 1, to = N_ITER), , , drop = FALSE]

MISC <- slot(object, "misc")
MISC_nms <- names(MISC)
STAN_METHOD <- if ("stan_method" %in% MISC_nms)
  MISC$stan_method else "Not Stan"
STAN_ALGORITHM <- if ("stan_algorithm" %in% MISC_nms) 
  MISC$stan_algorithm else "Not Stan"

pp_yrep <- if ("pp_yrep" %in% MISC_nms) 
  MISC[["pp_yrep"]] else NULL
pp_y <- if ("pp_y" %in% MISC_nms) 
  MISC[["pp_y"]] else NULL

SAMPLER_PARAMS_post_warmup <- 
  if (!is.list(SAMPLER_PARAMS) | identical(SAMPLER_PARAMS, list(NA))) 
    FALSE else if (!is.matrix(SAMPLER_PARAMS[[1L]])) 
      FALSE else { 
        lapply(seq_along(SAMPLER_PARAMS), function(i) {
          out <- SAMPLER_PARAMS[[i]]
          out <- if (N_WARMUP == 0) out else out[-(1:N_WARMUP), ]
          rownames(out) <- seq(from = N_WARMUP + 1, to = N_WARMUP + nrow(out))
          out
        })
      }
if (!identical(FALSE, SAMPLER_PARAMS_post_warmup)) {
  .stepsize_pw <-
    .sampler_param_pw(SAMPLER_PARAMS_post_warmup,
                      which = "stepsize__",
                      warmup_val = N_WARMUP)
  .ndivergent_pw <-
    .sampler_param_pw(SAMPLER_PARAMS_post_warmup,
                      which = "divergent__",
                      warmup_val = N_WARMUP)
  .treedepth_pw <-
    .sampler_param_pw(SAMPLER_PARAMS_post_warmup,
                      which = "treedepth__",
                      warmup_val = N_WARMUP)
  .accept_stat_pw <-
    .sampler_param_pw(SAMPLER_PARAMS_post_warmup,
                      which = "accept_stat__",
                      warmup_val = N_WARMUP)
  .energy_pw <-
    .sampler_param_pw(SAMPLER_PARAMS_post_warmup,
                      which = "energy__",
                      warmup_val = N_WARMUP)
}

SUMMARY <- slot(object, "summary")
TABLE_STATS <- SUMMARY
if (!STAN_METHOD == "variational") {
  sel <- colnames(TABLE_STATS) %in% c("Rhat", "n_eff")
  TABLE_STATS <- cbind(TABLE_STATS[, sel], TABLE_STATS[,!sel])
  sel <- NULL
  TABLE_STATS[, "n_eff"] <- round(TABLE_STATS[, "n_eff"])
}

# ppcheck plots from rstanarm
if (isTRUE(MISC$stanreg)) 
  PPC_plots <- MISC$pp_check_plots

multiview_samps <- reactive({
  validate(
    need(input$param, message = FALSE),
    need(!is.null(input$multiview_warmup), message = "Loading...")
  )
  if (!input$multiview_warmup)
    par_samps_post_warmup()
  else
    par_samps_all()
})

dynamic_trace_plot_multiview <- reactive({
  if (input$param == "")
    return()
  stack <- FALSE
  chain <- 0
  do.call(
    ".param_trace_dynamic",
    args = list(
      param_samps = multiview_samps(),
      chain = chain,
      stack = stack,
      warmup_val = N_WARMUP,
      warmup_shade = isTRUE(input$multiview_warmup) && N_WARMUP > 0, 
      x_lab = "Iteration", 
      y_lab = input$param
    )
  )
})
autocorr_plot_multiview <- reactive({
  lags <- min(25, round((N_ITER - N_WARMUP) / 2))
  do.call(
    ".autocorr_single_plot",
     args = list(
       samps = multiview_samps(),
       lags = lags
    )
  )
})
density_plot_multiview <- reactive({
  do.call(
    ".param_dens",
    args = list(
      param       = input$param,
      dat         = multiview_samps(),
      chain       = 0,
      chain_split = FALSE,
      fill_color  = base_fill,
      line_color  = vline_base_clr,
      point_est   = "None",
      CI          = "None",
      x_breaks    = "Some",
      title       = FALSE
    )
  )
})

output$multiview_param_name <-
  renderUI(strong(style = "font-size: 250%; color: #f9dd67;", input$param))
output$multiview_trace_out <-
  dygraphs::renderDygraph(dynamic_trace_plot_multiview())
output$multiview_density_out <-
  renderPlot(density_plot_multiview(), bg = "transparent")
output$multiview_autocorr_out <-
  renderPlot(autocorr_plot_multiview(), bg = "transparent")

# download multiview plot
# output$download_multiview <- downloadHandler(
#   filename = 'shinystan_multiview.RData',
#   content = function(file) {
#     param_name <- input$param
#     shinystan_multiview <- list()
#     shinystan_multiview[[paste0("trace_", param_name)]] <- trace_plot_multiview()
#     shinystan_multiview[[paste0("density", param_name)]] <- density_plot_multiview()
#     shinystan_multiview[[paste0("ac_", param_name)]] <- autocorr_plot_multiview()
#     save(shinystan_multiview, file = file)
#   }
# )

# validate input tests ----------------------------------------------------
pp_tests <- reactive({
  validate(
    need(
      get_y(), 
      message = "Waiting for y \n"
    ),
    need(
      get_yrep(), 
      message = "Waiting for y_rep \n"
    )
  )
})

# y -------------------------------------------------------------------
get_y <- reactive({
  if (!is.null(pp_y)) {
    return(pp_y)
  } else {
    validate(need(input$y_name, message = "Waiting for y"))
    y <- get(input$y_name)
    validate(
      need(
        !isTRUE(length(dim(y)) > 1), 
        message = "Error: y should be a vector"
      ),
      need(
        is.numeric(y), 
        message = "Error: y should be a numeric vector"
      )
    )
    return(y)
  }
})

# y_rep -------------------------------------------------------------------
has_yrep_name <- reactive({
  a <- input$yrep_name  # name selected from model parameters / generated quantities
  b <- input$yrep_name2  # name of object in global environment
  validate(need(a != "" || b != "", message = "Waiting for y_rep"))
  if (a != "" && b != "")
    validate(need(FALSE, message = "y_rep can only be specified once"))
  return(TRUE)
})
get_yrep <- reactive({
  if (!is.null(pp_yrep)) {
    return(pp_yrep)
  } else {
    validate(need(has_yrep_name(), message = "Waiting for y_rep"))
    if (input$yrep_name2 != "") {
      return(get(input$yrep_name2))
    } else {
      yreps <- grep(paste0("^", input$yrep_name, "\\["), PARAM_NAMES)
      out <- SAMPS_post_warmup[, , yreps]
      dd <- dim(out)
      validate(need(
        dd[3] == length(as.vector(get_y())), 
        message = "ncol(y_rep) should equal length(y)"
      ))
      out <- array(out, dim = c(prod(dd[1:2]), dd[3]))
      return(out)
    }
  }
})

# sample_ids_for_hist ------------------------------------------------------
nrow_yrep <- reactive({
  nrow(get_yrep())
})
sample_ids_for_hist <- reactive({
  go <- input$resample_hist_go          
  isolate(sample(nrow_yrep(), 8))
})
# sample_ids_for_dens ------------------------------------------------------
sample_ids_for_dens <- reactive({
  go <- input$resample_dens_go          
  isolate(sample(nrow_yrep(), min(nrow_yrep(), 50)))
})
# sample_id_for_resids ------------------------------------------------------
sample_id_for_resids <- reactive({
  go <- input$resample_resids_go          
  isolate(sample(nrow_yrep(), 1))
})

summary_stats_latex <- reactive({
  params <- unique(.update_params_with_groups(input$tex_params, PARAM_NAMES))
  nParams <- length(params)
  if (nParams == 0)
    params <- PARAM_NAMES
  if (nParams == 1) {
    x <- do.call(".param_summary", args = list(
      param = params, 
      summary = SUMMARY
    ))
  } else {
    x <- do.call(".tex_summary", args = list(
      summary = SUMMARY[params,], 
      cols = input$tex_columns
    ))
  }
  
  pkgs <- input$tex_pkgs
  tab_env <- if ("Longtable" %in% pkgs)
    "longtable" else getOption("xtable.tabular.environment", "tabular")
  caption <- if (nzchar(input$tex_caption)) 
    input$tex_caption else NULL
  xt <- xtable::xtable(x, caption = caption)
  xtable::digits(xt) <- input$tex_digits
  if ("n_eff" %in% colnames(xt))
    xtable::display(xt)[1 + which(colnames(xt) == "n_eff")] <- "d"
  xtable::print.xtable(
    xt,
    booktabs = "Booktabs" %in% pkgs,
    tabular.environment = tab_env,
    include.rownames = FALSE
  )
})

output$summary_stats_latex_out <- renderPrint({
  input$tex_go
  isolate(summary_stats_latex())
})

# stepsize
fluidRow(
  column(
    width = 7,
    help_dynamic,
    dygraphOutput_175px("dynamic_trace_diagnostic_stepsize_out"),
    br(),br(),
    plotOutput("stepsize_vs_lp_out", height = "150px")
  ),
  column(width = 5, plotOutput_400px("stepsize_vs_accept_stat_out"))
)

div(
  style = "text-align: center; margin-top: 100px;",
  shinystan_version(),
  br(),br(),
  a(
    style = "font-size: 16px;",
    strong("Stan Development Team"),
    href = "http://mc-stan.org/team/"
  ),
  br(),
  a(
    style = "font-size: 14px;", 
    "mc-stan.org", 
    href = "http://mc-stan.org/"
  ),
  div(
    actionLink(
      inputId = "shinystan_citation_show",
      label = "Show Citation",
      style = "font-size: 12px;",
      class = "aoptions"
    ),
    div(shinyjs::hidden(
      wellPanel(
        id = "citation_div",
        style = "text-align: left;",
        includeHTML("html/citation.html")
      )
    ))
  ),
  br(),br(),
  h6("Author"),
  helpText(style = "font-size: 12px;", "Jonah Gabry"),
  br(),
  h6(style = "font-size: 12px;", "Contributors"),
  helpText(style = "font-size: 12px;", includeHTML("html/contribs.html")),
  br(),
  h6("Logo"),
  helpText(
    style = "font-size: 12px;",
    a(
      href = "http://mc-stan.org/team/", 
      "Michael Betancourt"
    )
  ),
  br(),
  h6("Shiny"),
  helpText(
    style = "font-size: 12px;",
    "ShinyStan is powered by the",
    a(
      href = "http://shiny.rstudio.com", 
      "Shiny web application framework"
    ),
    "(RStudio)"
  ),
  br(),
  h6("Source code"),
  a(
    style = "color: #190201;",
    href = "http://github.com/stan-dev/shinystan",
    target = "_blank",
    tags$i(class = "fa fa-github fa-3x")
  )
)

# 
# output$ui_multitrace_customize <- renderUI({
#   my_palette <- "Default"
#   my_rect <- "Warmup"
#   my_rect_alpha <- 0.15
#   my_layout <- "Long"
#   
#   absolutePanel(id = "controls_multitrace", 
#                 class = "draggable_controls",
#                 fixed = TRUE,
#                 # top = 175, right = 20, width = 270,
#                 top = 300, right = 20, width = 270,
#                 draggable = TRUE,
#                 shinyjs::hidden(
#                   div(id = "multitrace_options",
#                       wellPanel(
#                         class = "optionswell",
#                         strongBig("Parameter estimates"),
#                         hr(class = "hroptions"),
#                         selectInput("multitrace_options_display", label = strongBig("Control"),
#                                     choices = c("Options", "Aesthetics"),
#                                     selected = "Options", width = "100%"),
#                         conditionalPanel(condition = "input.multitrace_options_display == 'Options'",
#                                          numericInput("multitrace_chain", label = "Chain (0 = all chains)", min = 0, max = object@nChains, step = 1, value = 0),
#                                          radioButtons("multitrace_layout", label = "Layout",
#                                                       choices = c("Long", "Grid"), selected = my_layout, inline = TRUE),
#                                          downloadButton("download_multitrace", "Save as ggplot2 object")
#                         ),
#                         conditionalPanel(condition = "input.multitrace_options_display == 'Aesthetics'",
#                                          selectizeInput("multitrace_palette", "Color palette", choices = c("Default", "Brewer (spectral)", "Rainbow", "Gray"), selected = my_palette),
#                                          # selectInput("multitrace_rect", label = "Shading", choices = c("None", "Samples", "Warmup"), selected = my_rect, size = 2, selectize = FALSE),
#                                          radioButtons("multitrace_rect", label = "Shading", choices = c("None", "Samples", "Warmup"), selected = my_rect, inline = TRUE),
#                                          sliderInput("multitrace_rect_alpha", "Shading opacity", value = my_rect_alpha, min = 0, max = 1, step = 0.01)
#                         )
#                       )
#                   )
#                 )
#   )
# })

shinyjs::hidden(div(
  id = "trivariate_options",
  wellPanel(
    class = "optionswell",
    hr(class = "hroptions"),
    strongBig("Transformation"),
    transform_helpText("x,y,z"),
    fluidRow(
      column(
        width = 3,
        transformation_selectInput("trivariate_transform_x")
      ),
      column(
        width = 3,
        transformation_selectInput("trivariate_transform_y")
      ),
      column(
        width = 3,
        transformation_selectInput("trivariate_transform_z")
      ),
      column(
        width = 2,
        actionButton(
          "trivariate_transform_go",
          label = "Transform",
          class = "transform-go"
        )
      )
    ),
    hr(class = "hroptions"),
    fluidRow(
      column(
        width = 3,
        colourpicker::colourInput("trivariate_pt_color", strongMed("Color"), value = base_fill)
      ),
      column(
        width = 3,
        sliderInput(
          "trivariate_pt_size",
          strongMed("Size"),
          value = 0.5,
          min = 0,
          max = 2,
          step = 0.1,
          ticks = FALSE
        )
      ),
      column(
        width = 2,
        radioButtons(
          "trivariate_grid",
          strongMed("Grid"),
          choices = list(Show = "show", Hide = "hide"),
          selected = "show",
          inline = FALSE
        )
      ),
      column(
        width = 2,
        radioButtons(
          "trivariate_flip",
          strongMed("y-axis"),
          choices = list(Normal = "normal", Flipped = "flip"),
          selected = "normal",
          inline = FALSE
        )
      )
    )
  )
))

# output$hamiltonian_gif <- renderImage({
#   list(src="www/ham-sim-stepsize-ok.gif")
# }, deleteFile = FALSE)

output$user_text_saved <- renderText({
  if (input$save_user_model_info > 0) {
    paste("Saved", format(Sys.time(), "%a %b %d %Y %X"))
  }
})
output$user_code_saved <- renderText({
  if (input$save_user_model_code > 0) {
    paste("Saved", format(Sys.time(), "%a %b %d %Y %X"))
  }
})
observeEvent(input$save_user_model_info, handlerExpr = {
  model_info <- input$user_model_info
  if (model_info == "")
    model_info <- "Use this space to store notes about your model"
  slot(object, "user_model_info") <<- model_info
})
observeEvent(input$save_user_model_code, handlerExpr = {
  model_code <- input$user_model_code
  if (model_code == "")
    model_code <- "Use this space to store your model code" 
  slot(object, "model_code") <<- model_code
})

source("global_utils.R", local = TRUE)
source("ui_utils.R", local = TRUE)

# Begin shinyUI -----------------------------------------------------------
# _________________________________________________________________________
tagList(
  tags$noscript(
    style = "color: orange; font-size: 30px; text-align: center;",
    "Please enable JavaScript to use ShinyStan."
  ), 
  shinyjs::useShinyjs(),
  includeCSS("css/ShinyStan.css"),
  
  navbarPage(
    save_and_close_button(), # title = NULL
    id = "nav",
    position = "fixed-top",
    collapsible = TRUE,
    theme = shinythemes::shinytheme("flatly"),
    windowTitle = "ShinyStan",
    
    
    #### HOME ####
    tabPanel(
      title = strong(style = "color: #B2011D;", "ShinyStan"),
      value = "home",
      source_ui("PAGE_home.R")
    ),
    
    #### DIAGNOSE ####
    tabPanel(
      title = "Diagnose",
      icon = icon("medkit"),
      source_ui("PAGE_diagnose.R")
    ),
    
    #### ESTIMATE ####
    tabPanel(
      title = "Estimate",
      icon = icon("stats", lib = "glyphicon"),
      withMathJax(),
      source_ui("PAGE_estimate.R")
    ),
    
    #### EXPLORE ####
    tabPanel(
      title = "Explore",
      icon = icon("eye-open", lib = "glyphicon"),
      source_ui("PAGE_explore.R")
    ),
             
    #### More ####
    source_ui("PAGE_more_menu.R")
    
  ) # End navbarPage
) # End tagList

# End shinyUI -------------------------------------------------------------
# -------------------------------------------------------------------------

make_param_list_with_groups_sort <- reactive({
  validate(need(!is.null(input$param_plot_sort_j), message = "Loading..."))
  sort_j <- input$param_plot_sort_j
  choices <- list()
  param_groups <- names(PARAM_DIMS)
  ll <- length(PARAM_DIMS)
  LL <- sapply(seq_len(ll), function(i) 
    length(PARAM_DIMS[[i]]))

  choices[seq_len(ll)] <- ""
  names(choices) <- param_groups
  for(i in seq_len(ll)) {
    if (LL[i] == 0) {
      choices[[i]] <- list(param_groups[i])
    } else {
      group <- param_groups[i]
      temp <- paste0("^",group,"\\[")
      ch <- PARAM_NAMES[grep(temp, PARAM_NAMES)]
      # the next line avoids parameters whose names include the group name of a
      # different group of parameters being included in the latter group, e.g.
      # if we have b_bias[1], b_bias[2], bias[1], bias[2] then we want to avoid
      # bias[1] and bias[2] being included in the b_bias group
      ch <- ch[which(substr(ch, 1, nchar(group)) == group)]

      if (sort_j == TRUE & (LL[i] > 1)) {
        # change sorting so e.g. "beta[1,1] beta[1,2] beta[2,1] beta[2,2]"
        # instead of "beta[1,1] beta[2,1] beta[1,2] beta[2,2]"
        ch <- gtools::mixedsort(ch)
      }
      ch_out <- c(paste0(group,"_as_shinystan_group"), ch)
      names(ch_out) <- c(paste("ALL", group), ch)
      choices[[i]] <- ch_out
    }
  }

  choices
})

vb_check <- function() {
  validate(
    need(
      STAN_METHOD != "variational", 
      message = "Not available for variational inference"
    )
  )
}

n_eff_plot <- reactive({
  vb_check()
  dat <- SUMMARY[, "n_eff"]
  N <- prod(dim(SAMPS_post_warmup)[1:2])
  dat <- data.frame(parameter = names(dat), x = dat / N)
  do.call(".rhat_neff_mcse_hist",
          args = list(
            dat = dat,
            N = nrow(SAMPS_post_warmup),
            which = "n_eff"
          ))
})
rhat_plot <- reactive({
  vb_check()
  dat <- SUMMARY[, "Rhat"]
  dat <- data.frame(parameter = names(dat), x = dat)
  do.call(".rhat_neff_mcse_hist", args = list(dat = dat, which = "rhat"))
})
mcse_over_sd_plot <- reactive({
  vb_check()
  dat <- SUMMARY[, c("se_mean", "sd")]
  dat <- dat[, 1] / dat[, 2]
  dat <- data.frame(parameter = names(dat), x = dat)
  do.call(".rhat_neff_mcse_hist", args = list(dat = dat, which = "mcse"))
})
n_eff_warnings <- reactive({
  vb_check()
  paste(
    .n_eff_warnings(
      SUMMARY,
      threshold = input$n_eff_threshold,
      N_total = length(SAMPS_post_warmup[, , 1L])
    ),
    collapse = "\n"
  )
})
rhat_warnings <- reactive({
  vb_check()
  paste(.rhat_warnings(SUMMARY, threshold = input$rhat_threshold), collapse = "\n")
})
mcse_over_sd_warnings <- reactive({
  vb_check()
  paste(.mcse_over_sd_warnings(SUMMARY, threshold = input$mcse_threshold), collapse = "\n")
})

output$n_eff_warnings_title <- renderText({
  paste0(
    "The following parameters have an effective sample size less than ", 
    input$n_eff_threshold,
    "% of the total sample size: "
  )
})
output$rhat_warnings_title <- renderText({
  paste0(
    "The following parameters have an Rhat value above ", 
    input$rhat_threshold,
    ": "
  )
})
output$mcse_over_sd_warnings_title <- renderText({
  paste0(
    "The following parameters have a Monte Carlo standard error greater than ", 
    input$mcse_threshold,
    "% of the posterior standard deviation:"
  )
})

rhat_neff_mcse <- c("rhat", "n_eff", "mcse_over_sd")
for (i in seq_along(rhat_neff_mcse)) {
  local({
    fn <- paste0(rhat_neff_mcse[i], "_plot")
    output[[paste0(fn, "_out")]] <- renderPlot({
      x <- do.call(fn, list())
      suppress_and_print(x)
    }, bg = "transparent")
  })
  local({
    fn <- paste0(rhat_neff_mcse[i], "_warnings")
    output[[fn]] <- renderText(do.call(fn, list()))
  })
}

output$ui_pp_get_y <- renderUI({
  if (is.null(pp_y)) {
    div(
      h4(
        withMathJax("Select \\(\\mathbf{y}\\) (vector of observations)")
      ),
      selectizeInput(
        "y_name",
        label = "Object from global environment",
        choices = c("", objects(envir = .GlobalEnv)),
        options = list(placeholder = "Select an object"),
        width = "50%"
      )
    )
  } else {
    helpText("All set: y found in shinystan object.")
  }
})

output$ui_pp_get_yrep <- renderUI({
  if (is.null(pp_yrep)) {
    choices <- PARAM_NAMES
    choices <- strsplit(choices, split = "[", fixed = TRUE)
    choices <- lapply(choices, function(i) return(i[1]))
    choices <- unique(unlist(choices))
    div(h4(
      withMathJax(
        "Select \\(\\mathbf{y^{rep}}\\) (posterior predictive replications)"
      )
    ),
    flowLayout(
      selectizeInput(
        "yrep_name",
        label = "Parameter/generated quantity from model",
        choices = c("", choices),
        options = list(placeholder = "Select a parameter name")
      ),
      selectizeInput(
        "yrep_name2",
        label = "Or object from global environment",
        choices = c("", objects(envir = .GlobalEnv)),
        options = list(placeholder = "Select an object")
      )
    ))
  } else {
    helpText("All set: yrep found in shinystan object. Select a plot to view.")
  }
})

# From: https://gist.github.com/jcheng5/6141ea7066e62cafb31c
# Returns a reactive that debounces the given expression by the given time in
# milliseconds.
#
# This is not a true debounce in that it will not prevent \code{expr} from being
# called many times (in fact it may be called more times than usual), but
# rather, the reactive invalidation signal that is produced by expr is debounced
# instead. This means that this function should be used when \code{expr} is
# cheap but the things it will trigger (outputs and reactives that use
# \code{expr}) are expensive.
debounce <- function(expr, 
                     millis, 
                     env = parent.frame(), 
                     quoted = FALSE,
                     domain = getDefaultReactiveDomain()) {
  
  force(millis)
  
  f <- exprToFunction(expr, env, quoted)
  label <- sprintf("debounce(%s)", paste(deparse(body(f)), collapse = "\n"))
  
  v <- reactiveValues(
    trigger = NULL,
    when = NULL # the deadline for the timer to fire; NULL if not scheduled
  )
  
  # Responsible for tracking when f() changes.
  observeEvent(f(), {
    # The value changed. Start or reset the timer.
    v$when <- Sys.time() + millis / 1000
  }, ignoreNULL = FALSE)
  
  # This observer is the timer. It rests until v$when elapses, then touches
  # v$trigger.
  observe({
    if (is.null(v$when))
      return()
    
    now <- Sys.time()
    if (now >= v$when) {
      v$trigger <- runif(1)
      v$when <- NULL
    } else {
      invalidateLater((v$when - now) * 1000, domain)
    }
  })
  
  # This is the actual reactive that is returned to the user. It returns the
  # value of f(), but only invalidates/updates when v$trigger is touched.
  eventReactive(v$trigger, {
    f()
  }, ignoreNULL = FALSE)
}

# treedepth
div(
  class = "diagnostics-navlist-tabpanel",
  fluidRow(
    column(
      width = 7,
      help_dynamic,
      dygraphOutput_175px("dynamic_trace_diagnostic_treedepth_out"),
      br(), br(),
      plotOutput("treedepth_vs_lp_out", height = "150px")
    ),
    column(width = 5, plotOutput_400px("treedepth_vs_accept_stat_out"))
  ),
  splitLayout(
    plotOutput("treedepth_ndivergent_hist_out", height = "125px"),
    plotOutput("treedepth_ndivergent0_hist_out", height = "125px"),
    plotOutput("treedepth_ndivergent1_hist_out", height = "125px")
  ),
  br()
)

# Energy

# fluidRow(
#   column(
#     width = 7,
#     help_dynamic,
#     dygraphOutput_175px("dynamic_trace_diagnostic_ndivergent_out"),
#     br(), br(),
#     plotOutput("ndivergent_vs_lp_out", height = "150px")
#   ),
#   column(width = 5, plotOutput_400px("ndivergent_vs_accept_stat_out"))
# )

plotOutput("energy_hist_out")

navbarMenu(
  title = "More",
  
  #### model code ####
  tabPanel(
    title = "Model Code",
    source_ui("model_code.R")
  ),
  
  #### notepad ####
  tabPanel(
    title = "Notepad",
    source_ui("notepad.R")
  ),
  
  #### about ####
  tabPanel(
    title = "About",
    logo_and_name(),
    div(
      style = "margin-top: 75px;",
      source_ui("about.R")
    )
  ),
  
  #### glossary ####
  tabPanel(
    title = "Glossary",
    div(
      style = "background-color: white;",
      h1(style = "text-align: center;", "Glossary"),
      source_ui("glossary.R"),
      hr(),
      stan_manual()
    )
  ),
  
  #### help ####
  tabPanel(
    title = "Help",
    h1(style = "text-align: center;", "Help"),
    source_ui("help.R")
  )
)

# shinystan is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 3 of the License, or (at your option) any later
# version.
# 
# shinystan is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# this program; if not, see <http://www.gnu.org/licenses/>.

.sso_env <- new.env(parent=emptyenv()) 

.onAttach <- function(...) {
  ver <- utils::packageVersion("shinystan")
  msg <- paste0("\nThis is shinystan version ", ver,"\n")
  packageStartupMessage(msg)
}

.onLoad <- function(libname, pkgname) {
  op <- options()
  op.shinystan <- list(
    shinystan.rstudio = FALSE
  )
  set_ops <- !(names(op.shinystan) %in% names(op))
  if (any(set_ops)) options(op.shinystan[set_ops])
  invisible()
}

# shinystan is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 3 of the License, or (at your option) any later
# version.
# 
# shinystan is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# this program; if not, see <http://www.gnu.org/licenses/>.


#' View or change metadata associated with a \code{shinystan} object
#' 
#' @name shinystan-metadata
#' @template args-sso
#' 
#' @template seealso-as.shinystan
#' @template seealso-drop_parameters
#' @template seealso-generate_quantity
#' 
#' @examples 
#' # use eight_schools example object
#' sso <- eight_schools
#' 
NULL

# sso_info ----------------------------------------------------------------
#' @rdname shinystan-metadata
#' @export
#' 
#' @return \code{sso_info} prints basic metadata including number of parameters, 
#'   chains, iterations, warmup iterations, etc. It does not return anything.
#' 
#' @examples 
#' ################
#' ### sso_info ###
#' ################
#' 
#' sso_info(sso)
#'
sso_info <- function(sso) {
  sso_check(sso)
  sso_name <- deparse(substitute(sso))
  has_notes <-
    sso@user_model_info != "Use this space to store notes about your model"
  has_code <-
    sso@model_code != "Use this space to store your model code"
  
  cat(
    sso_name,
    "---------------------",
    paste("Model name:", sso@model_name),
    paste("Parameters:", length(sso@param_names)),
    paste("Parameter groups:", length(names(sso@param_dims))),
    paste("Chains:", sso@n_chain),
    paste("Iterations:", sso@n_iter),
    paste("Warmup:", sso@n_warmup),
    paste("Has model code:", has_code),
    paste("Has user notes:", has_notes),
    sep = "\n"
  )
}



# model_code --------------------------------------------------------------
#' @rdname shinystan-metadata
#' @export
#' @param code A string, containing model code to be added, that can be
#'   used as an argument to \code{\link{cat}}. See \strong{Examples}.
#'   
#' @return \code{model_code} returns or replaces model code stored in a 
#'   \code{shinystan} object. If \code{code} is \code{NULL} then any existing 
#'   model code stored in \code{sso} is returned as a character string. If 
#'   \code{code} is specified then an updated \code{shinystan} object is 
#'   returned with \code{code} added. For \code{shinystan} objects created from 
#'   stanfit (\pkg{rstan}) and stanreg (\pkg{rstanarm}) objects, model code is 
#'   automatically taken from that object and does not need to be added 
#'   manually. From within the 'ShinyStan' interface model code can be viewed on
#'   the \strong{Model Code} page.
#'
#' @examples
#' ##################
#' ### model_code ###
#' ##################
#' 
#' # view model code in example shinystan object 'eight_schools'
#' cat(model_code(sso))
#' 
#' # change the model code in sso 
#' # some jags style code
#' my_code <- "
#'  model {
#'    for (i in 1:length(Y)) {
#'      Y[i] ~ dpois(lambda[i])
#'      log(lambda[i]) <- inprod(X[i,], theta[])
#'    }
#'    for (j in 1:J) {
#'      theta[j] ~ dt(0.0, 1.0, 1.0)
#'    }
#'  }
#' "
#' sso <- model_code(sso, my_code)
#' cat(model_code(sso))
#'
model_code <- function(sso, code = NULL) {
  sso_check(sso)
  validate_model_code(code)
  
  if (is.null(code))
    return(slot(sso, "model_code"))
  
  slot(sso, "model_code") <- code
  message(
    paste0(
      "Successfully added code.",
      "\nYou can view the code in the",
      "ShinyStan GUI on the 'Model Code' page."
    )
  )
  sso
}

validate_model_code <- function(code) {
  if (is.null(code) || is.character(code)) {
    invisible(TRUE)
  } else {
    stop("Model code should be NULL or a string", call. = FALSE)
  }
}



# notes -------------------------------------------------------------------
#' @rdname shinystan-metadata
#' @export
#' @param note A string containing a note to add to any existing notes
#'   or replace existing notes, depending on the value of \code{replace}.
#' @param replace If \code{TRUE} the existing notes are overwritten by 
#'   \code{note} if \code{note} is specified. If \code{FALSE} (the default) 
#'   if \code{note} is specified then its content is appended to the existing
#'   notes.
#'   
#' @return \code{notes} returns, amends, or replaces notes stored in a
#'   \code{shinystan} object. If \code{note} is \code{NULL} then any existing
#'   notes stored in \code{sso} are returned as a character string. If
#'   \code{note} is specified then an updated \code{shinystan} object is
#'   returned with either \code{note} added to the previous notes (if
#'   \code{replace=FALSE}) or overwritten by \code{note} (if \code{replace =
#'   TRUE}). From within the 'ShinyStan' interface, notes are viewable on the
#'   \strong{Notepad} page.
#'   
#' @examples 
#' #############
#' ### notes ###
#' #############
#' 
#' # view existing notes
#' notes(sso)
#' 
#' # add a note to the existing notes
#' sso <- notes(sso, "New note")
#' notes(sso)
#' cat(notes(sso))
#' 
#' # replace existing notes
#' sso <- notes(sso, "replacement note", replace = TRUE)
#' notes(sso)
#'  
notes <- function(sso, note = NULL, replace = FALSE) {
  sso_check(sso)
  if (is.null(note))
    return(slot(sso, "user_model_info"))
  
  if (!is.character(note) || !isTRUE(length(note) == 1))
    stop("'note' should be a single string")
  
  slot(sso, "user_model_info") <- if (replace)
    note else c(slot(sso, "user_model_info"), paste0("\n\n", note))
  
  message(
    paste(
      "Successfully added note.",
      "\nYou can view the notes in the",
      "ShinyStan GUI on the 'Notepad' page."
    )
  )
  sso
}



# model_name (renaming) -----------------------------------------------------#' 
#' @rdname shinystan-metadata
#' @export
#' @param name A string giving the new model name to use.
#'   
#' @return \code{model_name} returns or replaces the model name associated with 
#'   a \code{shinystan} object. If \code{name} is \code{NULL} then the current
#'   model name is returned. If \code{name} is specified then \code{sso} is
#'   returned with an updated model name.
#' 
#' @examples
#' ##################
#' ### model_name ###
#' ##################
#' 
#' # view model name
#' model_name(sso)
#' 
#' # change model name
#' sso <- model_name(sso, "some other name")
#' identical(model_name(sso), "some other name")
#' 
model_name <- function(sso, name = NULL) {
  sso_check(sso)
  if (is.null(name))
    return(slot(sso, "model_name"))
  
  if (!is.character(name) || !isTRUE(length(name) == 1))
    stop("'name' should be a single string")
  
  slot(sso, "model_name") <- name
  message(paste("Successfully changed model name to", name))
  sso
}


# nocov start
#' rename_model (deprecated)
#' 
#' This function is deprecated and will be removed in a future release. Please 
#' use the \code{\link{model_name}} function instead.
#' 
#' @export
#' @keywords internal
#' @param sso,new_model_name Use the \code{\link{model_name}} function instead.
#' 
rename_model <- function(sso, new_model_name) {
  .Deprecated("model_name()")
  model_name(sso, new_model_name)
}
# nocov end

library(shinystan)
context("Using shinystan objects")

sso <- eight_schools
not_sso <- sso@model_name
not_sso_msg <- "specify a shinystan object"

# load 'old_sso', a shinystan object created by previous shinystan version
load("old_sso_for_tests.rda")
old_sso_msg <- "use the 'update_sso' function to update your object"


# launch_shinystan --------------------------------------------------------
test_that("launch_shinystan throws appropriate errors", {
  expect_error(launch_shinystan(sso@summary), "object not compatible")
  expect_error(launch_shinystan(old_sso), old_sso_msg)
})


# model_name --------------------------------------------------------------
test_that("model_name works", {
  expect_error(model_name(old_sso), old_sso_msg)
  expect_error(model_name(not_sso), not_sso_msg)
  sso2 <- model_name(sso, "test_rename")
  expect_identical(model_name(sso2), "test_rename")
  expect_error(model_name(sso, 1234), "should be a single string")
  expect_error(model_name(sso, c("a", "b")), "should be a single string")
})

# model_code --------------------------------------------------------------
test_that("model_code works", {
  expect_error(model_code(old_sso), old_sso_msg)
  expect_error(model_code(not_sso), not_sso_msg)
  sso2 <- model_code(sso, "test_code")
  expect_identical(model_code(sso2), "test_code")
  expect_identical(model_code(sso2), slot(sso2, "model_code"))
  expect_error(model_code(sso, 1234), "should be NULL or a string")
})

# notes -------------------------------------------------------------------
test_that("notes works", {
  expect_error(notes(old_sso), old_sso_msg)
  expect_error(notes(not_sso), not_sso_msg)
  sso2 <- notes(sso, "test_notes_replace", replace = TRUE)
  expect_identical(slot(sso2, "user_model_info"), "test_notes_replace")
  sso2 <- notes(sso2, "test_notes_keep", replace = FALSE)
  expect_identical(slot(sso2, "user_model_info"), notes(sso2))
  expect_error(notes(sso, 1234), "should be a single string")
  expect_error(notes(sso, c("a", "b")), "should be a single string")
})


# retrieve ----------------------------------------------------------------
test_that("retrieve works", {
  expect_error(retrieve(old_sso), old_sso_msg)
  expect_error(retrieve(not_sso), not_sso_msg)
  expect_error(retrieve(not_sso, what = "mean"), not_sso_msg)
  
  stats1 <- c("median", "mean", "rhat", "ess", "sd", "mcse")
  whats <- c(stats1, "quantiles", "divergent", "treedepth", "stepsize", "accept_stat")
  for (what in whats)
    expect_equal_to_reference(
      retrieve(sso, what), 
      file = file.path("retrieve_sso_answers", paste0("test_answer_", what, ".rds"))
    )
  
  for (what in stats1)
    expect_equal(names(retrieve(sso, what, pars = c("mu", "tau"))), c("mu", "tau"))
  
  expect_equal(rownames(retrieve(sso, what = "quantiles", pars = c("mu", "theta[2]"))), 
               c("mu", "theta[2]"))
})


# generate_quantity -------------------------------------------------------
test_that("generate_quantity works", {
  expect_error(generate_quantity(old_sso), old_sso_msg)
  expect_error(generate_quantity(not_sso), not_sso_msg)
  
  sso2 <- generate_quantity(sso, fun = function(x) x^2,
                           param1 = "tau", new_name = "tau_sq")
  expect_equivalent(sso2@posterior_sample[,, "tau_sq", drop=FALSE], 
                    sso@posterior_sample[,, "tau", drop=FALSE]^2)
  
  sso2 <- generate_quantity(sso, fun = "-",
                           param1 = "theta[1]", param2 = "theta[2]",
                           new_name = "theta1minus2")
  expect_equivalent(sso2@posterior_sample[,, "theta1minus2", drop=FALSE], 
                    sso@posterior_sample[,, "theta[1]", drop=FALSE] - 
                      sso@posterior_sample[,, "theta[2]", drop=FALSE])
  
  # test when sso only has one chain
  sso3 <- as.shinystan(list(cbind(b1 = rnorm(100), b2 = rnorm(100), s = rexp(100))))
  sso3 <- generate_quantity(sso3, param1 = "b1", param2 = "b2", 
                            fun = "+", new_name = "b1plusb2")
  expect_equivalent(sso3@posterior_sample[,, "b1plusb2", drop=FALSE], 
                    sso3@posterior_sample[,, "b1", drop=FALSE] + 
                      sso3@posterior_sample[,, "b2", drop=FALSE])
})


# drop_parameters ---------------------------------------------------------
test_that("drop_parameters throws correct warnings/errors", {
  expect_error(drop_parameters(old_sso, pars = "mu"), old_sso_msg)
  expect_error(drop_parameters(not_sso, pars = "mu"), not_sso_msg)
  expect_error(drop_parameters(sso, pars = "log-posterior"), 
               "log-posterior can't be dropped")
  expect_error(drop_parameters(sso, pars = c("theta[1]", "mu")), 
               regexp = "individual elements")
  expect_error(drop_parameters(sso, pars = "omega"), regexp = "No matches")
  expect_warning(drop_parameters(sso, pars = c("mu", "omega")), 
                 regexp = "not found and ignored: omega")
})
test_that("drop_parameters works", {
  pn <- sso@param_names
  pd <- sso@param_dims
  s <- sso@summary
  samp <- sso@posterior_sample
  
  sso2 <- drop_parameters(sso, pars = "mu")
  expect_identical(sso2@param_names, pn[pn != "mu"])
  expect_identical(sso2@param_dims, pd[names(pd) != "mu"])
  expect_identical(sso2@summary, s[rownames(s) != "mu", ])
  expect_identical(sso2@posterior_sample, samp[,, dimnames(samp)[[3]] != "mu"])
  
  sso2 <- drop_parameters(sso, pars = "theta")
  expect_identical(sso2@param_names, grep("theta", pn, value = TRUE, invert = TRUE))
  expect_identical(sso2@param_dims, pd[names(pd) != "theta"])
  tmp <- s[grep("theta", rownames(s), value = TRUE, invert = TRUE), ]
  expect_identical(sso2@summary, tmp)
  tmp <- samp[,, grep("theta", dimnames(samp)[[3]], value = TRUE, invert = TRUE)]
  expect_identical(sso2@posterior_sample, tmp)
  
  sso2 <- drop_parameters(sso, pars = c("theta", "tau"))
  tmp <- grep("theta|tau", pn, value = TRUE, invert = TRUE)
  expect_identical(sso2@param_names, tmp)
  tmp <- pd[grep("theta|tau", names(pd), value = TRUE, invert = TRUE)]
  expect_identical(sso2@param_dims, tmp)
  tmp <- s[grep("theta|tau", rownames(s), value = TRUE, invert = TRUE), ]
  expect_identical(sso2@summary, tmp)
  tmp <- samp[,, grep("theta|tau", dimnames(samp)[[3]], value = TRUE, invert = TRUE)]
  expect_identical(sso2@posterior_sample, tmp)
})


# sso_info ----------------------------------------------------------------
test_that("sso_info error checking", {
  expect_error(sso_info(old_sso), old_sso_msg)
  expect_error(sso_info(sso@posterior_sample), "specify a shinystan object")
})

test_that("sso_info prints output", {
  expect_output(sso_info(sso), "sso")
  expect_output(sso_info(sso), "Model name: Demo")
  expect_output(sso_info(sso), "Parameters: 11")
  expect_output(sso_info(sso), "Chains: 4")
  expect_output(sso_info(sso), "Has model code: TRUE")
  expect_output(sso_info(sso), "Has user notes: FALSE")
})

sidebarLayout(
  position = "right",
  sidebarPanel(
    width = 3,
    class = "optionswell",
    strongBig("Definitions"),
    hr(class = "hroptions"),
    div(actionLink("open_quick_mcse", "mcse (se_mean)")),
    div(actionLink("open_quick_neff", "n_eff (ESS)")),
    div(actionLink("open_quick_rhat", "Rhat")),
    br(),
    strongBig("Warnings"),
    hr(class = "hroptions"),
    withMathJax(),
    sliderInput(
      "n_eff_threshold",
      "\\(n_{eff} / N\\) warning threshold",
      ticks = FALSE,
      value = 10,
      min = 0,
      max = 100,
      step = 5,
      post = "%"
    ),
    sliderInput(
      "mcse_threshold",
      "\\(\\text{se}_{mean} / sd\\) warning threshold",
      ticks = FALSE,
      value = 10,
      min = 0,
      max = 100,
      step = 5,
      post = "%"
    ),
    sliderInput(
      "rhat_threshold",
      "\\(\\hat{R}\\) warning threshold",
      ticks = FALSE,
      value = 1.1,
      min = 1,
      max = 1.2,
      step = 0.01
    )
  ),
  mainPanel(
    width = 9,
    withMathJax(),
    br(),
    splitLayout(
      h4("\\(n_{eff} / N\\)", align = "center"),
      h4("\\(mcse / sd\\)", align = "center"),
      h4("\\(\\hat{R}\\)", align = "center")
    ),
    splitLayout(
      plotOutput("n_eff_plot_out", height = "200px"),
      plotOutput("mcse_over_sd_plot_out", height = "200px"),
      plotOutput("rhat_plot_out", height = "200px")
    ),
    hr(),
    div(
      fluidRow(
        column(width = 4, strong(textOutput(
          "n_eff_warnings_title"
        ))),
        column(width = 4, strong(
          textOutput("mcse_over_sd_warnings_title")
        )),
        column(width = 4, strong(textOutput(
          "rhat_warnings_title"
        )))
      ),
      tags$style(
        type = "text/css",
        "#n_eff_warnings_title, #rhat_warnings_title, #mcse_over_sd_warnings_title {font-size: 13px;}"
      ),
      br(),
      fluidRow(
        column(width = 4, div(style = "color: #006dcc;", textOutput("n_eff_warnings"))),
        column(width = 4, div(
          style = "color: #006dcc;", textOutput("mcse_over_sd_warnings")
        )),
        column(width = 4, div(style = "color: #006dcc;", textOutput("rhat_warnings")))
      ),
      tags$style(
        type = "text/css",
        "#n_eff_warnings, #rhat_warnings, #mcse_over_sd_warnings {font-size: 12px;}"
      )
    )
  )
)

div(
  withMathJax(),
  h3(style = "color: #337ab7;", "What is posterior predictive checking?"),
  p(
    strong("The idea behind posterior predictive checking is simple:")
  ),
  p(em("If our model is a good fit then we should be able to use it to generate")),
  p(em("data that looks a lot like the data we observed.")),
  br(),
  p(
    "To generate this 'replicated' data we use the",
    em("posterior predictive distribution")
  ),
  span(
    style = "color: #337ab7; font-face: bold;",
    withMathJax(
      "$$ p(y^{rep} | y )  = \\int p(y^{rep} | \\theta) p(\\theta | y ) d \\theta,$$"
    )
  ),
  p(
    "where \\(y\\) is the observed data and \\(\\theta\\) the parameters in our model."
  ),
  br(),
  p(
    "For each draw of \\(\\theta\\) from the posterior \\(p(\\theta | y) \\)
    we simulate data \\(y^{rep}\\) from the posterior predictive distribution \\(p(y^{rep} | y) \\)."
  ),
  br(),
  p(
    "Using the simulations of \\(y^{rep}\\) we can make various
    graphical displays comparing our observed data to the replications."
  ),
  hr(),
  helpText(
    "For a more thorough discussion of posterior predictive checking see Chapter 6 of",
    a("BDA3.", href = "http://www.stat.columbia.edu/~gelman/book/")
  )
)

# shinystan is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 3 of the License, or (at your option) any later
# version.
# 
# shinystan is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# this program; if not, see <http://www.gnu.org/licenses/>.


#' Add new quantity to shinystan object
#' 
#' Add to shinystan object a new parameter as a function of one or two existing
#' parameters.
#' 
#' @export
#' @template args-sso
#' @param fun Function to call, i.e. \code{function(param1)} or 
#'   \code{function(param1,param2)}. See Examples, below.
#' @param param1 Name of first parameter as character string.
#' @param param2 Optional. Name of second parameter as character string.
#' @param new_name Name for the new parameter as character string.
#'   
#' @return sso, updated. See Examples.
#' 
#' @template seealso-drop_parameters
#'
#' @examples
#' # Using example shinystan object 'eight_schools'
#' sso <- eight_schools
#' sso <- generate_quantity(sso, fun = function(x) x^2, 
#'                          param1 = "tau", new_name = "tau_sq")
#' sso <- generate_quantity(sso, fun = "-", 
#'                          param1 = "theta[1]", param2 = "theta[2]", 
#'                          new_name = "theta1minus2")
#'                          
generate_quantity <- function(sso, param1, param2, fun, new_name) {
  sso_check(sso)
  if (isTRUE(new_name %in% slot(sso, "param_names")))
    stop(paste("There is already a parameter named", new_name))
  
  message("\nThis might take a moment for large shinystan objects...")
  
  two_params <- !missing(param2)
  posterior <- slot(sso, "posterior_sample")
  dims <- dim(posterior)
  ndim <- length(dims)
  if (ndim == 3) {
    # i.e. multiple chains
    x_samp <- posterior[, , param1]
    if (two_params)
      y_samp <- posterior[, , param2]
  }
  
  arglist <- if (two_params)
    list(x_samp, y_samp) else list(x_samp)
  temp <- do.call(fun, args = arglist)
  
  new_dim <- dims
  new_dim[[ndim]] <- new_dim[[ndim]] + 1
  new_dim_names <- dimnames(posterior)
  new_dim_names[[ndim]] <- c(new_dim_names[[ndim]], new_name)
  posterior <-
    array(data = c(posterior, temp),
          dim = new_dim,
          dimnames = new_dim_names)
  
  param_dims_new <- slot(sso, "param_dims")
  param_dims_new[[new_name]] <- numeric(0)
  sso_new <- as.shinystan(
    posterior,
    model_name = slot(sso, "model_name"),
    warmup = slot(sso, "n_warmup"),
    param_dims = param_dims_new
  )
  slot(sso_new, "summary") <-
    shinystan_monitor(posterior, warmup = slot(sso, "n_warmup"))
  
  slot_names <- c("sampler_params", "model_code", "user_model_info", "misc")
  for (sn in slot_names)
    slot(sso_new, sn) <- slot(sso, sn)
  
  sso_new
}

# # This file is part of shinyStan
# # Copyright (C) 2015 Jonah Sol Gabry & Stan Development Team
# #
# # shinyStan is free software; you can redistribute it and/or modify it under the
# # terms of the GNU General Public License as published by the Free Software
# # Foundation; either version 3 of the License, or (at your option) any later
# # version.
# # 
# # shinyStan is distributed in the hope that it will be useful, but WITHOUT ANY
# # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# # A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# # 
# # You should have received a copy of the GNU General Public License along with
# # this program; if not, see <http://www.gnu.org/licenses/>.
# 
# tooltip_ids <- c(
#   "download_multiview", 
#   "dynamic_trace_stack", 
#   "download_all_summary", 
#   "tex_options", 
#   "dens_transform_x", "hist_transform_x", "bivariate_transform_x", "trivariate_transform_x", 
#   "bivariate_transform_y", "trivariate_transform_y", 
#   "trivariate_transform_z"
#   )
# 
# tooltip_msgs <- c(
#   "Will be a list with three elements corresponding the the ggplot2 objects for the three plots.", 
#   "If 'Stacked' is selected, the chains will be stacked on top of one another rather than drawing them independently. The first series specified in the input data will wind up on top of the chart and the last will be on bottom. Note that the y-axis values no longer correspond to the true values when this option is enabled.",
#   "Save as data.frame (.RData)", 
#   "Print latex table to R console", 
#   rep("A function of x, e.g. log(x), sqrt(x), x^2, 1/x, etc. Should be a valid R expression.", 4),
#   rep("A function of y, e.g. log(y), sqrt(y), y^2, 1/y, etc. Should be a valid R expression.", 2),
#   "A function of z, e.g. log(z), sqrt(z), z^2, 1/z, etc. Should be a valid R expression."
#   )
# tooltip_placements <- c(rep("right", 4), rep("top", 7))

# shinystan is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 3 of the License, or (at your option) any later
# version.
# 
# shinystan is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# this program; if not, see <http://www.gnu.org/licenses/>.


#' Launch the 'ShinyStan' app
#' 
#' Launch the 'ShinyStan' app in the default web browser. 'RStudio' users also
#' have the option of launching the app in the pop-up Viewer.
#' 
#' @export
#' @param object The object to use. For the default method this can be an object
#'   of class \code{"shinystan"}, \code{"stanfit"}, or \code{"stanreg"}. To use
#'   other types of objects first create a shinystan object using
#'   \code{\link{as.shinystan}}.
#' @param rstudio Only relevant for 'RStudio' users. The default (\code{FALSE})
#'   is to launch the app in the user's default web browser rather than the 
#'   pop-up Viewer provided by 'RStudio'. Users can change the default to
#'   \code{TRUE} by setting the global option \code{options(shinystan.rstudio =
#'   TRUE)}.
#' @param ... Optional arguments passed to \code{\link[shiny]{runApp}}.
#' 
#' @return The \code{launch_shinystan} function is used for the side effect of 
#'   starting the 'ShinyStan' app, but it also returns a \code{shinystan}
#'   object, an instance of S4 class \code{"shinystan"}.
#'   
#' @template seealso-as.shinystan 
#' @template seealso-update_sso 
#' @template seealso-demo
#' 
#' @template reference-muth
#' @template reference-bayesvis
#'   
#' @examples
#' \dontrun{
#' #######################################
#' # Example 1: 'sso' is a shinystan object
#' #######################################
#' 
#' # Just launch shinystan
#' launch_shinystan(sso)
#' 
#' # Launch shinystan and replace sso with an updated version of itself
#' # if any changes are made to sso while using the app
#' sso <- launch_shinystan(sso)
#' 
#' # Launch shinystan but save any changes made to sso while running the app
#' # in a new shinystan object sso2. sso will remained unchanged. 
#' sso2 <- launch_shinystan(sso) 
#' 
#' #######################################
#' # Example 2: 'sf' is a stanfit object
#' #######################################
#' 
#' # Just launch shinystan
#' launch_shinystan(sf)
#' 
#' # Launch shinystan and save the resulting shinystan object
#' sf_sso <- launch_shinystan(sf)
#' 
#' # Now sf_sso is a shinystan object and so Example 1 (above) applies when
#' # using sf_sso. 
#' 
#' #######################################
#' # Example 3: 'fit' is an mcmc.list, array or list of matrices
#' #######################################
#'
#' # First create shinystan object (see ?as.shinystan) for full details)
#' }
#'
launch_shinystan <- function(object, ...) {
  UseMethod("launch_shinystan")
}

#' @rdname launch_shinystan
#' @export
launch_shinystan.default <-
  function(object,
           ...,
           rstudio = getOption("shinystan.rstudio")) {
    if (!is.shinystan(object) && 
        !is.stanfit(object) && 
        !inherits(object, "CmdStanMCMC") &&
        !is.stanreg(object)) {
      stop("object not compatible with 'launch_shinystan'. ",
           "Try converting to a shinystan object first using 'as.shinystan'.")
    }
    object <- as.shinystan(object)
    message("\nLaunching ShinyStan interface... ",
            "for large models this  may take some time.")
    invisible(launch(object, rstudio, ...))
  }

#' @rdname launch_shinystan
#' @export
launch_shinystan.shinystan <-
  function(object,
           ...,
           rstudio = getOption("shinystan.rstudio")) {
    sso_check(object)
    message("\nLaunching ShinyStan interface... ",
            "for large models this  may take some time.")
    invisible(launch(object, rstudio, ...))
  }


#' 'ShinyStan' demo
#'
#' @aliases eight_schools
#' @export
#' @inheritParams launch_shinystan
#' @param demo_name The name of the demo. Currently \code{"eight_schools"} is 
#'   the only option, but additional demos may be available in future releases.
#'   \describe{
#'   \item{\code{eight_schools}}{Hierarchical meta-analysis model. See 
#'    \emph{Meta Analysis} chapter of the 'Stan' manual 
#'    \url{https://mc-stan.org/users/documentation/}.}
#'   }
#' @return An S4 shinystan object.
#'   
#' @template seealso-launch
#' @template seealso-as.shinystan
#' 
#' @examples
#' \dontrun{
#' # launch demo but don't save a shinystan object
#' launch_shinystan_demo() 
#' 
#' # launch demo and save the shinystan object for the demo 
#' sso_demo <- launch_shinystan_demo()
#' }
#'
launch_shinystan_demo <- function(demo_name = "eight_schools",
                                  rstudio = getOption("shinystan.rstudio"),
                                  ...) {
  demo_name <- match.arg(demo_name)
  data(list = demo_name, package = "shinystan", envir = environment())
  invisible(launch(get(demo_name, inherits = FALSE), rstudio = rstudio, ...))
}

# Internal launch function 
# @param sso shinystan object
# @param rstudio launch in rstudio viewer instead of web browser? 
# @param ... passed to shiny::runApp
launch <- function(sso, rstudio = FALSE, ...) {
  launch.browser <- if (!rstudio) 
    getOption("shiny.launch.browser", TRUE) else getOption("shiny.launch.browser", interactive())
  
  .sso_env$.SHINYSTAN_OBJECT <- sso  # see zzz.R for .sso_env
  on.exit(.sso_env$.SHINYSTAN_OBJECT <- NULL, add = TRUE)
  shiny::runApp(system.file("ShinyStan", package = "shinystan"), 
                launch.browser = launch.browser, ...)
}

sidebarPanel(
  width = 4,
  h4(strong(
    withMathJax("\\(\\LaTeX\\) table generator")
  )),
  selectInput(
    "tex_params",
    width = "100%",
    label = "Parameters (blank = all)",
    multiple = TRUE,
    choices = .param_list_with_groups,
    selected = if (length(.param_names) == 1) .param_names else .param_names[1:2]
  ),
  numericInput(
    "tex_digits",
    label = "Digits",
    value = 1,
    min = 0
  ),
  div(
    style = "padding: 1px;",
    checkboxGroupInput(
      "tex_columns",
      label = "Columns",
      choices = c(
        "Rhat",
        "Effective sample size" = "n_eff",
        "Posterior mean" = "mean",
        "Posterior standard deviation" = "sd",
        "Monte Carlo error" = "se_mean",
        "Quantile: 2.5%" = "2.5%",
        "Quantile: 25%" = "25%",
        "Quantile: 50%" = "50%",
        "Quantile: 75%" = "75%",
        "Quantile: 97.5%" = "97.5%"
      ),
      selected = c("Rhat", "n_eff", "mean", "sd", "2.5%", "50%", "97.5%")
    )
  ),
  textInput("tex_caption", label = "Caption"),
  checkboxGroupInput(
    "tex_pkgs",
    "Packages",
    choices = c("Booktabs", "Longtable"),
    selected = "Booktabs",
    inline = TRUE
  ),
  br()
)

# shinystan is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 3 of the License, or (at your option) any later
# version.
# 
# shinystan is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# this program; if not, see <http://www.gnu.org/licenses/>.

# check which shinystan created a shinystan object
sso_version <- function(sso) {
  ver <- sso@misc[["sso_version"]]
  if (!is.null(ver)) {
    package_version(ver)
  } else {
    package_version("2.0")
  }
}

# check object types
sso_check <- function(sso) {
  if (!is.shinystan(sso)) {
    stop("Please specify a shinystan object.", call. = FALSE)
  } else if (sso_version(sso) < utils::packageVersion("shinystan")) {
    stop(
      "Your shinystan object was created with a previous version of shinystan. ",
      "Please use the 'update_sso' function to update your object.",
      call. = FALSE
    )
  }
  invisible(TRUE)
}

is.stanfit <- function(x) inherits(x, "stanfit")
is.stanreg <- function(x) inherits(x, "stanreg")


# check for suggested (not required) packages
check_suggests <- function(pkg) {
  if (!requireNamespace(pkg, quietly = TRUE))
    stop(
      "You need to have the ", pkg,
      " package installed to use this option.",
      call. = FALSE
    )
}

# grepl with ignore.case defaulting to TRUE
grepl_ic <- function(pattern, x, ignore.case = TRUE) {
  grepl(pattern = pattern, x = x, ignore.case = ignore.case)
}


# nocov start
# release reminders (for devtools)
release_questions <- function() {
  c(
    "Have you updated version numbers in inst/CITATION?",
    "Have you updated version numbers in the citation on the About page?",
    "Have you updated version numbers in the citation on the Stan website?",
    "Have you updated NEWS.md?"
  )
}
# nocov end

if (exists(".SHINYSTAN_OBJECT")) {
  object <- .SHINYSTAN_OBJECT
} else {
  object <- get(".SHINYSTAN_OBJECT", envir = shinystan:::.sso_env)  
}

path_to_extract_sso <- file.path("server_files","utilities","extract_sso.R")
server_files <- list.files("server_files", full.names = TRUE, recursive = TRUE)
SERVER_FILES <- server_files[!server_files %in% path_to_extract_sso]
source("global_utils.R", local = TRUE)
source("server_utils.R", local = TRUE)
source(path_to_extract_sso, local = TRUE)

# BEGIN server ------------------------------------------------------
# ___________________________________________________________________
function(input, output, session) {
  
  # If not running on server then automatically stop app whenever browser tab
  # (or any session) is closed
  if (!nzchar(Sys.getenv("SHINY_PORT"))) {
    session$onSessionEnded(function() stopApp(object))
  }
  
  # Stop the app when "Save & Close" button is clicked
  observeEvent(
    input$save_and_close_button, 
    stopApp(object)
  )
  
  # Source all files from server_files directory and subdirectories
  for (f in SERVER_FILES) 
    source(f, local = TRUE)

  # Link to pages from home page table of contents
  toc_entries <- c("Estimate", "Diagnose", "Explore", "Model Code")
  observe({
    local({
      lapply(toc_entries, function(x) {
        id <- paste0("toc_", if (x == "Model Code") "more" else tolower(x))
        shinyjs::onclick(id, updateTabsetPanel(session, "nav", selected = x))
      })
    })
  })
  
  # Toggle options dropdowns
  options_trigger_ids <- c("table", "multiparam", "autocorr", "rhat_warnings", 
                           "bivariate", "trivariate", "density", "hist")
  observe({
    lapply(seq_along(options_trigger_ids), function(j) {
      shinyjs::onclick(
        id = paste0(options_trigger_ids[j], "_options_show"),
        shinyjs::toggle(
          id = paste0(options_trigger_ids[j], "_options"),
          anim = TRUE,
          animType = "slide",
          time = 0.4
        )
      )
    })
  })
  
  # Enable/disable individual options 
  density_trigger_ids <-
    c("point_est", "ci", "x_breaks", "fill_color", "line_color")
  observe({
    lapply(seq_along(density_trigger_ids), function(j) {
      shinyjs::toggleState(
        id = paste0("dens_", density_trigger_ids[j]),
        condition = input$dens_chain_split == "Together"
      )
    })
    shinyjs::toggleState(id = "ac_flip", condition = input$ac_combine == FALSE)
  })
  
  # Links to glossary
  observe({
    shinyjs::onclick(
      "open_glossary_from_table",
      updateTabsetPanel(session, "nav", selected = "Glossary")
    )
    shinyjs::onclick(
      "open_glossary_from_nuts_table",
      updateTabsetPanel(session, "nav", selected = "Glossary")
    )
  })
  
  # Enable/disable diagnostic plots
  diagnostic_trigger_ids <- 
    paste0("diagnostic_", c("param", "param_transform", "param_transform_go"))
  observe({
    diag_nav <- input$diagnostics_navlist
    local({
      if (diag_nav != 'By model parameter')
        lapply(diagnostic_trigger_ids, function(x)
          shinyjs::disable(id = x))
      else
        lapply(diagnostic_trigger_ids, function(x)
          shinyjs::enable(id = x))
    })
  })
  
  # Links to quick definitions
  observeEvent(
    input$open_quick_rhat, 
    shinyjs::info(includeText("text/quick_rhat.txt"))
  )
  observeEvent(
    input$open_quick_neff, 
    shinyjs::info(includeText("text/quick_neff.txt"))
  )
  observeEvent(
    input$open_quick_mcse, 
    shinyjs::info(includeText("text/quick_mcse.txt"))
  )
  
  # Show/hide citation
  observeEvent(
    input$shinystan_citation_show,
    shinyjs::toggle(
      id = "citation_div",
      anim = TRUE,
      animType = "fade"
    )
  )
  
} 
# END server ------------------------------------------------------
# _________________________________________________________________

# sample (accept_stat, lp)
div(class = "diagnostics-navlist-tabpanel",
    fluidRow(
      column(
        width = 7,
        fluidRow(
          column(
            width = 6,
            help_dynamic,
            dygraphOutput_175px("dynamic_trace_diagnostic_lp_out"),
            br(),
            dygraphOutput_175px("dynamic_trace_diagnostic_accept_stat_out")
          ),
          column(
            width = 6,
            help_lines,
            plotOutput_200px("lp_hist_out"),
            br(),
            plotOutput_200px("accept_stat_hist_out")
          )
        )), 
      column(
        width = 5,
        help_points,
        plotOutput_400px("accept_stat_vs_lp_out")
      )
    ))

# model parameter
div(class = "diagnostics-navlist-tabpanel",
    fluidRow(
      column(
        width = 7,
        help_dynamic,
        dygraphOutput_175px("dynamic_trace_diagnostic_parameter_out")
      ),
      column(width = 5, help_lines, plotOutput_200px("p_hist_out"))
    ),
    help_points,
    fluidRow(
      column(
        width = 6,
        plotOutput_200px("param_vs_lp_out"),
        plotOutput_200px("param_vs_stepsize_out")
      ),
      column(
        width = 6,
        plotOutput_200px("param_vs_accept_stat_out"),
        plotOutput_200px("param_vs_treedepth_out")
      )
    ),
    br()
)

fluidRow(
  column(
    width = 3,
    radioButtons(
      "sampler_warmup",
      label = h5("Warmup"),
      choices = list(Omit = "omit", Include = "include"),
      inline = TRUE
    )
  ),
  column(
    width = 4,
    radioButtons(
      "sampler_report",
      label = h5("Statistic"),
      choices = list(
        Mean = "average",
        SD = "sd",
        Max = "maximum",
        Min = "minimum"
      ),
      inline = TRUE
    )
  ),
  column(
    width = 2,
    numericInput(
      "sampler_digits",
      label = h5("Decimals"),
      value = 4,
      min = 0,
      max = 10,
      step = 1
    )
  )
)

navlistPanel(
  id = "pp_navlist",
  widths = c(3, 9),
  well = TRUE,
  tabPanel(
    "Select data",
    div(
      br(),
      uiOutput("ui_pp_get_y"),
      br(),
      uiOutput("ui_pp_get_yrep")
    )
  ), 
  "Plots",
  tabPanel(
    "Distribution of observed data vs replications",
    div(
      br(),
      h4(withMathJax(plot_descriptions["plot_hists_rep_vs_obs"])),
      br(),
      actionButton(
        "resample_hist_go",
        label = "Show different replications",
        icon = icon("refresh")
      ),
      fluidRow(
        column(
          width = 5,
          radioButtons(
            "pp_hists_rep_vs_obs_type",
            label = "",
            choices = list(Histograms = "histogram", Densities = "density"),
            inline = TRUE
          )
        ),
        column(
          width = 4,
          conditionalPanel(
            condition = "input.pp_hists_rep_vs_obs_type == 'density'",
            radioButtons(
              "pp_hists_rep_vs_obs_overlay",
              label = "",
              choices = list(Separate = FALSE, Overlay = TRUE),
              selected = FALSE,
              inline = TRUE
            )
          )
        )
      ),
      plotOutput("pp_hists_rep_vs_obs_out", width = "90%"),
      br()
    )
  ),
  tabPanel(
    "Distributions of test statistics",
    div(
      br(),
      h4(withMathJax(plot_descriptions["plot_test_statistics"])),
      helpText(
        "The blue lines show \\(T(y)\\), the value of the statistic computed from the observed data."
      ),
      radioButtons(
        "pp_hists_test_statistics_type",
        label = "",
        choices = list(Histograms = "histogram", Densities = "density"),
        inline = TRUE
      ),
      fluidRow(
        column(
          width = 6,
          plotOutput("pp_hists_test_statistics_mean_out", height = "200px")
        ),
        column(
          width = 6,
          plotOutput("pp_hists_test_statistics_sd_out", height = "200px")
        )
      ),
      br(),
      fluidRow(
        column(
          width = 6,
          plotOutput("pp_hists_test_statistics_min_out", height = "200px")
        ),
        column(
          width = 6,
          plotOutput("pp_hists_test_statistics_max_out", height = "200px")
        )
      ), 
      br()
    )
  ),
  tabPanel(
    "Scatterplots",
    div(
      br(),
      h4(withMathJax(plot_descriptions["plot_obs_vs_avg_y_rep"])),
      checkboxInput("pp_zoom_to_zero", "Zoom to include (0,0)", value = FALSE),
      plotOutput("pp_y_vs_avg_rep_out", height = "250px", width = "80%"),
      # h5(withMathJax(plot_descriptions["plot_avg_rep_vs_avg_resid_rep"])),
      # plotOutput("pp_avg_rep_vs_avg_resid_rep_out", height = "250px", width = "80%"),
      br()
    )
  ), 
  tabPanel(
    "Histograms of residuals",
    div(
      br(),
      h4(withMathJax(plot_descriptions["plot_hist_resids"])),
      br(),
      actionButton(
        "resample_resids_go",
        label = "Show a different replication",
        icon = icon("refresh")
      ),
      br(),br(),
      plotOutput("pp_hist_resids_out", height = "250px", width = "75%")
    )
  ),
  "About",
  tabPanel(
    "About graphical posterior predictive checking",
    source(file.path("ui_files", "pp_about.R"), local = TRUE)$value
  ),
  tabPanel(
    "Tutorial",
    includeMarkdown("markdown/pp_check_tutorial.md")
  )
)

# shinystan is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 3 of the License, or (at your option) any later
# version.
# 
# shinystan is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# this program; if not, see <http://www.gnu.org/licenses/>.


#' Update an object created by the previous version of shinystan
#' 
#' If you encounter any errors when using a shinystan object (\code{sso}) 
#' created by a previous version of \pkg{shinystan}, you might need to run
#' \code{update_sso}. If \code{update_sso} does not resolve the problem and 
#' you still have the object (e.g. stanfit,  stanreg, mcmc.list) from which 
#' \code{sso} was originally created, you can create a new shinystan object 
#' using \code{\link{as.shinystan}}.
#' 
#' @export
#' @template args-sso
#' @return If \code{sso} is already compatible with your version of
#'   \pkg{shinystan} then \code{sso} itself is returned and a message is printed
#'   indicating that \code{sso} is already up-to-date. Otherwise an updated
#'   version of \code{sso} is returned unless an error is encountered.
#'   
#' @template seealso-as.shinystan
#'   
#' @examples 
#' \dontrun{
#' sso_new <- update_sso(sso)
#' }
#' 
update_sso <- function(sso) {
  stopifnot(is.shinystan(sso))
  sso_ver <- sso_version(sso)
  shinystan_ver <- utils::packageVersion("shinystan")
  if (sso_ver == shinystan_ver) {
    message(deparse(substitute(sso)), " already up-to-date.")
    return(sso)
  } else if (sso_ver > shinystan_ver) {
    stop(
      deparse(substitute(sso)),
      " was created using a more recent version ",
      "of shinystan than the one you are currently using. ",
      "Please update your version of the shinystan package."
    )
  }
  
  slot(sso, "sampler_params") <-
    .rename_sampler_param(slot(sso, "sampler_params"),
                          oldname = "n_divergent__",
                          newname = "divergent__")
  sso_new <- shinystan()
  for (sn in slotNames(sso_new)) {
    if (.hasSlot(sso, sn)) {
      slot(sso_new, sn) <- slot(sso, sn)
    } else {
      new_slots <- c("posterior_sample", "n_chain", "n_iter", "n_warmup")
      old_slots <- c("samps_all", "nChains", "nIter", "nWarmup")
      j <- which(new_slots == sn)
      if (!length(j))
        stop("Bug found. Slot ", sn, " can't be updated.")
      if (.hasSlot(sso, old_slots[j])) {
        slot(sso_new, sn) <- slot(sso, old_slots[j])
      } else {
        stop("slot ", sn, " not found in ", deparse(substitute(sso))) 
      }
    }
  }
  sso_new@misc[["sso_version"]] <- utils::packageVersion("shinystan")
  message("shinystan object updated.")
  sso_new
}

# kernel density plot
dens_transform_x <- eventReactive(
  input$dens_transform_x_go > 0,
  input$dens_transform_x
)

user_xlim <- function(lim) {
  xz <- strsplit(lim, split = "c(", fixed = TRUE)[[1L]][2]
  xz <- strsplit(xz, split = ",", fixed = TRUE)[[1L]]
  if (identical(xz, NA_character_))
    return(FALSE)
  x_lim <- unlist(strsplit(xz, split = ")", fixed = TRUE))
  x_lim <- gsub(" ", "", x_lim)
  if (x_lim[1L] == "min")
    x_lim[1L] <- NA
  if (x_lim[2L] == "max")
    x_lim[2L] <- NA
  as.numeric(x_lim)
}

density_plot <- reactive({
  xzoom <- input$dens_xzoom
  if (xzoom == "")
    return(last_plot())
  
  validate(
    need(input$param, message = FALSE),
    need(!is.null(input$dens_chain), message = FALSE),
    need(xzoom, message = FALSE)
  )
  
  x_lim <- if (xzoom == "c(min, max)") {
    NULL
  } else {
    check <- try(user_xlim(xzoom))
    validate(need(check, message = "Invalid input"))
    check
  }
  
  chain <- input$dens_chain
  if (is.na(chain))
    chain <- 0
  prior_fam <- input$dens_prior
  prior_params <- if (prior_fam == "None") {
    NULL
  } else if (prior_fam == "Normal") {
    list(
      mean = input$dens_prior_normal_mu,
      sd = input$dens_prior_normal_sigma
    )
  } else if (prior_fam == "t") {
    list(
      df = input$dens_prior_t_df,
      location = input$dens_prior_t_mu,
      scale = input$dens_prior_t_sigma
    )
  } else if (prior_fam == "Cauchy") {
    list(
      location = input$dens_prior_cauchy_mu,
      scale = input$dens_prior_cauchy_sigma
    )
  } else if (prior_fam == "Beta") {
    list(
      shape1 = input$dens_prior_beta_shape1,
      shape2 = input$dens_prior_beta_shape2
    )
  } else if (prior_fam == "Exponential") {
    list(rate = input$dens_prior_expo_rate)
  } else if (prior_fam == "Gamma") {
    list(
      shape = input$dens_prior_gamma_shape,
      rate = input$dens_prior_gamma_rate
    )
  } else if (prior_fam == "Inverse Gamma") {
    list(
      shape = input$dens_prior_inversegamma_shape,
      scale = input$dens_prior_inversegamma_scale
    )
  } else {
    NULL
  }
  
  do.call(
    ".param_dens",
    args = list(
      param = input$param,
      dat = par_samps_post_warmup(),
      chain = chain,
      chain_split = input$dens_chain_split == "Separate",
      fill_color = input$dens_fill_color,
      line_color = input$dens_line_color,
      point_est = input$dens_point_est,
      CI = input$dens_ci,
      #     y_breaks    = input$dens_y_breaks,
      x_breaks = input$dens_x_breaks,
      x_lim = x_lim,
      prior_fam = prior_fam,
      prior_params = prior_params,
      transform_x = dens_transform_x()
    )
  )
})


output$density_plot_out <- renderPlot({
  suppress_and_print(density_plot())
}, bg = "transparent")

# download plot
output$download_density <- downloadHandler(
  filename = 'shinystan-density-gg.RData',
  content = function(file) {
    shinystan_density_gg <- density_plot()
    save(shinystan_density_gg, file = file)
  }
)
output$save_pdf_density = downloadHandler(
  filename = "shinstan-density.pdf",
  content = function(file) {
    ggsave(file, plot = density_plot(), device = pdf)
  }
)

# give ShinyStan app access to ggplot functions
load("ggplot_fns.rda")
lapply(ggplot_fns, function(f) {
  try(assign(f, getFromNamespace(f, "ggplot2"), envir = parent.frame(2)), silent = TRUE)
})

# load helper_functions
helpers <- file.path("helper_functions", list.files("helper_functions", full.names = FALSE))
for (h in helpers) 
  source(h, local = TRUE)

# avoid conflict with inline::code if rstan is loaded
code <- shiny::code


# make_param_list ------------------------------------------------------
# generate list of parameter names (formatted for shiny::selectInput)
.make_param_list <- function(object) {
  param_names <- slot(object, "param_names")
  param_dims <- slot(object, "param_dims")
  param_groups <- names(param_dims)
  choices <- list()
  ll <- length(param_dims)
  choices[seq_len(ll)] <- ""
  names(choices) <- param_groups
  for(i in seq_len(ll)) {
    if (length(param_dims[[i]]) == 0) {
      choices[[i]] <- list(param_groups[i])
    }
    else {
      temp <- paste0(param_groups[i],"\\[")
      choices[[i]] <- param_names[grep(temp, param_names)]
    }
  }
  choices
}

# make_param_list_with_groups ------------------------------------------------------
# generate list of parameter names and include parameter groups (formatted for
# shiny::selectInput)
.make_param_list_with_groups <- function(object, sort_j = FALSE) {
  param_names <- slot(object, "param_names")
  param_dims <- slot(object, "param_dims")
  param_groups <- names(param_dims)
  ll <- length(param_dims)
  LL <- sapply(seq_len(ll), function(i) length(param_dims[[i]]))
  choices <- list()
  choices[seq_len(ll)] <- ""
  names(choices) <- param_groups
  for(i in seq_len(ll)) {
    if (LL[i] == 0) {
      choices[[i]] <- list(param_groups[i])
    } else {
      group <- param_groups[i]
      temp <- paste0("^",group,"\\[")
      ch <- param_names[grep(temp, param_names)]

      #       toggle row/column major sorting so e.g. "beta[1,1], beta[1,2],
      #       beta[2,1], beta[2,2]" instead of "beta[1,1], beta[2,1], beta[1,2],
      #       beta[2,2]"
      if (sort_j == TRUE & LL[i] > 1)
        ch <- gtools::mixedsort(ch)

      ch_out <- c(paste0(group,"_as_shinystan_group"), ch)
      names(ch_out) <- c(paste("ALL", group), ch)
      choices[[i]] <- ch_out
    }
  }

  choices
}

# update parameter selection for multi-parameter plots --------------------
# update with regex
.test_valid_regex <- function(pattern) {
  trygrep <- try(grep(pattern, ""), silent = TRUE)
  if (inherits(trygrep, "try-error"))
    FALSE
  else
    TRUE
}
.update_params_with_regex <- function(params, all_param_names, regex_pattern) {
  sel <- which(all_param_names %in% params)
  to_search <- if (length(sel)) 
    all_param_names[-sel] else all_param_names
  if (!length(regex_pattern)) 
    return(params)
  
  to_add <- grep(regex_pattern, to_search, value = TRUE)
  if (!length(to_add)) 
    params 
  else 
    c(params, to_add)
}

# update with groups
.update_params_with_groups <- function(params, all_param_names) {
  as_group <- grep("_as_shinystan_group", params)
  if (!length(as_group)) 
    return(params)
  make_group <- function(group_name) {
    all_param_names[grep(paste0("^",group_name,"\\["), all_param_names)]
  }
  single_params <- params[-as_group]
  grouped_params <- params[as_group]
  groups <- gsub("_as_shinystan_group", "", grouped_params)
  groups <- sapply(groups, make_group)
  c(single_params, unlist(groups))
}


# generate color vectors --------------------------------------------------
color_vector <- function(n) {
  hues = seq(15, 375, length = n + 1)
  hcl(h=hues, l=50, c=50)[1:n]
}
color_vector_chain <- function(n) {
  hues = seq(15, 375, length = n + 1)
  hcl(h=hues, l=80, c=50)[1:n]
}

# transformations ---------------------------------------------------------
transformation_choices <-
  c(
    "abs", "atanh",
    cauchit = "pcauchy", "cloglog",
    "exp", "expm1",
    "identity", "inverse", inv_logit = "plogis",
    "log", "log10", "log2", "log1p", logit = "qlogis",
    probit = "pnorm",
    "square", "sqrt"
  )

inverse <- function(x) 1/x
cloglog <- function(x) log(-log1p(-x))
square <- function(x) x^2

# extra distributions for density comparisons -----------------------------
# t distribution with location and scale
.dt_loc_scale <- function(x, df, location, scale) {
  1/scale * dt((x - location)/scale, df)
}
# inverse gamma distribution
.dinversegamma <- function(x, shape, scale) {
  logout <- log(scale)*shape - lgamma(shape) - (1+shape)*log(x) - (scale/x)
  exp(logout)
}


# bold/strong text generators ---------------------------------------------
# used both in ui files and server files that call renderUI
strongMed <- function(...) 
  strong(style = "font-size: 14px; margin-bottom: 5px;", ...)
strongBig <- function(...) 
  strong(style = "font-size: 18px; margin-bottom: 5px;", ...)
strong_bl <- function(...) 
  strong(style = "color: #006DCC;", ...)

# shinystan is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 3 of the License, or (at your option) any later
# version.
#
# shinystan is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, see <http://www.gnu.org/licenses/>.


#' Deploy a 'ShinyStan' app on the web using 'shinyapps.io' by 'RStudio'
#'
#' Requires a (free or paid) 'ShinyApps' account. Visit
#' \url{https://www.shinyapps.io/} to sign up.
#'
#' @export
#' @template args-sso
#' @param appName The name to use for the application. Application names must be
#'   at least four characters long and may only contain letters, numbers, dashes
#'   and underscores.
#' @param account shinyapps.io account username. Only required if more than one
#'   account is configured on the system.
#' @param ... Optional arguments. See Details.
#' @param deploy Should the app be deployed? The only reason for this to be
#'   \code{FALSE} is if you just want to check that the preprocessing before
#'   deployment is successful.
#'
#' @return \link[=invisible]{Invisibly}, \code{TRUE} if deployment succeeded
#'   (did not encounter an error) or, if \code{deploy} argument is set to
#'   \code{FALSE}, the path to the temporary directory containing the app ready
#'   for deployment (also invisibly).
#'
#' @details In \code{...}, the arguments \code{ppcheck_data} and
#'   \code{ppcheck_yrep} can be specified. \code{ppcheck_data} should be a
#'   vector of observations to use for graphical posterior predictive checking
#'   and \code{ppcheck_yrep} should be a character string naming the parameter
#'   in \code{sso} containing the posterior predictive simulations/replications.
#'   The value of \code{ppcheck_yrep} is only used to preselect the appropriate
#'   parameter/generated quantity to use for the posterior predictive checking.
#'   \code{ppcheck_yrep} (but not \code{ppcheck_data}) can also be set
#'   interactively on shinyapps.io when using the app.
#'
#' @seealso The example in the \emph{Deploying to shinyapps.io} vignette that
#'   comes with this package.
#'
#'   \url{https://www.shinyapps.io/} to sign up for a free or paid 'ShinyApps'
#'   account and for details on how to configure your account on your local
#'   system using the \pkg{rsconnect} package from 'RStudio'.
#'
#' @examples
#' \dontrun{
#' # For this example assume sso is the name of the \code{shinystan} object for
#' # the model you want to use. Assume also that you want to name your app
#' # 'my-model' and that your shinyapps.io username is 'username'.
#'
#' deploy_shinystan(sso, appName = "my-model", account = "username")
#'
#' # If you only have one ShinyApps account configured then you can also omit
#' # the 'account' argument.
#'
#' deploy_shinystan(sso, appName = "my-model")
#' }
#'
deploy_shinystan <- function(sso, appName, account = NULL, ..., deploy = TRUE) {
  sso_check(sso)
  if (missing(appName))
    stop("'appName' is required.")

  # copy contents to temporary directory and write necessary additional lines to
  # ui, server, and global
  appDir <- tempdir()
  deployDir <- file.path(appDir, "ShinyStan")
  contents <- system.file("ShinyStan", package = "shinystan")
  file.copy(from = contents, to = appDir, recursive = TRUE)

  server_pkgs <- c(
    "shiny",
    "shinyjs",
    "colourpicker",
    "markdown",
    "shinythemes"
  )
  ui_pkgs <- c(
    server_pkgs,
    "ggplot2",
    "bayesplot",
    "gtools",
    "reshape2",
    "dygraphs",
    "xts",
    "xtable",
    "gridExtra",
    "DT",
    "threejs"
  )
  server_lines <- paste0("library(", server_pkgs, ");")
  ui_lines <- paste0("library(", ui_pkgs, ");")
  global_lines <- paste(
    "load('sso.RData');",
    "if (file.exists('y.RData')) load('y.RData')"
  )
  for (ff in c("ui", "server", "global")) {
    file_name <- file.path(deployDir, paste0(ff, ".R"))
    fconn <- file(file_name, 'r+')
    original_content <- readLines(fconn)
    new_lines <- get(paste0(ff, "_lines"))
    writeLines(c(new_lines, original_content), con = fconn)
    close(fconn)
  }

  # save sso to deployDir
  .SHINYSTAN_OBJECT <- sso
  save(.SHINYSTAN_OBJECT, file = file.path(deployDir, "sso.RData"))

  # save ppcheck_data and set ppcheck defaults
  pp <- list(...)
  if ("ppcheck_data" %in% names(pp)) {
    y <- pp$ppcheck_data
    save(y, file = file.path(deployDir, "y.RData"))
    if ("ppcheck_yrep" %in% names(pp))
      set_ppcheck_defaults(
        appDir = deployDir,
        yrep_name = pp$ppcheck_yrep,
        y_name = "y"
      )
  }

  if (!deploy)
    return(invisible(deployDir))

  if (!requireNamespace("rsconnect", quietly = TRUE)) {
    stop("Please install the 'rsconnect' package.", call. = FALSE)
  }
  rsconnect::deployApp(
    appDir = deployDir,
    appName = appName,
    account = account,
    lint = TRUE
  )
}



# functions to set defaults for ppcheck shiny::selectInput for y and y_rep
set_ppcheck_defaults <- function(appDir, yrep_name, y_name = "y") {
  stopifnot(is.character(yrep_name), is.character(y_name),
            length(yrep_name) == 1, length(y_name) == 1)
  fileDir <- file.path(appDir, "server_files", "pages", "diagnose", "ppcheck", "ui")
  ppc_file <- file.path(fileDir, "pp_get_y_and_yrep.R")
  if (file.exists(ppc_file)) {
    file.remove(ppc_file)
    file.create(ppc_file)
  }
  .write_files(files = ppc_file, lines = .ppc_lines(y_name, yrep_name))
}

.write_files <- function(files, lines) {
  stopifnot(length(files) == length(lines))
  for (f in seq_along(files)) {
    fileConn <- file(files[f])
    writeLines(lines[f], fileConn)
    close(fileConn)
  }
}

.ppc_lines <- function(y_name = "y", yrep_name) {
  paste0(
    "output$ui_pp_get_y <- renderUI({
      choices <- objects(envir = .GlobalEnv)
      selectizeInput('y_name', label = span(style = 'color: #337ab7;', 'y, a vector of observations'),
      choices = c('', choices),
      selected = '", y_name,"')
      })

    output$ui_pp_get_yrep <- renderUI({
      choices <- PARAM_NAMES
      choices <- strsplit(choices, split = '[', fixed = TRUE)
      choices <- lapply(choices, function(i) return(i[1]))
      choices <- unique(unlist(choices))
      selectizeInput('yrep_name',
      label = span(style = 'color: #337ab7;', 'y_rep, posterior predictive replications'),
      choices = c('', choices),
      selected = '", yrep_name,"')
    })"
  )
}

library(shinystan)
context("Deploying")

sso <- eight_schools

# load 'old_sso', a shinystan object created by previous shinystan version
load("old_sso_for_tests.rda")
old_sso_msg <- "use the 'update_sso' function to update your object"

test_that("deploy_shinystan error checking works", {
  expect_error(deploy_shinystan(old_sso), old_sso_msg)
  expect_error(deploy_shinystan(sso@posterior_sample), 
               regexp = "specify a shinystan object")
  expect_error(deploy_shinystan(sso), 
               regexp = "'appName' is required")
})

test_that("deploy_shinystan preprocessing doesn't error", {
  expect_silent(deploy_test <- deploy_shinystan(sso, appName = "test", deploy = FALSE))
  expect_true(grepl("shinystan", deploy_test, ignore.case = TRUE))
  expect_true(dir.exists(deploy_test))
})

test_that("deploy_shinystan pp_check processing functions ok", {
  x <- "123454321"
  expect_output(cat(.ppc_lines(x, x)), x)
  
  deploy_pp_test <- deploy_shinystan(sso, appName = "test", deploy = FALSE, 
                                     ppcheck_data = rep(1, 8), 
                                     ppcheck_yrep = "yrep")
  expect_true(dir.exists(deploy_pp_test))
})

output$ui_multiparam_selectize <- renderUI({
  choices <- make_param_list_with_groups_sort()
  selected <- c(input$params_to_plot)
  selectizeInput(
    "params_to_plot",
    label = h5("Select or enter parameter names"),
    width = '100%',
    choices = choices,
    multiple = TRUE
  )
})


# updating the choices and selected for the selectizeInput input$params_to_plot
# when the sorting option is changed in input$param_plot_sort_j or when
# parameters added by regex search

copy_params_to_plot <- reactive({
  copy <- input$params_to_plot
  if (is.null(copy) || !length(copy))
    NULL
  else
    copy
})

observe({
  x <- input$param_plot_sort_j
  choices <- make_param_list_with_groups_sort()
  selected <- copy_params_to_plot()
  selected <- .update_params_with_groups(selected, PARAM_NAMES)
  updateSelectizeInput(
    session,
    inputId = "params_to_plot",
    choices = choices,
    selected = selected
  )
})

observeEvent(input$param_plot_regex, {
  pattern <- input$params_to_plot_regex
  if (pattern != "") {
    choices <- make_param_list_with_groups_sort()
    selected <- copy_params_to_plot()
    selected <- .update_params_with_groups(selected, PARAM_NAMES)
    if (.test_valid_regex(pattern)) {
      selected <- .update_params_with_regex(selected, PARAM_NAMES, pattern)
      updateSelectizeInput(
        session,
        inputId = "params_to_plot",
        choices = choices,
        selected = selected
      )
    }
  }
})

output$invalid_regex <- renderText({
  pattern <- input$params_to_plot_regex
  if (length(pattern)) {
    msg <- "Invalid regular expression.\nYou might need to add the escape character '\\' ."
    validate(need(.test_valid_regex(pattern), message = msg))
  }
})

div(id = "notepad_div",
    sidebarLayout(
      sidebarPanel(
        width = 3,
        style = "height: 550px;",
        br(),
        h4("Notes"),
        helpText(
          style = "font-size: 12px;",
          p("Notes are displayed here each time you launch ShinyStan with this shinystan object.")
        ),
        br(),
        actionButton(
          "save_user_model_info",
          label = "Save notes",
          icon = icon("save")
        ),
        div(style = "font-size: 11px;", textOutput("user_text_saved")),
        conditionalPanel(
          condition = "input.save_user_model_info > 0",
          br(),
          save_and_close_reminder("save_user_model_info_safe_quit")
        )
      ),
      mainPanel(
        width = 9,
        br(), br(),
        tags$textarea(
          id = "user_model_info",
          rows = 20,
          cols = 80,
          .notes
        )
      )
    ))

# param_trace_multi ------------------------------------------------------
# trace plots for multiple parameters
.param_trace_multi <- function(params = NULL, all_param_names, dat,
                               warmup_val = 0,
                               chain = 0, palette = "Default",
                               rect = "Samples", rect_color = "skyblue",
                               rect_alpha = 0.1,
                               layout = "Long", x1, x2) {


  params <- .update_params_with_groups(params, all_param_names)
  if(length(params) == 0) {
    params <- dimnames(dat)$parameters[1:min(4, dim(dat)[3])]
  }
  params <- unique(params)
  dat <- reshape2::melt(dat[,,params, drop=FALSE])

  if (!("chains" %in% colnames(dat))) { # fixes for if there's only 1 chain:
    dat$chains <- "chain:1"
  }
  dat$iterations <- x1:x2
  if (chain != 0)
    dat <- subset(dat, chains == paste0("chain:",chain))
  rect_xmin <- ifelse(rect == "Samples", Inf, -Inf)
  shading_rect <- annotate("rect", xmin = rect_xmin, xmax = warmup_val,
                           ymin = -Inf, ymax = Inf, fill = rect_color, alpha = rect_alpha)

  xy_labs <- labs(y = "Value", x = "Iteration")
  nclrs <- length(unique(dat$chains))

  lgnd_title <- ""
  if(palette == "Default") clrs <- scale_color_discrete(name = lgnd_title)
  if(palette == "Gray") clrs <- scale_color_grey(name = lgnd_title)
  if(palette == "Brewer (spectral)") clrs <- scale_color_brewer(name = lgnd_title, palette = "Spectral")
  if(palette == "Rainbow") clrs <- scale_colour_manual(name = lgnd_title, values = rainbow(nclrs))

  lgnd_txt <- theme(legend.text =  element_text(size = 13, face = "bold"))

  graph <- ggplot(dat, aes(x = iterations, y = value, color = chains))
  graph <- graph + xy_labs + clrs + theme_classic() %+replace% (axis_color + axis_labs + fat_axis + h_lines + lgnd_top + lgnd_txt + strip_txt + transparent)
  if (rect != "None") graph <- graph + shading_rect
  graph <- graph + geom_line(size = 0.35) + scale_x_continuous(limits = c(x1, x2))

  if (layout == "Grid") {
    graph <- graph + facet_wrap(~ parameters, scales = "free_y")
  } else {
    graph <- graph + facet_grid(parameters ~., scales = "free_y")
  }

  graph
}


# param_hist --------------------------------------------------------------
# histogram for a single parameter
.param_hist <- function(param, dat, chain, binwd,
                        transform_x = "identity",
                        fill_color = "gray20", line_color = "gray35",
                        title = TRUE) {

  ttl <- "Histogram of Posterior Draws \n"
  if (transform_x != "identity") {
    t_x <- get(transform_x)
    dat <- apply(dat, 2, t_x)
  }
  x_lab <- if (transform_x != "identity")
    paste0(transform_x, "(", param, ")") else param

  dat <- reshape2::melt(dat)
  if (!("chains" %in% colnames(dat))) { # fixes for if there's only 1 chain:
    dat$chains <- "chain:1"
    dat$iterations <- 1:nrow(dat)
  }
  if (chain != 0) dat <- subset(dat, chains == paste0("chain:",chain))

  graph <- ggplot(dat, aes(x = value))

  if (binwd == 0) {
    graph <- graph + geom_histogram(fill = fill_color, color = line_color, size = 0.2)
  } else {
    graph <- graph + geom_histogram(fill = fill_color, color = line_color,
                                    binwidth = binwd, size = 0.2)
  }
  graph <- graph +
    labs(x = x_lab, y = "") +
    theme_classic() %+replace%
    (title_txt + axis_color + axis_labs + fat_axis + no_yaxs + transparent)
  if (title == TRUE) graph <- graph + ggtitle(ttl)

  graph
}


# param_dens --------------------------------------------------------------
# density plot for a single parameter

# data.frame of prior families and function names
priors <- data.frame(family = c("Normal", "t", "Cauchy", "Beta", "Exponential",
                                "Gamma", "Inverse Gamma"),
                     fun = c("dnorm", ".dt_loc_scale", "dcauchy", "dbeta",
                             "dexp", "dgamma", ".dinversegamma"))

.param_dens <- function(param, dat, chain,
                        fill_color = NULL, line_color = NULL,
                        point_est = "None", CI,
                        x_breaks = "Some", # y_breaks = "None",
                        x_lim = NULL,
                        chain_split = FALSE,
                        title = TRUE,
                        transform_x = "identity",
                        prior_fam = "None", prior_params) {

  ttl <- "Kernel Density Estimate \n"
  if (transform_x != "identity") {
    t_x <- get(transform_x)
    dat <- apply(dat, 2, t_x)
  }
  x_lab <- if (transform_x != "identity")
    paste0(transform_x, "(", param, ")") else param

  dat <- reshape2::melt(dat)
  if (!("chains" %in% colnames(dat))) { # fixes for if there's only 1 chain:
    dat$chains <- "chain:1"
    dat$iterations <- 1:nrow(dat)
  }
  if (chain != 0)
    dat <- subset(dat, chains == paste0("chain:",chain))

  Mean <- mean(dat$value)
  Median <- median(dat$value)
  dens_dat <- with(density(dat$value), data.frame(x,y))
  MAP <- with(dens_dat, x[which.max(y)])

  fclr <- ifelse(is.null(fill_color), "black", fill_color)
  lclr <- ifelse(is.null(line_color), "lightgray", line_color)

  many_breaks <- function(x) pretty(x, n = 15)
  too_many_breaks <- function(x) pretty(x, n = 35)
  if(x_breaks == "None") x_scale <- scale_x_continuous(breaks = NULL)
  if(x_breaks == "Some") x_scale <- scale_x_continuous()
  if(x_breaks == "Many") x_scale <- scale_x_continuous(breaks = many_breaks)

  if (chain == 0 & chain_split == TRUE) {
    graph <- ggplot(dat, aes(x = value, color = chains, fill = chains))
    if (prior_fam != "None") {
      graph <- graph + stat_function(alpha=0.75,color = "black", fun = as.character(priors$fun[priors$family==prior_fam]), args = prior_params, show_guides = TRUE)
    }
    graph <- graph +
      geom_density(alpha = 0.15) +
      scale_color_discrete("") +
      scale_fill_discrete("") +
      labs(x = x_lab, y = "") +
      x_scale + # y_scale +
      theme_classic() %+replace% (title_txt + axis_color + axis_labs + fat_axis + no_yaxs + transparent)

    if (title == TRUE) graph <- graph + ggtitle(ttl)
    return(graph)
  }

  graph <- ggplot(dens_dat, aes(x = x, ymax = y))
  if (prior_fam != "None") {
    graph <- graph +
      stat_function(fun = as.character(priors$fun[priors$family==prior_fam]),
                    args = prior_params)
  }
  graph <- graph +
    labs(x = param, y = "") +
    x_scale + # y_scale +
    labs(x = x_lab, y = "") +
    geom_ribbon(ymin = 0, fill = fclr, color = lclr, alpha = if (prior_fam == "None") 1 else 0.85) +
    theme_classic() %+replace% (title_txt + axis_color + axis_labs + fat_axis + no_yaxs + transparent)

  if (title == TRUE) graph <- graph + ggtitle(ttl)
  if (point_est != "None") {
    graph <- graph + annotate("segment",
                              x = get(point_est), xend = get(point_est),
                              y = 0, yend = max(dens_dat$y),
                              color = lclr, lwd = 1, lty = 2)
  }
  if (CI != "None") {
    lev <- (1 - as.numeric(CI))/2
    quant <- quantile(dat$value, probs = c(lev, 1 - lev))
    graph <- (graph +
                annotate("segment", x = quant, xend = quant, y = 0, yend = max(dens_dat$y), color = lclr, lty = rep(1:length(CI),2))
    )
  }
  if (!is.null(x_lim)) graph <- graph + scale_x_continuous(limits = x_lim)
  graph
}


# autocorr ----------------------------------------------------
.ac_fun <- function(x, lag.max, partial = FALSE) {
  if (!partial)
    acf(x, lag.max = lag.max, plot = FALSE)$acf[,, 1L]
  else
    pacf(x, lag.max = lag.max, plot = FALSE)$acf[,, 1L]
}
.ac_plot_data <- function(dat, lags, partial = FALSE) {
  nc <- length(unique(dat$chains))
  ac_list <- tapply(dat$value, INDEX = dat$chains, FUN = .ac_fun, lag.max = lags,
                    partial = partial, simplify = FALSE)
  nl <- if (partial) lags else lags + 1
  ch <- factor(rep(1:nc, each = nl), labels = paste0("chain:", 1:nc))
  ll <- rep(seq(if (partial) 1 else 0, lags), nc)
  data.frame(chains = ch, ac = do.call("c", args = ac_list), lag = ll)
}
.ac_plot_data_multi <- function(dat, lags, partial = FALSE) {
  nc <- length(unique(dat$chains))
  np <- length(unique(dat$parameters))
  ac_list <- tapply(dat$value, INDEX = list(dat$chains, dat$parameters),
                    FUN = .ac_fun, lag.max = lags,
                    partial = partial, simplify = FALSE)
  nl <- if (partial) lags else lags + 1
  ch <- factor(rep(rep(1:nc, each = nl), np), labels = paste0("chain:", 1:nc))
  ll <- rep(seq(if (partial) 1 else 0, lags), nc * np)
  pp <- factor(rep(1:np, each = nc * nl), labels = levels(dat$parameters))
  data.frame(parameters = pp, chains = ch, ac = do.call("c", args = ac_list), lag = ll)
}

# markov chain autocorrelation plot for single parameters
.autocorr_single_plot <- function(samps, lags) {
  dat <- reshape2::melt(samps)
  if (!("chains" %in% colnames(dat))) { # fixes for if there's only 1 chain:
    dat$chains <- "chain:1"
    dat$iterations <- 1:nrow(dat)
  }
  ac_dat <- .ac_plot_data(dat, lags)
  ac_labs <- labs(x = "Lag", y = "Autocorrelation")
  ac_theme <- theme_classic() %+replace%
    (axis_color + axis_labs + fat_axis + no_lgnd + transparent)
  y_scale <- scale_y_continuous(breaks = seq(0, 1, 0.25),
                                labels = c("0","","0.5","",""))
  graph <- ggplot(ac_dat, aes(x = lag, y = ac))
  graph <- graph +
    geom_bar(position = "identity", stat = "identity", fill = base_fill) +
    y_scale + ac_theme
  graph
}

# markov chain autocorrelation plot for multiple parameters
.autocorr_plot <- function(samps, partial = FALSE,
                           lags = 25, flip = FALSE,
                           combine_chains = FALSE) {

  params <- dimnames(samps)$parameters
  nParams <- length(params)
  nChains <- dim(samps)[2L]
  dat <- reshape2::melt(samps)
  if (!("chains" %in% colnames(dat))) { # fixes for if there's only 1 chain:
    dat$chains <- "chain:1"
    dat$iterations <- 1:nrow(dat)
  }
  ac_type <- if (partial) "partial" else "correlation"
  if (nParams == 1) ac_dat <- .ac_plot_data(dat, lags = lags, partial = partial)
  else ac_dat <- .ac_plot_data_multi(dat, lags = lags, partial = partial)

  ac_labs <- labs(x = "Lag", y = if (partial)
    "Partial autocorrelation" else "Autocorrelation")
  ac_theme <- theme_classic() %+replace%
    (axis_color + axis_labs + fat_axis + no_lgnd + strip_txt + transparent)
  y_scale <- scale_y_continuous(breaks = seq(0, 1, 0.25),
                                labels = c("0","","0.5","",""))
  title_theme <- theme(plot.title =  element_text(face = "bold", size = 18))
  if (combine_chains) {
    graph <- ggplot(ac_dat, aes(x= lag, y = ac))
    graph <- graph +
      geom_bar(position = "identity", stat = "identity",
               fill = base_fill, size = 0.4) +
      y_scale +
      ac_labs +
      ac_theme

    if (nParams == 1) return(graph + ggtitle(paste(params, "\n")) + title_theme)
    else return(graph + facet_wrap(~parameters))
  }

  graph <- ggplot(ac_dat, aes(x = lag, y = ac, color = factor(chains),
                              fill = factor(chains)))
  graph <- graph +
    geom_bar(position = "identity", stat = "identity", size = 0.4) +
    scale_fill_manual(values = rep(base_fill, nChains)) +
    scale_color_manual(values = rep(vline_base_clr, nChains)) +
    y_scale +
    ac_labs +
    ac_theme

  if (nParams == 1) {
    graph <- graph +
      facet_wrap(~chains) + ggtitle(paste(params, "\n")) + title_theme
    return(graph)
  } else { # nParams > 1

    while(is.null(flip)) return()

    graph <- graph +
      if (flip) facet_grid(chains ~ parameters) else facet_grid(parameters ~ chains)
    return(graph)
  }
}





# multiparam_plot --------------------------------------------------
# main plot of multiple parameters
.multiparam_plot <- function(samps, params = NULL, all_param_names,
                             show_density, show_ci_line, CI.level = 0.5,
                             show.level = 0.95, point_est, rhat_values,
                             color_by_rhat, rhat_palette, fill_color,
                             outline_color, est_color) {

  # params <- .update_params_with_regex(params, all_param_names)
  params <- .update_params_with_groups(params, all_param_names)
  .e <- environment()
  dim.samps <- dim(samps) #nIter, nChain, nParam
  if(length(params) == 0) {
    params <- dimnames(samps)$parameters[1:min(12, dim.samps[3])]
    if ("log-posterior" %in% params) {
      params <- params[-which(params == "log-posterior")]
    }
  }
  params <- unique(params)

  Blues <- c("#C6DBEF", "#4292C6", "#08306B")
  Grays <- c("#D9D9D9", "#737373", "#000000")
  Greens <- c("#C7E9C0", "#41AB5D", "#00441B")
  Oranges <- c("#FDD0A2", "#F16913", "#7F2704")
  Purples <- c("#DADAEB", "#807DBA", "#3F007D")
  Reds <- c("#FCBBA1", "#EF3B2C", "#67000D")
  rhat_pal <- get(rhat_palette)
  rhat_id <- ifelse(rhat_values < 1.05, "A",
                    ifelse(rhat_values < 1.1, "B", "C"))
  rhat_id <- factor(rhat_id[params], levels = c("A","B", "C"),
                    labels = c("<1.05", "<1.1", ">1.1"))
  rhat_colors <- scale_color_manual(name = bquote(hat(R)),
                                    values = rhat_pal,
                                    drop = FALSE)
  rhat_lgnd <- theme(legend.position = "top",
                     legend.title =  element_text(size = 13, face = "bold"),
                     legend.text =  element_text(size = 12))

  nParams <- length(params)
  nIter <- prod(dim.samps[1:2])
  samps.use <- array(samps[,,params], c(nIter, nParams))
  colnames(samps.use) <- params

  probs.use <- c(0.5 - show.level / 2,
                 0.5 - CI.level / 2,
                 0.5,
                 0.5 + CI.level / 2,
                 0.5 + show.level / 2)
  samps.quantile <- t(apply(samps.use, 2, quantile, probs = probs.use))
  y <- as.numeric(seq(nParams, 1, by = -1))
  xlim.use <- c(min(samps.quantile[,1]), max(samps.quantile[,5]))
  xrange <- diff(xlim.use)
  xlim.use[1] <- xlim.use[1] - 0.05 * xrange
  xlim.use[2] <- xlim.use[2] + 0.05 * xrange

  xy.df <- data.frame(params, y, samps.quantile)

  colnames(xy.df) <- c("params", "y", "ll", "l", "m", "h", "hh")
  if (point_est == "Mean") {
    xy.df$m <- unname(colMeans(samps.use))
  }
  p.base <- ggplot(xy.df, environment = .e)
  p.name <- scale_y_continuous(breaks = y, labels = params, limits = c(0.5, nParams + 1))
  p.theme <- theme(axis.title= element_blank(),
                   panel.background =  element_blank(),
                   panel.border =  element_blank(),
                   axis.ticks.y =  element_blank(),
                   axis.text= element_text(size=12),
                   axis.text.y= element_text(face = "bold"),
                   axis.line= element_line(size = 4, color = axis_line_color),
                   axis.line.y= element_line(size = 0.5, color = axis_line_color),
                   legend.position = "none",
                   panel.grid.major =  element_line(size = 0.4),
                   panel.grid.minor.y =  element_blank())

  p.all <- p.base + xlim(xlim.use) + p.name + theme_bw() + p.theme + transparent

  if (show_ci_line | show_density) {
    p.ci <- geom_segment(aes(x = ll, xend = hh, y = y, yend = y),
                         colour = outline_color)
    p.all <- p.all + p.ci
  }
  if (show_density) {
    nPoint.den <- 512
    #plot density
    y.den <- matrix(0, nrow = nPoint.den, ncol = nParams)
    x.den <- matrix(0, nrow = nPoint.den, ncol = nParams)
    for(i in 1:nParams){
      d.temp <- density(samps.use[,i],
                        from = samps.quantile[i,1],
                        to = samps.quantile[i,5],
                        n = nPoint.den)
      x.den[,i] <- d.temp$x
      y.max <- max(d.temp$y)
      y.den[,i] <- d.temp$y / y.max * 0.8 + y[i]
    }
    df.den <- data.frame(x = as.vector(x.den), y = as.vector(y.den),
                         name = rep(params, each = nPoint.den))
    p.den <- geom_line(data = df.den, aes(x = x, y = y, group = name),
                       color = outline_color)

    #shaded interval
    y.poly <- matrix(0, nrow = nPoint.den + 2, ncol = nParams)
    x.poly <- matrix(0, nrow = nPoint.den + 2, ncol = nParams)
    for(i in 1:nParams){
      d.temp <- density(samps.use[,i],
                        from = samps.quantile[i,2],
                        to = samps.quantile[i,4],
                        n = nPoint.den)
      x.poly[,i] <- c(d.temp$x[1], as.vector(d.temp$x), d.temp$x[nPoint.den])
      y.max <- max(d.temp$y)
      y.poly[,i] <- as.vector(c(0, as.vector(d.temp$y) / y.max * 0.8, 0) + y[i])
    }
    df.poly <- data.frame(x = as.vector(x.poly), y = as.vector(y.poly),
                          name = rep(params, each = nPoint.den + 2))
    p.poly <- geom_polygon(data = df.poly, aes(x = x, y = y, group = name, fill = y))
    p.col <- scale_fill_gradient(low = fill_color, high = fill_color, guide = "none")

    #point estimator
    if (color_by_rhat) {
      p.point <- geom_segment(aes(x = m, xend = m, y = y, yend = y + 0.25,
                                  color = rhat_id), size = 1.5)
      p.all + p.poly + p.den + p.col + p.point + rhat_colors + rhat_lgnd
    } else {
      p.point <- geom_segment(aes(x = m, xend = m, y = y, yend = y + 0.25),
                              colour = est_color,
                              size = 1.5)
      p.all + p.poly + p.den + p.col + p.point
    }

  } else {
    p.ci.2 <- geom_segment(aes(x = l, xend = h, y = y, yend = y),
                           colour = fill_color, size = 2)
    if (color_by_rhat) {
      p.point <- geom_point(aes(x = m, y = y, fill = rhat_id), color = "black",
                            shape = 21, size = 4)
      p.all + p.ci.2 + p.point + rhat_colors + rhat_lgnd
    } else {
      p.point <- geom_point(aes(x = m, y = y), size = 4, color = fill_color,
                            fill = est_color, shape = 21)
      p.all + p.ci.2 + p.point
    }
  }
}


# histogram of rhat, n_eff/N or mcse/sd -----------------------------------
.rhat_neff_mcse_hist <- function(dat, which, N) {
  # samps: post-warmup samples
  xlab <- switch(which,
         rhat = "Rhat statistic",
         n_eff = "Effective sample size / iterations",
         mcse = "Monte Carlo se / posterior sd"
         )
  my_labs <- labs(y = "", x = xlab)
  base_fill
  graph <- qplot(x = x, data = dat, color = I(vline_base_clr),
                 fill = I(base_fill), size = I(0.2))
  graph <- graph +
    my_labs +
    theme_classic() %+replace% (axis_color + axis_labs + fat_axis + no_yaxs + transparent)

  graph
}

# n_eff_warnings -----------------------------------------------------------
.n_eff_warnings <- function(summary, threshold = 10,
                            N_total = NULL) {
  n_eff <- summary[,"n_eff"]
  warn_params <- names(which(n_eff / N_total < threshold / 100))
  ll <- length(warn_params)
  if (ll == 0) "None"
  else paste0(warn_params, collapse = ", ")
}

# rhat_warnings -----------------------------------------------------------
.rhat_warnings <- function(summary, threshold = 1.10) {
  rhat <- summary[,"Rhat"]
  warn_params <- names(which(rhat > threshold))
  ll <- length(warn_params)
  if (ll == 0) "None"
  else paste0(warn_params, collapse = ", ")
}

# mcse_over_sd_warnings -----------------------------------------------------------
.mcse_over_sd_warnings <- function(summary, threshold = 10) {
  dat <- summary[,c("se_mean", "sd")]
  warn_params <- names(which(dat[,1] > (threshold/100) * dat[,2]))

  ll <- length(warn_params)
  if (ll == 0) "None"
  else paste0(warn_params, collapse = ", ")
}


# dynamic trace plot ------------------------------------------------------
.param_trace_dynamic <- function(param_samps, chain,
                                 warmup_val, warmup_shade = TRUE,
                                 stack = FALSE, grid = FALSE,
                                 x_lab = NULL, y_lab = NULL) {

  dim_samps <- dim(param_samps)
  if (is.null(dim_samps)) nChains <- 1
  else nChains <- dim_samps[2]
  if (nChains == 1) {
    param_chains <- xts::as.xts(ts(param_samps, start = 1))
  } else {
    if (chain != 0) {
      param_samps <- param_samps[, chain]
      param_chains <- xts::as.xts(ts(param_samps, start = 1))
    } else {
      param_chains <- xts::as.xts(ts(param_samps[,1], start = 1))
      for (i in 2:nChains) {
        param_chains <- cbind(param_chains, xts::as.xts(ts(param_samps[,i], start = 1)))
      }
      colnames(param_chains) <- paste0("Chain", 1:nChains)
    }
  }

  `%>%` <- dygraphs::`%>%`
  shade_to <- if (warmup_shade)
    paste0(warmup_val,"-01-01") else "0001-01-01"
  y_axis_label_remove <- if (stack)
    "white" else NULL
  clrs <- color_vector(nChains)
  if (chain != 0) clrs <- clrs[chain]
  dygraphs::dygraph(param_chains, xlab = x_lab, ylab = y_lab) %>%
    dygraphs::dyAxis("y", axisLabelColor = y_axis_label_remove) %>%
    dygraphs::dyAxis("x", axisLabelColor = "white") %>%
    dygraphs::dyOptions(colors = clrs, stackedGraph = stack, drawGrid = grid,
                        animatedZooms = TRUE, axisLineColor = axis_line_color) %>%
    dygraphs::dyLegend(show = "never") %>%
    dygraphs::dyHighlight(highlightCircleSize = 4,
                highlightSeriesBackgroundAlpha = 1/3,
                hideOnMouseOut = TRUE,
                highlightSeriesOpts = list(strokeWidth = 1.75)) %>%
    dygraphs::dyRoller(rollPeriod = 1) %>%
    dygraphs::dyShading(from = "0001-01-01", to = shade_to, color = "#EFEFEF", axis = "x") %>%
    dygraphs::dyCSS(css = "css/ShinyStan_dygraphs.css")
}

# trivariate_plot ---------------------------------------------------------
.param_trivariate <- function(samps, params,
                              transform_x = "identity", transform_y = "identity",
                              transform_z = "identity",
                              pt_size = 1, pt_color = "gray35", show_grid = TRUE,
                              flip_y = TRUE) {
  nParams <- 3
  dim_samps <- dim(samps)
  nIter <- dim_samps[1] * dim_samps[2]
  samps_use <- array(samps[,, params], c(nIter, nParams))
  colnames(samps_use) <- params

  t_x <- get(transform_x)
  t_y <- get(transform_y)
  t_z <- get(transform_z)

  if (transform_x != "identity") {
    samps_use[,1] <- t_x(samps_use[,1])
    colnames(samps_use)[1] <- paste0(transform_x, "(", params[1], ")")
  }
  if (transform_y != "identity") {
    samps_use[,2] <- t_y(samps_use[,2])
    colnames(samps_use)[2] <- paste0(transform_y, "(", params[2], ")")
  }
  if (transform_z != "identity") {
    samps_use[,3] <- t_z(samps_use[,3])
    colnames(samps_use)[3] <- paste0(transform_z, "(", params[3], ")")
  }
  threejs::scatterplot3js(samps_use, size = pt_size, color = pt_color,
                          grid = show_grid, flip.y = flip_y)
}


# bivariate plot ----------------------------------------------------------
.bivariate_plot <- function(samps, sp = NULL, max_td = NULL,
                            param, param2,
                            pt_alpha = 0.10,
                            pt_size = 2,
                            pt_shape = 10,
                            pt_color = "gray20",
                            ellipse_color = "black",
                            ellipse_lev = "None",
                            ellipse_lty = 1,
                            ellipse_lwd = 1,
                            ellipse_alpha = 1,
                            lines = "back",
                            lines_color = "gray",
                            lines_alpha,
                            points = TRUE,
                            transform_x = "identity",
                            transform_y = "identity"
){

  shape_translator <- function(x) {
    shape <- if (x >= 6) x + 9 else x
    shape
  }

  params <- c(param, param2)
  nParams <- 2
  nIter <- dim(samps)[1] * dim(samps)[2]
  samps_use <- array(samps[,,params], c(nIter, nParams))
  colnames(samps_use) <- params

  t_x <- get(transform_x)
  # t_x <- function(x) eval(parse(text = transform_x))
  t_y <- get(transform_y)
  x_lab <- if (transform_x != "identity")
    paste0(transform_x, "(", param, ")") else param
  y_lab <- if (transform_y != "identity")
    paste0(transform_y, "(", param2, ")") else param2
  param_labs <- labs(x = x_lab, y = y_lab)

  dat <- data.frame(
    x = if (transform_x == "identity")
      samps_use[,param] else t_x(samps_use[,param]),
    y = if (transform_y == "identity")
      samps_use[,param2] else t_y(samps_use[,param2]))
  if (!is.null(sp)) {
    dat$divergent <- c(sapply(sp, FUN = function(y) y[, "divergent__"]))
    dat$hit_max_td <- if (is.null(max_td)) 0 else
      c(sapply(sp, FUN = function(y) as.numeric(y[, "treedepth__"] == max_td)))
  } else {
    dat$divergent <- 0
    dat$hit_max_td <- 0
  }
  graph <- ggplot(dat, aes(x = x, y = y, xend=c(tail(x, n=-1), NA),
                           yend=c(tail(y, n=-1), NA)))

  if (lines == "hide") {
    graph <- graph + geom_point(alpha = pt_alpha, size = pt_size,
                                shape = shape_translator(pt_shape),
                                color = pt_color)
  } else { # if lines = "back" or "front"
    if (lines == "back") {
      graph <- graph +
        geom_path(alpha = lines_alpha, color = lines_color) +
        geom_point(alpha = pt_alpha, size = pt_size,
                   shape = shape_translator(pt_shape), color = pt_color)
    } else { # lines = "front"
      graph <- graph +
        geom_point(alpha = pt_alpha, size = pt_size,
                   shape = shape_translator(pt_shape), color = pt_color) +
        geom_path(alpha = lines_alpha, color = lines_color)
    }
  }
  if (ellipse_lev != "None")
    graph <- graph + stat_ellipse(level = as.numeric(ellipse_lev), color = ellipse_color,
                                  linetype = ellipse_lty, size = ellipse_lwd, alpha = ellipse_alpha)
  if (!all(dat$divergent == 0))
    graph <- graph + geom_point(data = subset(dat, divergent == 1), aes(x,y),
                                size = pt_size + 0.5, shape = 21,
                                color = "#570000", fill = "#ae0001")
  if (!all(dat$hit_max_td == 0))
    graph <- graph + geom_point(data = subset(dat, hit_max_td == 1), aes(x,y),
                                size = pt_size + 0.5, shape = 21,
                                color = "#5f4a13", fill = "#eeba30")
  graph + param_labs +
    theme_classic() %+replace% (no_lgnd + axis_labs + fat_axis + axis_color + transparent)
}

sidebarLayout(
  sidebarPanel(
    width = 3,
    style = "height: 550px;",
    br(),
    h4("Model Code"),
    helpText(
      style = "font-size: 12px;",
      p(
        "Model code will be displayed here each",
        "time you launch ShinyStan with this shinystan object."
      )
    ),
    br(),
    actionButton(
      "save_user_model_code",
      label = "Save code",
      icon = icon("save")
    ),
    div(style = "font-size: 11px;", textOutput("user_code_saved")),
    conditionalPanel(
      condition = "input.save_user_model_code > 0",
      br(),
      save_and_close_reminder("save_user_model_code_safe_quit")
    )
  ),
  mainPanel(
    width = 9,
    br(), br(),
    tags$textarea(
      id = "user_model_code",
      wrap = "off",
      cols = 80,
      rows = 20,
      .model_code
    )
  )
)

output$pp_rep_vs_obs_out_rstanarm <- renderPlot({
  overlay <- input$pp_rep_vs_obs_overlay_rstanarm == "density"
  if (overlay)
    print(PPC_plots[["pp_check_dens"]])
  else
    suppress_and_print(PPC_plots[["pp_check_hist"]])
}, bg = "transparent")

output$pp_hists_test_statistics_mean_out_rstanarm <- renderPlot({
  suppress_and_print(PPC_plots[["pp_check_stat_mean"]])
}, bg = "transparent")

output$pp_hists_test_statistics_sd_out_rstanarm <- renderPlot({
  suppress_and_print(PPC_plots[["pp_check_stat_sd"]])
}, bg = "transparent")

output$pp_hists_test_statistics_min_out_rstanarm <- renderPlot({
  suppress_and_print(PPC_plots[["pp_check_stat_min"]])
}, bg = "transparent")

output$pp_hists_test_statistics_max_out_rstanarm <- renderPlot({
  suppress_and_print(PPC_plots[["pp_check_stat_max"]])
}, bg = "transparent")

output$pp_y_vs_avg_rep_out_rstanarm <- renderPlot({
  print(PPC_plots[["pp_check_scatter"]])
}, bg = "transparent")

output$pp_hist_resids_out_rstanarm <- renderPlot({
  suppress_and_print(PPC_plots[["pp_check_resid"]])
}, bg = "transparent")

calc_height_autocorr_plot <- reactive({
  params <- input$ac_params
  params <- .update_params_with_groups(params, PARAM_NAMES)
  LL <- length(params)
  LL <- ifelse(LL < 8, 8, LL)
  round(60 * LL)
})

autocorr_plot <- reactive({
  validate(
    need(input$ac_lags, message = "Loading..."),
    need(!is.null(input$ac_warmup), message = "Loading...")
  )
  samps <- if (!input$ac_warmup)
    SAMPS_post_warmup else SAMPS_all
  params <- .update_params_with_groups(input$ac_params, PARAM_NAMES)
  if (!length(params))
    params <- dimnames(samps)$parameters[1] # default to first parameter
  params <- unique(params)
  samps <- samps[, , params, drop = FALSE]
  do.call(
    ".autocorr_plot",
    args = list(
      samps           = samps,
      lags            = input$ac_lags,
      flip            = input$ac_flip,
      combine_chains  = input$ac_combine,
      partial         = input$ac_partial
    )
  )
})

output$autocorr_plot_out <- renderPlot({
  autocorr_plot()
}, bg = "transparent")

# download the plot
output$download_autocorr <- downloadHandler(
  filename = paste0('shinystan-autocorr-gg.RData'),
  content = function(file) {
    shinystan_autocorr_gg <- autocorr_plot()
    save(shinystan_autocorr_gg, file = file)
})
output$save_pdf_autocorr = downloadHandler(
  filename = "shinstan-autocorr.pdf",
  content = function(file) {
    ggsave(file, plot = autocorr_plot(), device = pdf)
})

# shinystan is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 3 of the License, or (at your option) any later
# version.
# 
# shinystan is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# this program; if not, see <http://www.gnu.org/licenses/>.


#' Get summary statistics from shinystan object
#'
#' From a shinystan object get rhat, effective sample size, posterior
#' quantiles, means, standard deviations, sampler diagnostics, etc.
#' 
#' @export
#' @template args-sso
#' @param what What do you want to get? See Details, below.
#' @param ... Optional arguments, in particular \code{pars} to specify parameter
#'   names (by default all parameters will be used). For NUTS sampler parameters
#'   only (e.g. stepsize, treedepth) \code{inc_warmup} can also be specified to 
#'   include/exclude warmup iterations (the default is \code{FALSE}). See
#'   Details, below.
#'   
#' @details The argument \code{what} can take on the values below. 'Args:
#'   \code{arg}' means that \code{arg} can be specified in \code{...} for this
#'   value of \code{what}.
#' \describe{
#'   \item{\code{"rhat"}, \code{"Rhat"}, \code{"r_hat"}, or \code{"R_hat"}}{returns: Rhat statistics. Args: \code{pars}}
#'   \item{\code{"N_eff"}, \code{"n_eff"}, \code{"neff"}, \code{"Neff"}, \code{"ess"}, or \code{"ESS"}}{returns: Effective sample sizes. Args: \code{pars}}
#'   \item{\code{"mean"}}{returns: Posterior means. Args: \code{pars}}
#'   \item{\code{"sd"}}{returns: Posterior standard deviations. Args: \code{pars}}
#'   \item{\code{"se_mean"} or \code{"mcse"}}{returns: Monte Carlo standard error. Args: \code{pars}}
#'   \item{\code{"median"}}{returns: Posterior medians. Args: \code{pars}.}
#'   \item{\code{"quantiles"} or any string with \code{"quant"} in it (not case sensitive)}{returns: 2.5\%, 25\%, 50\%, 75\%, 97.5\% posterior quantiles. Args: \code{pars}.}
#'   \item{\code{"avg_accept_stat"} or any string with \code{"accept"} in it (not case sensitive)}{returns: Average value of "accept_stat" (which itself is the average acceptance probability over the NUTS subtree). Args: \code{inc_warmup}}
#'   \item{\code{"prop_divergent"} or any string with \code{"diverg"} in it (not case sensitive)}{returns: Proportion of divergent iterations for each chain. Args: \code{inc_warmup}}
#'   \item{\code{"max_treedepth"} or any string with \code{"tree"} or \code{"depth"} in it (not case sensitive)}{returns: Maximum treedepth for each chain. Args: \code{inc_warmup}}
#'   \item{\code{"avg_stepsize"} or any string with \code{"step"} in it (not case sensitive)}{returns: Average stepsize for each chain. Args: \code{inc_warmup}}
#' }
#' 
#' @note Sampler diagnostics (e.g. \code{"avg_accept_stat"}) only available for
#'   models originally fit using Stan.
#'   
#' @examples
#' # Using example shinystan object 'eight_schools'
#' sso <- eight_schools
#' retrieve(sso, "rhat")
#' retrieve(sso, "mean", pars = c('theta[1]', 'mu'))
#' retrieve(sso, "quantiles")
#' retrieve(sso, "max_treedepth")  # equivalent to retrieve(sso, "depth"), retrieve(sso, "tree"), etc.
#' retrieve(sso, "prop_divergent")
#' retrieve(sso, "prop_divergent", inc_warmup = TRUE)
#'
retrieve <- function(sso, what, ...) {
  sso_check(sso)
  .retrieve(sso, what, ...)
}


# retrieve helpers
.retrieve <- function(sso, what, ...) {
  if (what %in% c("rhat", "rhats", "Rhat", "Rhats", "r_hat", "R_hat"))
    return(retrieve_rhat(sso, ...))
  if (what %in% c("N_eff", "n_eff", "neff", "Neff", "ess", "ESS"))
    return(retrieve_neff(sso, ...))
  if (grepl_ic("mean", what))
    return(retrieve_mean(sso, ...))
  if (grepl_ic("sd", what))
    return(retrieve_sd(sso, ...))
  if (what %in% c("se_mean", "mcse"))
    return(retrieve_mcse(sso, ...))
  if (grepl_ic("quant", what))
    return(retrieve_quant(sso, ...))
  if (grepl_ic("median", what))
    return(retrieve_median(sso, ...))
  if (grepl_ic("tree", what) | grepl_ic("depth", what))
    return(retrieve_max_treedepth(sso, ...))
  if (grepl_ic("step", what))
    return(retrieve_avg_stepsize(sso, ...))
  if (grepl_ic("diverg", what))
    return(retrieve_prop_divergent(sso, ...))
  if (grepl_ic("accept", what))
    return(retrieve_avg_accept(sso, ...))
}


retrieve_rhat <- function(sso, pars) {
  if (missing(pars))
    return(slot(sso, "summary")[, "Rhat"])
  slot(sso, "summary")[pars, "Rhat"]
}

retrieve_neff <- function(sso, pars) {
  if (missing(pars))
    return(slot(sso, "summary")[, "n_eff"])
  slot(sso, "summary")[pars, "n_eff"]
}

retrieve_mcse <- function(sso, pars) {
  if (missing(pars))
    return(slot(sso, "summary")[, "se_mean"])
  slot(sso, "summary")[pars, "se_mean"]
}

retrieve_quant <- function(sso, pars) {
  cols <- paste0(100 * c(0.025, 0.25, 0.5, 0.75, 0.975), "%")
  if (missing(pars))
    return(slot(sso, "summary")[, cols])
  slot(sso, "summary")[pars, cols]
}

retrieve_median <- function(sso, pars) {
  if (missing(pars))
    return(retrieve_quant(sso)[, "50%"])
  retrieve_quant(sso, pars)[, "50%"]
}

retrieve_mean <- function(sso, pars) {
  if (missing(pars))
    return(slot(sso, "summary")[, "mean"])
  slot(sso, "summary")[pars, "mean"]
}

retrieve_sd <- function(sso, pars) {
  if (missing(pars))
    return(slot(sso, "summary")[, "sd"])
  slot(sso, "summary")[pars, "sd"]
}


.sp_check <- function(sso) {
  if (identical(slot(sso, "sampler_params"), list(NA)))
    stop("No sampler parameters found", call. = FALSE)
}

.which_rows <- function(sso, inc_warmup) {
  if (inc_warmup) {
    seq_len(slot(sso, "n_iter"))
  } else {
    seq(from = 1 + slot(sso, "n_warmup"),  to = slot(sso, "n_iter"))
  }
}

retrieve_max_treedepth <- function(sso, inc_warmup = FALSE) {
  .sp_check(sso)
  rows <- .which_rows(sso, inc_warmup)
  max_td <- sapply(slot(sso, "sampler_params"), function(x) 
    max(x[rows, "treedepth__"]))
  names(max_td) <- paste0("chain", 1:length(max_td))
  max_td
}

retrieve_prop_divergent <- function(sso, inc_warmup = FALSE) {
  .sp_check(sso)
  rows <- .which_rows(sso, inc_warmup)
  prop_div <- sapply(slot(sso, "sampler_params"), function(x) 
    mean(x[rows, "divergent__"]))
  names(prop_div) <- paste0("chain", 1:length(prop_div))
  prop_div
}

retrieve_avg_stepsize <- function(sso, inc_warmup = FALSE) {
  .sp_check(sso)
  rows <- .which_rows(sso, inc_warmup)
  avg_ss <- sapply(slot(sso, "sampler_params"), function(x) 
    mean(x[rows, "stepsize__"]))
  names(avg_ss) <- paste0("chain", 1:length(avg_ss))
  avg_ss
}

retrieve_avg_accept <- function(sso, inc_warmup = FALSE) {
  .sp_check(sso)
  rows <- .which_rows(sso, inc_warmup)
  avg_accept <- sapply(slot(sso, "sampler_params"), function(x) 
    mean(x[rows, "accept_stat__"]))
  names(avg_accept) <- paste0("chain", 1:length(avg_accept))
  avg_accept
}

# param_summary -----------------------------------------------------------
# summary stats for a single parameter
.param_summary <- function(param, summary) {
  stats <-  c("Rhat", "n_eff", "mean", "sd", "2.5%", "50%", "97.5%")
  out <- summary[param, stats]
  out["n_eff"] <- round(out["n_eff"])
  outmat <- matrix(out, 1, length(out))
  colnames(outmat) <- names(out)
  rownames(outmat) <- NULL
  outmat
}


# all_summary -------------------------------------------------------------
# summary stats for all parameters
.all_summary <- function(summary, digits = 2, cols) {
  if (missing(cols))
    cols <- seq_len(ncol(summary))
  df <- as.data.frame(summary[, cols])
  df <- round(df, digits)
  if ("n_eff" %in% cols) 
    df[, "n_eff"] <- round(df[, "n_eff"])
  df
}

# tex_summary -------------------------------------------------------------
# prep for latex table
.tex_summary <- function(summary, params, cols) {
  df <- as.data.frame(summary[, cols])
  if ("n_eff" %in% cols) 
    df[, "n_eff"] <- round(df[, "n_eff"])
  cbind(Parameter = rownames(df), df)
}

# sampler_summary ---------------------------------------------------------
.sampler_stuff <- function(X, param, report) {
  sapply_funs <- function(x, fun_name) {
    funs <- list(
      maxf = function(x) max(x[, param]),
      minf = function(x) min(x[, param]),
      meanf = function(x) mean(x[, param]),
      sdf = function(x) sd(x[, param])
    )
    sapply(x, FUN = funs[[fun_name]])
  }
  out <- if (report == "maximum") sapply_funs(X, "maxf") 
    else if (report == "minimum") sapply_funs(X, "minf")
    else if (report == "sd") sapply_funs(X, "sdf")
    else sapply_funs(X, "meanf")
  
  names(out) <- paste0("chain",1:length(out))
  out
}

# summary statistics for algorithm=NUTS or algorithm=HMC sampler parameters
.sampler_summary <- function(sampler_params, warmup_val,
                             report = "average", digits = 4){ 
  
  params <- colnames(sampler_params[[1]])
  out <- sapply(params, FUN = function(p) 
    .sampler_stuff(X = sampler_params, param = p, report = report))
  
  if (length(dim(out)) > 1) { # if multiple chains
    out <- rbind("All chains" = colMeans(out), out)
    colnames(out) <- gsub("__","",colnames(out))
    out <- formatC(round(out, digits), format = 'f', digits = digits)
  } else { # if only 1 chain
    names(out) <- gsub("__.chain1", "", names(out))
    out <- round(t(out), digits)
  }
  out
}

div(
  class = "help-glossary-div",
  withMathJax(),
  br(),br(),
  div(
    withMathJax(),
    class = "help-glossary-nav-container",
    navlistPanel(
      well = TRUE,
      id = "glossary_navlist",
      tabPanel(
        "Effective sample size",
        includeHTML("html/neff.html")
      ),
      tabPanel(
        "Monte Carlo uncertainty",
        includeHTML("html/mcse.html")
      ),
      tabPanel(
        "Rhat",
        includeHTML("html/rhat.html")
      ),
      tabPanel(
        "No-U-Turn Sampler (NUTS)",
        includeHTML("html/nuts.html")
      ),
      tabPanel(
        "accept_stat",
        includeHTML("html/accept_stat.html")
      ),
      tabPanel(
        "divergent",
        withMathJax(),
        includeHTML("html/ndivergent.html")
      ),
      tabPanel(
        "energy",
        withMathJax(),
        includeHTML("html/energy.html")
      ),
      tabPanel(
        "stepsize",
        includeHTML("html/stepsize.html")
      ),
      tabPanel(
        "n_leapfrog",
        includeHTML("html/nleapfrog.html")
      ),
      tabPanel(
        "treedepth",
        includeHTML("html/treedepth.html")
      )
    )
  ),
  br(),br()
)

output$ui_trivariate_select_x <- renderUI({
  selectizeInput(
    "trivariate_param_x",
    label = strong_bl("x-axis"),
    choices = .make_param_list(object),
    selected = input$param,
    multiple = FALSE
  )
})

### Functions in this file ###
# shinystan_monitor
# ess_rfun
# split_rhat_rfun



# shinystan_monitor -------------------------------------------------------
# Slightly modified version of monitor from RStan package
shinystan_monitor <- function(sims,
                              warmup = floor(dim(sims)[1]/2),
                              probs = c(0.025, 0.25, 0.5, 0.75, 0.975),
                              digits_summary = 1) {
  dim_sims <- dim(sims)
  dimnames_sims <- dimnames(sims)
  parnames <- dimnames_sims[[3]]
  if (length(dim_sims) != 3)
    stop("'sims' is not a 3-d array")
  if (warmup > dim_sims[1])
    stop("warmup is larger than the total number of iterations")
  num_par <- dim_sims[3]
  if (is.null(parnames))
    parnames <- paste0("V", 1:num_par)
  sims_wow <- if (warmup >= 1)
    apply(sims, c(2, 3), FUN = function(x) x[-(1:warmup)])
  else sims
  m <- apply(sims_wow, 3, mean)
  sd <- sapply(1:num_par, FUN = function(i) sd(as.vector(sims_wow[, , i])))
  quan <- lapply(1:num_par, FUN = function(i) quantile(sims_wow[, , i], probs = probs))
  probs_str <- names(quan[[1]])
  quan <- do.call(rbind, quan)
  rhat <- sapply(1:num_par, FUN = function(i) split_rhat_rfun(sims_wow[, , i]))
  ess <- sapply(1:num_par, FUN = function(i) ess_rfun(sims_wow[, , i]))
  sem <- sd/sqrt(ess)
  summary <- cbind(m, sem, sd, quan, ess, rhat)
  colnames(summary) <- c("mean", "se_mean", "sd", probs_str, "n_eff", "Rhat")
  rownames(summary) <- parnames
  invisible(summary)
}


# From RStan package
# ess_rfun ----------------------------------------------------------------
ess_rfun <- function (sims) {
  if (is.vector(sims))
    dim(sims) <- c(length(sims), 1)
  chains <- ncol(sims)
  n_samples <- nrow(sims)
  acov <- lapply(1:chains, FUN = function(i) {
    cov <- acf(sims[, i], lag.max = n_samples - 1, plot = FALSE,
               type = c("covariance"))
    cov$acf[, , 1]
  })
  acov <- do.call(cbind, acov)
  chain_mean <- apply(sims, 2, mean)
  mean_var <- mean(acov[1, ]) * n_samples/(n_samples - 1)
  var_plus <- mean_var * (n_samples - 1)/n_samples
  if (chains > 1)
    var_plus <- var_plus + var(chain_mean)
  rho_hat_sum <- 0
  for (t in 2:nrow(acov)) {
    rho_hat <- 1 - (mean_var - mean(acov[t, ]))/var_plus
    if (is.nan(rho_hat))
      rho_hat <- 0
    if (rho_hat < 0)
      break
    rho_hat_sum <- rho_hat_sum + rho_hat
  }
  ess <- chains * n_samples
  if (rho_hat_sum > 0)
    ess <- ess/(1 + 2 * rho_hat_sum)
  ess
}


# From RStan package
# split_rhat_rfun ---------------------------------------------------------
split_rhat_rfun <- function (sims)  {
  if (is.vector(sims))
    dim(sims) <- c(length(sims), 1)
  chains <- ncol(sims)
  n_samples <- nrow(sims)
  half_n <- floor(n_samples/2)
  idx_2nd <- n_samples - half_n + 1
  split_chain_mean <- numeric(chains * 2)
  split_chain_var <- numeric(chains * 2)
  for (i in 1:chains) {
    split_chain_mean[i] <- mean(sims[1:half_n, i])
    split_chain_var[i] <- var(sims[1:half_n, i])
    split_chain_mean[chains + i] <- mean(sims[idx_2nd:n_samples,
                                              i])
    split_chain_var[chains + i] <- var(sims[idx_2nd:n_samples,
                                            i])
  }
  var_between <- half_n * var(split_chain_mean)
  var_within <- mean(split_chain_var)
  sqrt((var_between/var_within + half_n - 1)/half_n)
}

mainPanel(
  width = 8,
  actionButton(
    "tex_go",
    withMathJax("Update \\(\\LaTeX\\)"),
    icon = icon("print", lib = "glyphicon")
  ),
  br(), br(),
  verbatimTextOutput("summary_stats_latex_out")
)

# ggplot theme elements to be used as needed  


# transparent background --------------------------------------------------
transparent <- theme(
  panel.background = element_blank(), 
  plot.background = element_blank()
  )


# axes and titles ---------------------------------------------------------
axis_line_color <- "gray20"
axis_color <- theme(
  axis.line = element_line(color = axis_line_color)
  )
axis_labs <- theme(
  axis.title = element_text(face = "bold", size = 13)
  )
title_txt <- theme(
  plot.title = element_text(face = "bold", size = 14)
  )
fat_axis <- theme(
  axis.line.x = element_line(linewidth = 3, color = axis_line_color), 
  axis.line.y = element_line(linewidth = 0.5, color = axis_line_color)
  )
no_yaxs <- theme(
  axis.line.y = element_blank(),
  axis.ticks.y = element_blank(),
  axis.text.y = element_blank()
  )

# facet strips ------------------------------------------------------------
strip_txt <- theme(
  strip.text = element_text(size = 12, face = "bold", color = "white"),
  strip.background = element_rect(color = axis_line_color, fill = axis_line_color)
  )


# horizontal and vertical lines -------------------------------------------
h_lines <- theme(
  panel.grid.major = element_line(linewidth = 0.10, linetype = 3, color = "turquoise4"),
  panel.grid.major.x = element_blank()
  )
v_lines <- theme(
  panel.grid.major = element_line(linewidth = 0.25, linetype = 3, color = "turquoise4"),
  panel.grid.major.y = element_blank()
  )


# legends -----------------------------------------------------------------
no_lgnd <- theme(
  legend.position = "none"
  )
lgnd_bot <- theme(
  legend.position = "bottom", 
  legend.background = element_blank()
  )
lgnd_top <- theme(
  legend.position = "top", 
  legend.background = element_blank()
  )
lgnd_left <- theme(
  legend.position = "left", 
  legend.background = element_blank()
  )
lgnd_right <- theme(
  legend.position = "right", 
  legend.background = element_blank()
  )


# colors and shapes -------------------------------------------------------
base_fill <- "#66a7e0" 
overlay_fill <- "#006dcc" 
vline_base_clr <- "#006dcc"
pt_outline_clr <- "#328ad6"
divergent_fill <- "#ae0001" 
hit_max_td_fill <- "#eeba30"
divergent_clr <-  "black" 
hit_max_td_clr <- "black" 
div_and_hit_shape <- 21

blue_color <- "#006dcc"

source_ui <- function(...) {
  source(
    file.path("ui_files", ...),
    local = TRUE
  )$value
}

save_and_close_button <- function() {
  tags$button(
    id = 'save_and_close_button',
    type = "button",
    class = "btn action-button",
    onclick = "window.close();",
    "Save & Close"
  )
}

shinystan_version <- function() {
  # prevents error when deployed to shinyapps.io
  ver <- try(utils::packageVersion("shinystan"))
  if (inherits(ver, "try-error")) {
    return()
  } else {
    strong(paste("Version", ver))
  }
}

logo_and_name <- function() {
  div(
    div(
      img(
        src = "wide_ensemble.png",
        class = "wide-ensemble",
        width = "100%"
      )
    ),
    div(
      style = "margin-top: 25px",
      img(src = "stan_logo.png", class = "stan-logo"),
      div(id = "shinystan-title", "ShinyStan")
    )
  )
}


# save and close reminder -------------------------------------------------
save_and_close_reminder <- function(id) {
  helpText(
    id = id,
    p(
      "To make sure the changes aren't lost, use the",
      span(class = "save-close-reminder", "Save & Close"),
      "button in the top left corner to exit the app before",
      "closing the browser window."
    )
  )
}


# show/hide options/glossary ---------------------------------------------
a_options <- function(name) {
  lab <- if (name == "table") {
    "Table Options"
  } else {
    "Show/Hide Options"
  }
  div(
    class = "aoptions",
    checkboxInput(
      inputId = paste0(name, "_options_show"),
      label = strong(style = "margin-top: 20px; color: #222222;", lab),
      value = FALSE
    )
  )
}
a_glossary <- function(id) {
  div(
    class = "aoptions",
    actionLink(
      inputId = id,
      label = strong(style = "margin-top: 20px; color: #222222;", "Glossary"),
      icon = icon("book", lib = "glyphicon")
    )
  )
}


# plotOutput generators ---------------------------------------------------
dygraphOutput_175px <- function(id) {
  dygraphs::dygraphOutput(id, height = "175px")
}
plotOutput_200px <- function(id, ...) {
  plotOutput(id, height = "200px")
}
plotOutput_400px <- function(id, ...) {
  plotOutput(id, height = "400px")
}


# conditionalPanel generator for EXPLORE/density  -------------------------
condPanel_dens_together <- function(...) {
  conditionalPanel(condition = "input.dens_chain_split == 'Together'", ...)
}
condPanel_dens_prior <- function(dist, ...) {
  cond <- paste0("input.dens_prior ==", "'", dist, "'")
  conditionalPanel(cond, ...)
}


# conditional transparency settings ---------------------------------------
alpha_calc_pt <- function(N) {
  if (N <= 100) {
    return(1)
  } else if (N <= 200) {
    return(0.75)
  } else if (N >= 1500) {
    return(0.15)
  } else {
    1 - pnorm(N / 1500)
  }
}

alpha_calc_lines <- function(N) {
  if (N < 50) {
    return(0.5)
  }
  if (N < 500) {
    return(0.4)
  }
  if (N < 1000) {
    return(0.3)
  }
  if (N < 5000) {
    return(0.2)
  } else {
    return(0.1)
  }
}


# transformations ---------------------------------------------------------
transformation_selectInput <- function(id) {
  selectInput(
    id,
    label = NULL,
    choices = transformation_choices,
    selected = "identity"
  )
}

transform_helpText <- function(var = "x") {
  div(
    if (var == "x") {
      helpText(
        style = "font-size: 13px;",
        "To apply a transformation",
        "select a function and click",
        code("Transform")
      )
    } else if (var == "x,y") {
      helpText(
        style = "font-size: 13px;",
        "To apply transformations",
        "select a function for x and/or y",
        "and click",
        code("Transform")
      )
    } else {
      helpText(
        style = "font-size: 13px;",
        "To apply transformations",
        "select a function for x, y, and/or z",
        "and click",
        code("Transform")
      )
    }
  )
}


# diagnostics help text ---------------------------------------------------
hT11 <- function(...) {
  helpText(style = "font-size: 11px;", ...)
}
help_interval <- hT11("Highlighted interval shows \\(\\bar{x} \\pm sd(x)\\)")
help_lines <- hT11("Lines are mean (solid) and median (dashed)")
help_max_td <- hT11("Horizontal line indicates the max_treedepth setting")
help_points <- hT11(
  "Large red points indicate which (if any) iterations",
  "encountered a divergent transition. Yellow indicates",
  "a transition hitting the maximum treedepth."
)
help_dynamic <- hT11(
  "Use your mouse to select a range in the traceplot to zoom into. ",
  "The other plots on the screen will update accordingly. ",
  "Double-click to reset."
)


# ppcheck plot descriptions ----------------------------------------------
plot_descriptions <-
  c(
    plot_hists_rep_vs_obs = "Distributions of observed data and a random sample of replications",
    plot_dens_rep_vs_obs = "Density estimate of observed data (blue) and a random sample of replications",
    plot_obs_vs_avg_y_rep = "Observations vs average simulated value",
    plot_hist_resids = "Residuals",
    plot_avg_rep_vs_avg_resid_rep = "Average simulated value vs average residual",
    plot_test_statistics = "Distributions of test statistics \\(T(y^{rep})\\)"
  )


# stan manual reference ---------------------------------------------------
stan_manual <- function() {
  helpText(
    style = "font-size: 12px;",
    "Glossary entries are compiled (with minor edits) from various excerpts of the",
    a(
      "Stan Modeling Language User's Guide and Reference Manual",
      href = "http://mc-stan.org/documentation/"
    ),
    "(",
    a(href = "http://creativecommons.org/licenses/by/3.0/", "CC BY (v3)"),
    ")"
  )
}


# objects to use in ui.R and ui_files -------------------------------------
if (!exists(".SHINYSTAN_OBJECT")) {
  .SHINYSTAN_OBJECT <- shinystan:::.sso_env[[".SHINYSTAN_OBJECT"]]
}
.model_name <- slot(.SHINYSTAN_OBJECT, "model_name")
.param_names <- slot(.SHINYSTAN_OBJECT, "param_names")
.param_list <- .make_param_list(.SHINYSTAN_OBJECT)
.param_list_with_groups <- .make_param_list_with_groups(.SHINYSTAN_OBJECT)
.nChains <- slot(.SHINYSTAN_OBJECT, "n_chain")
.nIter <- slot(.SHINYSTAN_OBJECT, "n_iter")
.nWarmup <- slot(.SHINYSTAN_OBJECT, "n_warmup")
.model_code <- slot(.SHINYSTAN_OBJECT, "model_code")
.notes <- slot(.SHINYSTAN_OBJECT, "user_model_info")
.has_rstanarm_ppcs <-
  isTRUE(.SHINYSTAN_OBJECT@misc$stanreg) &&
  !is.null(.SHINYSTAN_OBJECT@misc$pp_check_plots)

if (exists("object")) {
  rm(object)
}
if (exists(".SHINYSTAN_OBJECT")) {
  rm(.SHINYSTAN_OBJECT)
}
gc()

# reactive function to get samples for a single parameter
par_samps_all <- reactive({
  param <- input$param
  p <- which(PARAM_NAMES == param)
  SAMPS_all[, , p]
})
par_samps_post_warmup <- reactive({
  param <- input$param
  p <- which(PARAM_NAMES == param)
  SAMPS_post_warmup[, , p]
})

# bivariate scatterplot
bivariate_transform_x <-
  eventReactive(input$bivariate_transform_go > 0, input$bivariate_transform_x)
bivariate_transform_y <-
  eventReactive(input$bivariate_transform_go > 0, input$bivariate_transform_y)
bivariate_plot <- reactive({
  validate(
    need(input$param, message = FALSE),
    need(input$bivariate_ellipse_lev, message = FALSE),
    need(input$bivariate_param_y, message = FALSE)
  )
  
  if (!is.null(input$bivariate_ellipse_lev)) {
    validate(
      need(is.numeric(input$bivariate_pt_size), message = "Point size must be numeric"),
      need(is.numeric(input$bivariate_pt_shape), message = "Point shape must be numeric")
    )
    
    if (input$bivariate_ellipse_lev != "None") {
      validate(
        need(
          input$param != input$bivariate_param_y,
          "For this option the x and y can't be the same parameter."
        ),
        need(
          is.numeric(input$bivariate_ellipse_lwd),
          message = "Ellipse size must be numeric"
        ),
        need(
          is.numeric(input$bivariate_ellipse_lty),
          message = "Ellipse shape must be numeric"
        )
      )
    }
  }
  
  do.call(
    ".bivariate_plot",
    args = list(
      samps = SAMPS_post_warmup,
      sp = if (!identical(SAMPLER_PARAMS_post_warmup, FALSE) && STAN_ALGORITHM == "NUTS")
        SAMPLER_PARAMS_post_warmup else NULL,
      max_td = if ("max_td" %in% names(MISC)) MISC$max_td else NULL,
      param = input$param,
      param2 = input$bivariate_param_y,
      pt_alpha = input$bivariate_pt_alpha,
      pt_size = input$bivariate_pt_size,
      pt_shape = input$bivariate_pt_shape,
      pt_color = input$bivariate_pt_color,
      ellipse_lev = input$bivariate_ellipse_lev,
      ellipse_color = input$bivariate_ellipse_color,
      ellipse_lty = input$bivariate_ellipse_lty,
      ellipse_lwd = input$bivariate_ellipse_lwd,
      ellipse_alpha = input$bivariate_ellipse_alpha,
      lines  = input$bivariate_lines,
      lines_color = input$bivariate_lines_color,
      lines_alpha = input$bivariate_lines_alpha,
      transform_x = bivariate_transform_x(),
      transform_y = bivariate_transform_y()
    )
  )
})

output$bivariate_plot_out <- renderPlot({
  x <- bivariate_plot()
  suppressWarnings(print(x))
}, bg = "transparent")

# download
output$download_bivariate <- downloadHandler(
  filename = 'shinystan-bivariate-gg.RData',
  content = function(file) {
    shinystan_bivariate_gg <- bivariate_plot()
    save(shinystan_bivariate_gg, file = file)
  }
)
output$save_pdf_bivariate = downloadHandler(
  filename = "shinstan-bivariate.pdf",
  content = function(file) {
    ggsave(file, plot = bivariate_plot(), device = pdf)
  }
)

pp_hist_resids <- reactive({
  pp_tests()
  s <- sample_id_for_resids()
  resids <- get_y() - get_yrep()[s,]
  names(resids) <- paste0("resids(yrep_", s, ")")
  do.call(".pp_hist_resids", args = list(resids = resids))
})

output$pp_hist_resids_out <- renderPlot({
  x <- suppressMessages(pp_hist_resids())
  suppress_and_print(x)
}, bg = "transparent")

# shinystan is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 3 of the License, or (at your option) any later
# version.
# 
# shinystan is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# this program; if not, see <http://www.gnu.org/licenses/>.


#' Drop parameters from a \code{shinystan} object
#' 
#' Remove selected parameters from a \code{shinystan} object. This is useful if 
#' you have a very large \code{shinystan} object when you only want to look at a
#' subset of parameters. With a smaller \code{shinystan} object, 
#' \code{\link{launch_shinystan}} will be faster and you should experience 
#' better performance (responsiveness) after launching when using the
#' 'ShinyStan' app.
#'
#' @export
#' @template args-sso
#' @param pars A character vector of parameter names. If the name of a 
#'   non-scalar (e.g. vector, matrix) parameter is included in \code{pars} all 
#'   of its elements will be removed. Currently it is not possible to remove 
#'   only a subset of the elements of a non-scalar parameter.
#' @return \code{sso}, with \code{pars} dropped.
#' 
#' @template seealso-generate_quantity
#' 
#' @examples 
#' # Using example shinystan object 'eight_schools'
#' print(eight_schools@param_names)
#' 
#' # Remove the scalar parameters mu and tau
#' sso <- drop_parameters(eight_schools, pars = c("mu", "tau"))
#' print(sso@param_names)
#' 
#' # Remove all elements of the parameter vector theta
#' sso <- drop_parameters(sso, pars = "theta")
#' print(sso@param_names)
#' 
drop_parameters <- function(sso, pars) {
  sso_check(sso)
  stopifnot(is.character(pars))
  if (any(c("log-posterior", "lp__") %in% pars))
    stop("log-posterior can't be dropped.")
  
  any_indiv_els <- any(grepl("[", pars, fixed = TRUE))
  if (any_indiv_els)
    stop("Currently, individual elements of non-scalar parameters can't be removed.")
  
  any_dimnames_in_pars <- any(names(sso@param_dims) %in% pars)
  if (any_dimnames_in_pars) {
    param_dims <- slot(sso, "param_dims")
    param_names <- slot(sso, "param_names")
    pd <- which(names(param_dims) %in% pars)
    nms <- names(param_dims[pd])
    for (j in seq_along(nms)) {
      if (!nms[j] %in% param_names) {
        pars <- pars[pars != nms[j]]
        tmp <- grep(paste0(nms[j], "["), param_names, fixed = TRUE, value = TRUE)
        pars <- c(pars, tmp)
      }
    }
    slot(sso, "param_dims") <- slot(sso, "param_dims")[-pd]
  }
  
  sel <- match(pars, slot(sso, "param_names"))
  if (!any_dimnames_in_pars && all(is.na(sel))) {
    stop("No matches for 'pars' were found.", call. = FALSE)
  } else if (any(is.na(sel))) {
    warning(paste(
      "Some 'pars' not found and ignored:",
      paste(pars[is.na(sel)], collapse = ", ")
    ))
  }
  
  .drop_parameters(sso, na.omit(sel))
}


# @param rmv A vector of indices indicating the positions of parameters to be
#   removed
.drop_parameters <- function(sso, rmv) {
  slot(sso, "param_names") <- slot(sso, "param_names")[-rmv]
  slot(sso, "posterior_sample") <- slot(sso, "posterior_sample")[, , -rmv, drop = FALSE]
  slot(sso, "summary") <- slot(sso, "summary")[-rmv, , drop = FALSE]
  sso
}

div(id = "diagnostics_customize",
    wellPanel(
      fluidRow(
        column(width = 3, h4(textOutput("diagnostic_chain_text"))),
        column(width = 4, h5("Parameter")),
        column(width = 4, h5("Transformation"))
      ),
      fluidRow(
        column(
          width = 3, div(style = "width: 100px;",
          numericInput(
            "diagnostic_chain",
            label = NULL,
            value = 0,
            min = 0,
            # don't allow changing chains if only 1 chain
            max = ifelse(.nChains == 1, 0, .nChains)
          )
        )),
        column(
          width = 4,
          selectizeInput(
            inputId = "diagnostic_param",
            label = NULL,
            multiple = FALSE,
            choices = .param_list,
            selected = .param_list[1]
          )
        ),
        column(
          width = 3,
          transformation_selectInput("diagnostic_param_transform")
        ),
        column(
          width = 2,
          actionButton("diagnostic_param_transform_go", "Transform", class = "transform-go")
        )
      ),
      helpText(strong(
        style = "color: red; font-size: 13px;",
        textOutput("diagnostics_warnings_text")
      ))
    ))

shinyjs::hidden(div(
  id = "hist_options",
  wellPanel(
    class = "optionswell",
    hr(class = "hroptions"),
    strongBig("Transformation"),
    transform_helpText("x"),
    fluidRow(
      column(width = 4, transformation_selectInput("hist_transform_x")),
      column(
        width = 2,
        actionButton("hist_transform_x_go", label = "Transform", class = "transform-go")
      )
    ),
    hr(class = "hroptions"),
    fluidRow(
      column(
        width = 2,
        numericInput(
          "hist_chain",
          label = strongMed("Chain"),
          min = 0,
          max = .nChains,
          step = 1,
          value = 0
        )
      ),
      column(
        width = 4,
        sliderInput(
          "hist_binwd",
          label =  strongMed("Binwidth (0 = default)"),
          min = 0,
          value = 0,
          max = 50,
          step = 0.05,
          ticks = FALSE
        )
      ),
      column(
        width = 3,
        colourpicker::colourInput("hist_fill_color", strongMed("Fill"), base_fill)
      ),
      column(
        width = 3,
        colourpicker::colourInput("hist_line_color", strongMed("Line"), vline_base_clr)
      )
    )
  )
))

source_ui <- function(...) {
  source(
    file.path("ui_files", ...),
    local = TRUE
  )$value
}

save_and_close_button <- function() {
  tags$button(
    id = 'save_and_close_button',
    type = "button",
    class = "btn action-button",
    onclick = "window.close();",
    "Save & Close"
  )
}

shinystan_version <- function() {
  # prevents error when deployed to shinyapps.io
  ver <- try(utils::packageVersion("shinystan"))
  if (inherits(ver, "try-error"))
    return()
  else
    strong(paste("Version", ver))
}

logo_and_name <- function() {
  div(div(
    img(
      src = "wide_ensemble.png",
      class = "wide-ensemble",
      width = "100%"
    )
  ),
  div(
    style = "margin-top: 25px",
    img(src = "stan_logo.png", class = "stan-logo"),
    div(id = "shinystan-title", "ShinyStan")
  ))
}


# save and close reminder -------------------------------------------------
save_and_close_reminder <- function(id) {
  helpText(
    id = id,
    p(
      "To make sure the changes aren't lost, use the",
      span(class = "save-close-reminder", "Save & Close"),
      "button in the top left corner to exit the app before",
      "closing the browser window."
    )
  )
}


# show/hide options/glossary ---------------------------------------------
a_options <- function(name) {
  lab <- if (name == "table")
    "Table Options" else "Show/Hide Options"
  div(class = "aoptions",
      checkboxInput(
        inputId = paste0(name, "_options_show"),
        label = strong(style = "margin-top: 20px; color: #222222;", lab),
        value = FALSE
      ))
}
a_glossary <- function(id) {
  div(class = "aoptions",
      actionLink(
        inputId = id,
        label = strong(style = "margin-top: 20px; color: #222222;", "Glossary"),
        icon = icon("book", lib = "glyphicon")
      ))
}



# plotOutput generators ---------------------------------------------------
dygraphOutput_175px <- function(id)
  dygraphs::dygraphOutput(id, height = "175px")
plotOutput_200px <- function(id, ...)
  plotOutput(id, height = "200px")
plotOutput_400px <- function(id, ...)
  plotOutput(id, height = "400px")



# conditionalPanel generator for EXPLORE/density  -------------------------
condPanel_dens_together <- function(...) {
  conditionalPanel(condition = "input.dens_chain_split == 'Together'", ...)
}
condPanel_dens_prior <- function(dist, ...) {
  cond <- paste0("input.dens_prior ==","'", dist,"'")
  conditionalPanel(cond, ...)
}


# conditional transparency settings ---------------------------------------
alpha_calc_pt <- function(N) {
  if (N <= 100) return(1)
  else if (N <= 200) return(0.75)
  else if (N >= 1500) return(0.15)
  else 1 - pnorm(N/1500)
}

alpha_calc_lines <- function(N) {
  if (N < 50) return(0.5)
  if (N < 500) return(0.4)
  if (N < 1000) return(0.3)
  if (N < 5000) return(0.2)
  else return(0.1)
}


# transformations ---------------------------------------------------------
transformation_selectInput <- function(id) {
  selectInput(
    id,
    label = NULL,
    choices = transformation_choices,
    selected = "identity"
  )
}

transform_helpText <- function(var = "x") {
  div(
    if (var == "x")
      helpText(style = "font-size: 13px;",
               "To apply a transformation",
               "select a function and click",
               code("Transform"))
    else if (var == "x,y")
      helpText(style = "font-size: 13px;",
               "To apply transformations",
               "select a function for x and/or y",
               "and click", code("Transform"))
    else
      helpText(style = "font-size: 13px;",
               "To apply transformations",
               "select a function for x, y, and/or z",
               "and click", code("Transform"))
  )
}


# diagnostics help text ---------------------------------------------------
hT11 <- function(...)
  helpText(style = "font-size: 11px;", ...)
help_interval <- hT11("Highlighted interval shows \\(\\bar{x} \\pm sd(x)\\)")
help_lines <- hT11("Lines are mean (solid) and median (dashed)")
help_max_td <- hT11("Horizontal line indicates the max_treedepth setting")
help_points <- hT11(
  "Large red points indicate which (if any) iterations",
  "encountered a divergent transition. Yellow indicates",
  "a transition hitting the maximum treedepth."
)
help_dynamic <- hT11(
  "Use your mouse to select a range in the traceplot to zoom into. ",
  "The other plots on the screen will update accordingly. ",
  "Double-click to reset."
)



# ppcheck plot descriptions ----------------------------------------------
plot_descriptions <-
  c(
    plot_hists_rep_vs_obs = "Distributions of observed data and a random sample of replications",
    plot_dens_rep_vs_obs = "Density estimate of observed data (blue) and a random sample of replications",
    plot_obs_vs_avg_y_rep = "Observations vs average simulated value",
    plot_hist_resids = "Residuals",
    plot_avg_rep_vs_avg_resid_rep = "Average simulated value vs average residual",
    plot_test_statistics = "Distributions of test statistics \\(T(y^{rep})\\)"
  )



# stan manual reference ---------------------------------------------------
stan_manual <- function() {
  helpText(
    style = "font-size: 12px;",
    "Glossary entries are compiled (with minor edits) from various excerpts of the",
    a(
      "Stan Modeling Language User's Guide and Reference Manual",
      href = "http://mc-stan.org/documentation/"
    ),
    "(",
    a(href = "http://creativecommons.org/licenses/by/3.0/", "CC BY (v3)"),
    ")"
  )
}


# objects to use in ui.R and ui_files -------------------------------------
if (!exists(".SHINYSTAN_OBJECT")) {
  .SHINYSTAN_OBJECT <- shinystan:::.sso_env[[".SHINYSTAN_OBJECT"]]
}
.model_name <- slot(.SHINYSTAN_OBJECT, "model_name")
.param_names <- slot(.SHINYSTAN_OBJECT, "param_names")
.param_list <- .make_param_list(.SHINYSTAN_OBJECT)
.param_list_with_groups <- .make_param_list_with_groups(.SHINYSTAN_OBJECT)
.nChains <- slot(.SHINYSTAN_OBJECT, "n_chain")
.nIter <- slot(.SHINYSTAN_OBJECT, "n_iter")
.nWarmup <- slot(.SHINYSTAN_OBJECT, "n_warmup")
.model_code <- slot(.SHINYSTAN_OBJECT, "model_code")
.notes <- slot(.SHINYSTAN_OBJECT, "user_model_info")
.has_rstanarm_ppcs <-
  isTRUE(.SHINYSTAN_OBJECT@misc$stanreg) &&
  !is.null(.SHINYSTAN_OBJECT@misc$pp_check_plots)

if (exists("object"))
  rm(object)
if (exists(".SHINYSTAN_OBJECT"))
  rm(.SHINYSTAN_OBJECT)
gc()

navlistPanel(
  well = FALSE,
  widths = c(2, 10),
  id = "diagnostics_help_navlist",
  tabPanel(
    "accept_stat",
    withMathJax(),
    includeHTML("html/accept_stat.html"),
    hr(),
    stan_manual()
  ),
  tabPanel(
    "divergent",
    withMathJax(),
    includeHTML("html/ndivergent.html"),
    hr(),
    stan_manual()
  ),
  tabPanel(
    "energy",
    withMathJax(),
    includeHTML("html/energy.html"),
    hr(),
    stan_manual()
  ),
  tabPanel(
    "stepsize",
    withMathJax(),
    includeHTML("html/stepsize.html"),
    hr(),
    stan_manual()
  ),
  tabPanel(
    "n_leapfrog",
    withMathJax(),
    includeHTML("html/nleapfrog.html"),
    hr(),
    stan_manual()
  ),
  tabPanel(
    "treedepth",
    withMathJax(),
    includeHTML("html/treedepth.html"),
    hr(),
    stan_manual()
  ),
  tabPanel(
    "NUTS",
    withMathJax(),
    includeHTML("html/nuts.html"),
    hr(),
    stan_manual()
  )
)

pp_yrep_clr <- "#487575"
pp_yrep_fill <- "#6B8E8E"

.pp_hists_rep_vs_obs <- function(y, yrep_samp, geom = "histogram") {
  thm <- theme_classic() %+replace% (axis_color + axis_labs + fat_axis + no_yaxs + no_lgnd)
  graphs <- lapply(1:(1 + nrow(yrep_samp)), function(i) {
    if (i == 1) 
      g <-  qplot(x = y, geom = geom, color = I(vline_base_clr), 
                  size = I(0.2), fill = I(base_fill)) + labs(y = "", x = "y")
    else 
      g <- qplot(x = yrep_samp[i-1, ], geom = geom, 
                 color = I(pp_yrep_clr), fill = I(pp_yrep_fill),
                 size = I(0.2)) + labs(y = "", x = rownames(yrep_samp)[i-1])
    g + thm 
  })
  graphs
}

.pp_dens_rep_vs_obs <- function(y, yrep_samp, x_lim) {
  dat <- data.frame(t(yrep_samp))
  dat <- cbind(y = y, dat)
  mdat <- suppressMessages(reshape2::melt(dat))
  mdat$which <- "yrep"
  mdat$which[mdat$variable == "y"] <- "y"
  graph <- ggplot(mdat, aes(x = value, group = variable, fill = which, 
                            color = which, alpha = which, size = which))
  graph <- graph + 
    geom_density() + 
    scale_color_manual(values = c(vline_base_clr, pp_yrep_clr)) + 
    scale_fill_manual(values = c(base_fill, pp_yrep_fill)) + 
    scale_alpha_manual(values = c(3/4, 0)) + 
    scale_size_manual(values = c(1/3, 1/2)) + 
    scale_x_continuous(limits = x_lim) 
  graph + labs(x = "", y = "") + 
    theme_classic() %+replace% (axis_color + axis_labs + fat_axis + no_yaxs + no_lgnd)
}

.pp_hists_test_statistics <- function(stat_y, stat_yrep, which, geom = "histogram") {
  thm <- theme_classic() %+replace% (axis_color + axis_labs + fat_axis + no_yaxs)
  graph <- ggplot(data.frame(x = stat_yrep), aes(x = x)) 
  if (geom == "histogram") { 
    graph <- graph + stat_bin(aes(y=..count../sum(..count..)), 
                              color = pp_yrep_clr, fill = pp_yrep_fill, size = 0.2) 
  }
  if (geom == "density") {
    graph <- graph +
      geom_density(color = pp_yrep_clr, fill = pp_yrep_fill, size = 0.2)
  }
  graph + 
    geom_vline(xintercept = stat_y, color = vline_base_clr, size = 1.5, alpha = 1) +
    labs(y = "", x = paste0(which, "(yrep)")) +
    thm 
}

.pp_hist_resids <- function(resids) {
  thm <- theme_classic() %+replace% (axis_color + axis_labs + fat_axis + no_yaxs + no_lgnd)
  graph <- ggplot(data.frame(x = resids), aes(x = x)) + 
    stat_bin(aes(y=..count../sum(..count..)), 
             color = vline_base_clr, fill = base_fill, size = 0.2)
  graph + thm + labs(y = "", x = names(resids))
}

.pp_avg_rep_vs_avg_resid_rep <- function(rowMeans_yrep, rowMeans_resids){
  dat <- data.frame(x = rowMeans_yrep, y = rowMeans_resids)
  xy_labs <- labs(x = "Average yrep", y = "Average residual")
  thm <- theme_classic() %+replace% (axis_color + axis_labs + fat_axis + no_lgnd)
  graph <- ggplot(dat, aes(x, y)) + 
    geom_hline(yintercept = 0, color = vline_base_clr, size = 0.75) + 
    geom_point(fill = pp_yrep_fill, color = pp_yrep_clr, size = 2.75, 
               alpha = 0.75, shape = 21) + 
    xy_labs 
    
  graph + xy_labs + thm 
}


.pp_y_vs_avg_rep <- function(y, colMeans_yrep, zoom_to_zero = FALSE){
  dat <- data.frame(x = y, y = colMeans_yrep, z = abs(y-colMeans_yrep))
  xy_labs <- labs(x = "y", y = "Average yrep")
  thm <- theme_classic() %+replace% (axis_color + axis_labs + fat_axis)
  graph <- ggplot(dat, aes(x, y)) + 
    geom_abline(intercept = 0, slope = 1, color = vline_base_clr, size = 0.75) +
    geom_point(fill = pp_yrep_fill, color = pp_yrep_clr, size = 2.75, 
               alpha = 0.75, shape = 21) + 
    xy_labs + thm
  
  if (zoom_to_zero) {
    graph <- graph + 
      geom_hline(yintercept = 0, size = 3, color = axis_line_color) + 
      geom_vline(xintercept = 0, size = 0.5, color = axis_line_color) +
      thm %+replace% theme(axis.line = element_blank())
  }
  
  graph 
}

# param_trace_multi ------------------------------------------------------
# trace plots for multiple parameters
.param_trace_multi <- function(params = NULL, all_param_names, dat,
                               warmup_val = 0,
                               chain = 0, palette = "Default",
                               rect = "Samples", rect_color = "skyblue",
                               rect_alpha = 0.1,
                               layout = "Long", x1, x2) {


  params <- .update_params_with_groups(params, all_param_names)
  if(length(params) == 0) {
    params <- dimnames(dat)$parameters[1:min(4, dim(dat)[3])]
  }
  params <- unique(params)
  dat <- reshape2::melt(dat[,,params, drop=FALSE])

  if (!("chains" %in% colnames(dat))) { # fixes for if there's only 1 chain:
    dat$chains <- "chain:1"
  }
  dat$iterations <- x1:x2
  if (chain != 0)
    dat <- subset(dat, chains == paste0("chain:",chain))
  rect_xmin <- ifelse(rect == "Samples", Inf, -Inf)
  shading_rect <- annotate("rect", xmin = rect_xmin, xmax = warmup_val,
                           ymin = -Inf, ymax = Inf, fill = rect_color, alpha = rect_alpha)

  xy_labs <- labs(y = "Value", x = "Iteration")
  nclrs <- length(unique(dat$chains))

  lgnd_title <- ""
  if(palette == "Default") clrs <- scale_color_discrete(name = lgnd_title)
  if(palette == "Gray") clrs <- scale_color_grey(name = lgnd_title)
  if(palette == "Brewer (spectral)") clrs <- scale_color_brewer(name = lgnd_title, palette = "Spectral")
  if(palette == "Rainbow") clrs <- scale_colour_manual(name = lgnd_title, values = rainbow(nclrs))

  lgnd_txt <- theme(legend.text =  element_text(size = 13, face = "bold"))

  graph <- ggplot(dat, aes(x = iterations, y = value, color = chains))
  graph <- graph + xy_labs + clrs + theme_classic() %+replace% (axis_color + axis_labs + fat_axis + h_lines + lgnd_top + lgnd_txt + strip_txt + transparent)
  if (rect != "None") graph <- graph + shading_rect
  graph <- graph + geom_line(linewidth = 0.35) + scale_x_continuous(limits = c(x1, x2))

  if (layout == "Grid") {
    graph <- graph + facet_wrap(~ parameters, scales = "free_y")
  } else {
    graph <- graph + facet_grid(parameters ~., scales = "free_y")
  }

  graph
}


# param_hist --------------------------------------------------------------
# histogram for a single parameter
.param_hist <- function(param, dat, chain, binwd,
                        transform_x = "identity",
                        fill_color = "gray20", line_color = "gray35",
                        title = TRUE) {

  ttl <- "Histogram of Posterior Draws \n"
  if (transform_x != "identity") {
    t_x <- get(transform_x)
    dat <- apply(dat, 2, t_x)
  }
  x_lab <- if (transform_x != "identity")
    paste0(transform_x, "(", param, ")") else param

  dat <- reshape2::melt(dat)
  if (!("chains" %in% colnames(dat))) { # fixes for if there's only 1 chain:
    dat$chains <- "chain:1"
    dat$iterations <- 1:nrow(dat)
  }
  if (chain != 0) dat <- subset(dat, chains == paste0("chain:",chain))

  graph <- ggplot(dat, aes(x = value))

  if (binwd == 0) {
    graph <- graph + geom_histogram(fill = fill_color, color = line_color, linewidth = 0.2)
  } else {
    graph <- graph + geom_histogram(fill = fill_color, color = line_color,
                                    binwidth = binwd, linewidth = 0.2)
  }
  graph <- graph +
    labs(x = x_lab, y = "") +
    theme_classic() %+replace%
    (title_txt + axis_color + axis_labs + fat_axis + no_yaxs + transparent)
  if (title == TRUE) graph <- graph + ggtitle(ttl)

  graph
}


# param_dens --------------------------------------------------------------
# density plot for a single parameter

# data.frame of prior families and function names
priors <- data.frame(family = c("Normal", "t", "Cauchy", "Beta", "Exponential",
                                "Gamma", "Inverse Gamma"),
                     fun = c("dnorm", ".dt_loc_scale", "dcauchy", "dbeta",
                             "dexp", "dgamma", ".dinversegamma"))

.param_dens <- function(param, dat, chain,
                        fill_color = NULL, line_color = NULL,
                        point_est = "None", CI,
                        x_breaks = "Some", # y_breaks = "None",
                        x_lim = NULL,
                        chain_split = FALSE,
                        title = TRUE,
                        transform_x = "identity",
                        prior_fam = "None", prior_params) {

  ttl <- "Kernel Density Estimate \n"
  if (transform_x != "identity") {
    t_x <- get(transform_x)
    dat <- apply(dat, 2, t_x)
  }
  x_lab <- if (transform_x != "identity")
    paste0(transform_x, "(", param, ")") else param

  dat <- reshape2::melt(dat)
  if (!("chains" %in% colnames(dat))) { # fixes for if there's only 1 chain:
    dat$chains <- "chain:1"
    dat$iterations <- 1:nrow(dat)
  }
  if (chain != 0)
    dat <- subset(dat, chains == paste0("chain:",chain))

  Mean <- mean(dat$value)
  Median <- median(dat$value)
  dens_dat <- with(density(dat$value), data.frame(x,y))
  MAP <- with(dens_dat, x[which.max(y)])

  fclr <- ifelse(is.null(fill_color), "black", fill_color)
  lclr <- ifelse(is.null(line_color), "lightgray", line_color)

  many_breaks <- function(x) pretty(x, n = 15)
  too_many_breaks <- function(x) pretty(x, n = 35)
  if(x_breaks == "None") x_scale <- scale_x_continuous(breaks = NULL)
  if(x_breaks == "Some") x_scale <- scale_x_continuous()
  if(x_breaks == "Many") x_scale <- scale_x_continuous(breaks = many_breaks)

  if (chain == 0 & chain_split == TRUE) {
    graph <- ggplot(dat, aes(x = value, color = chains, fill = chains))
    if (prior_fam != "None") {
      graph <- graph + stat_function(alpha=0.75,color = "black", fun = as.character(priors$fun[priors$family==prior_fam]), args = prior_params, show_guides = TRUE)
    }
    graph <- graph +
      geom_density(alpha = 0.15) +
      scale_color_discrete("") +
      scale_fill_discrete("") +
      labs(x = x_lab, y = "") +
      x_scale + # y_scale +
      theme_classic() %+replace% (title_txt + axis_color + axis_labs + fat_axis + no_yaxs + transparent)

    if (title == TRUE) graph <- graph + ggtitle(ttl)
    return(graph)
  }

  graph <- ggplot(dens_dat, aes(x = x, ymax = y))
  if (prior_fam != "None") {
    graph <- graph +
      stat_function(fun = as.character(priors$fun[priors$family==prior_fam]),
                    args = prior_params)
  }
  graph <- graph +
    labs(x = param, y = "") +
    x_scale + # y_scale +
    labs(x = x_lab, y = "") +
    geom_ribbon(ymin = 0, fill = fclr, color = lclr, alpha = if (prior_fam == "None") 1 else 0.85) +
    theme_classic() %+replace% (title_txt + axis_color + axis_labs + fat_axis + no_yaxs + transparent)

  if (title == TRUE) graph <- graph + ggtitle(ttl)
  if (point_est != "None") {
    graph <- graph + annotate("segment",
                              x = get(point_est), xend = get(point_est),
                              y = 0, yend = max(dens_dat$y),
                              color = lclr, lwd = 1, lty = 2)
  }
  if (CI != "None") {
    lev <- (1 - as.numeric(CI))/2
    quant <- quantile(dat$value, probs = c(lev, 1 - lev))
    graph <- (graph +
                annotate("segment", x = quant, xend = quant, y = 0, yend = max(dens_dat$y), color = lclr, lty = rep(1:length(CI),2))
    )
  }
  if (!is.null(x_lim)) graph <- graph + scale_x_continuous(limits = x_lim)
  graph
}


# autocorr ----------------------------------------------------
.ac_fun <- function(x, lag.max, partial = FALSE) {
  if (!partial)
    acf(x, lag.max = lag.max, plot = FALSE)$acf[,, 1L]
  else
    pacf(x, lag.max = lag.max, plot = FALSE)$acf[,, 1L]
}
.ac_plot_data <- function(dat, lags, partial = FALSE) {
  nc <- length(unique(dat$chains))
  ac_list <- tapply(dat$value, INDEX = dat$chains, FUN = .ac_fun, lag.max = lags,
                    partial = partial, simplify = FALSE)
  nl <- if (partial) lags else lags + 1
  ch <- factor(rep(1:nc, each = nl), labels = paste0("chain:", 1:nc))
  ll <- rep(seq(if (partial) 1 else 0, lags), nc)
  data.frame(chains = ch, ac = do.call("c", args = ac_list), lag = ll)
}
.ac_plot_data_multi <- function(dat, lags, partial = FALSE) {
  nc <- length(unique(dat$chains))
  np <- length(unique(dat$parameters))
  ac_list <- tapply(dat$value, INDEX = list(dat$chains, dat$parameters),
                    FUN = .ac_fun, lag.max = lags,
                    partial = partial, simplify = FALSE)
  nl <- if (partial) lags else lags + 1
  ch <- factor(rep(rep(1:nc, each = nl), np), labels = paste0("chain:", 1:nc))
  ll <- rep(seq(if (partial) 1 else 0, lags), nc * np)
  pp <- factor(rep(1:np, each = nc * nl), labels = levels(dat$parameters))
  data.frame(parameters = pp, chains = ch, ac = do.call("c", args = ac_list), lag = ll)
}

# markov chain autocorrelation plot for single parameters
.autocorr_single_plot <- function(samps, lags) {
  dat <- reshape2::melt(samps)
  if (!("chains" %in% colnames(dat))) { # fixes for if there's only 1 chain:
    dat$chains <- "chain:1"
    dat$iterations <- 1:nrow(dat)
  }
  ac_dat <- .ac_plot_data(dat, lags)
  ac_labs <- labs(x = "Lag", y = "Autocorrelation")
  ac_theme <- theme_classic() %+replace%
    (axis_color + axis_labs + fat_axis + no_lgnd + transparent)
  y_scale <- scale_y_continuous(breaks = seq(0, 1, 0.25),
                                labels = c("0","","0.5","",""))
  graph <- ggplot(ac_dat, aes(x = lag, y = ac))
  graph <- graph +
    geom_bar(position = "identity", stat = "identity", fill = base_fill) +
    y_scale + ac_theme
  graph
}

# markov chain autocorrelation plot for multiple parameters
.autocorr_plot <- function(samps, partial = FALSE,
                           lags = 25, flip = FALSE,
                           combine_chains = FALSE) {

  params <- dimnames(samps)$parameters
  nParams <- length(params)
  nChains <- dim(samps)[2L]
  dat <- reshape2::melt(samps)
  if (!("chains" %in% colnames(dat))) { # fixes for if there's only 1 chain:
    dat$chains <- "chain:1"
    dat$iterations <- 1:nrow(dat)
  }
  ac_type <- if (partial) "partial" else "correlation"
  if (nParams == 1) ac_dat <- .ac_plot_data(dat, lags = lags, partial = partial)
  else ac_dat <- .ac_plot_data_multi(dat, lags = lags, partial = partial)

  ac_labs <- labs(x = "Lag", y = if (partial)
    "Partial autocorrelation" else "Autocorrelation")
  ac_theme <- theme_classic() %+replace%
    (axis_color + axis_labs + fat_axis + no_lgnd + strip_txt + transparent)
  y_scale <- scale_y_continuous(breaks = seq(0, 1, 0.25),
                                labels = c("0","","0.5","",""))
  title_theme <- theme(plot.title =  element_text(face = "bold", size = 18))
  if (combine_chains) {
    graph <- ggplot(ac_dat, aes(x= lag, y = ac))
    graph <- graph +
      geom_bar(position = "identity", stat = "identity",
               fill = base_fill, linewidth = 0.4) +
      y_scale +
      ac_labs +
      ac_theme

    if (nParams == 1) return(graph + ggtitle(paste(params, "\n")) + title_theme)
    else return(graph + facet_wrap(~parameters))
  }

  graph <- ggplot(ac_dat, aes(x = lag, y = ac, color = factor(chains),
                              fill = factor(chains)))
  graph <- graph +
    geom_bar(position = "identity", stat = "identity", linewidth = 0.4) +
    scale_fill_manual(values = rep(base_fill, nChains)) +
    scale_color_manual(values = rep(vline_base_clr, nChains)) +
    y_scale +
    ac_labs +
    ac_theme

  if (nParams == 1) {
    graph <- graph +
      facet_wrap(~chains) + ggtitle(paste(params, "\n")) + title_theme
    return(graph)
  } else { # nParams > 1

    while(is.null(flip)) return()

    graph <- graph +
      if (flip) facet_grid(chains ~ parameters) else facet_grid(parameters ~ chains)
    return(graph)
  }
}





# multiparam_plot --------------------------------------------------
# main plot of multiple parameters
.multiparam_plot <- function(samps, params = NULL, all_param_names,
                             show_density, show_ci_line, CI.level = 0.5,
                             show.level = 0.95, point_est, rhat_values,
                             color_by_rhat, rhat_palette, fill_color,
                             outline_color, est_color) {

  # params <- .update_params_with_regex(params, all_param_names)
  params <- .update_params_with_groups(params, all_param_names)
  .e <- environment()
  dim.samps <- dim(samps) #nIter, nChain, nParam
  if(length(params) == 0) {
    params <- dimnames(samps)$parameters[1:min(12, dim.samps[3])]
    if ("log-posterior" %in% params) {
      params <- params[-which(params == "log-posterior")]
    }
  }
  params <- unique(params)

  Blues <- c("#C6DBEF", "#4292C6", "#08306B")
  Grays <- c("#D9D9D9", "#737373", "#000000")
  Greens <- c("#C7E9C0", "#41AB5D", "#00441B")
  Oranges <- c("#FDD0A2", "#F16913", "#7F2704")
  Purples <- c("#DADAEB", "#807DBA", "#3F007D")
  Reds <- c("#FCBBA1", "#EF3B2C", "#67000D")
  rhat_pal <- get(rhat_palette)
  rhat_id <- ifelse(rhat_values < 1.05, "A",
                    ifelse(rhat_values < 1.1, "B", "C"))
  rhat_id <- factor(rhat_id[params], levels = c("A","B", "C"),
                    labels = c("<1.05", "<1.1", ">1.1"))
  rhat_colors <- scale_color_manual(name = bquote(hat(R)),
                                    values = rhat_pal,
                                    drop = FALSE)
  rhat_lgnd <- theme(legend.position = "top",
                     legend.title =  element_text(size = 13, face = "bold"),
                     legend.text =  element_text(size = 12))

  nParams <- length(params)
  nIter <- prod(dim.samps[1:2])
  samps.use <- array(samps[,,params], c(nIter, nParams))
  colnames(samps.use) <- params

  probs.use <- c(0.5 - show.level / 2,
                 0.5 - CI.level / 2,
                 0.5,
                 0.5 + CI.level / 2,
                 0.5 + show.level / 2)
  samps.quantile <- t(apply(samps.use, 2, quantile, probs = probs.use))
  y <- as.numeric(seq(nParams, 1, by = -1))
  xlim.use <- c(min(samps.quantile[,1]), max(samps.quantile[,5]))
  xrange <- diff(xlim.use)
  xlim.use[1] <- xlim.use[1] - 0.05 * xrange
  xlim.use[2] <- xlim.use[2] + 0.05 * xrange

  xy.df <- data.frame(params, y, samps.quantile)

  colnames(xy.df) <- c("params", "y", "ll", "l", "m", "h", "hh")
  if (point_est == "Mean") {
    xy.df$m <- unname(colMeans(samps.use))
  }
  p.base <- ggplot(xy.df, environment = .e)
  p.name <- scale_y_continuous(breaks = y, labels = params, limits = c(0.5, nParams + 1))
  p.theme <- theme(axis.title= element_blank(),
                   panel.background =  element_blank(),
                   panel.border =  element_blank(),
                   axis.ticks.y =  element_blank(),
                   axis.text= element_text(size=12),
                   axis.text.y= element_text(face = "bold"),
                   axis.line= element_line(linewidth = 4, color = axis_line_color),
                   axis.line.y= element_line(linewidth = 0.5, color = axis_line_color),
                   legend.position = "none",
                   panel.grid.major =  element_line(linewidth = 0.4),
                   panel.grid.minor.y =  element_blank())

  p.all <- p.base + xlim(xlim.use) + p.name + theme_bw() + p.theme + transparent

  if (show_ci_line | show_density) {
    p.ci <- geom_segment(aes(x = ll, xend = hh, y = y, yend = y),
                         colour = outline_color)
    p.all <- p.all + p.ci
  }
  if (show_density) {
    nPoint.den <- 512
    #plot density
    y.den <- matrix(0, nrow = nPoint.den, ncol = nParams)
    x.den <- matrix(0, nrow = nPoint.den, ncol = nParams)
    for(i in 1:nParams){
      d.temp <- density(samps.use[,i],
                        from = samps.quantile[i,1],
                        to = samps.quantile[i,5],
                        n = nPoint.den)
      x.den[,i] <- d.temp$x
      y.max <- max(d.temp$y)
      y.den[,i] <- d.temp$y / y.max * 0.8 + y[i]
    }
    df.den <- data.frame(x = as.vector(x.den), y = as.vector(y.den),
                         name = rep(params, each = nPoint.den))
    p.den <- geom_line(data = df.den, aes(x = x, y = y, group = name),
                       color = outline_color)

    #shaded interval
    y.poly <- matrix(0, nrow = nPoint.den + 2, ncol = nParams)
    x.poly <- matrix(0, nrow = nPoint.den + 2, ncol = nParams)
    for(i in 1:nParams){
      d.temp <- density(samps.use[,i],
                        from = samps.quantile[i,2],
                        to = samps.quantile[i,4],
                        n = nPoint.den)
      x.poly[,i] <- c(d.temp$x[1], as.vector(d.temp$x), d.temp$x[nPoint.den])
      y.max <- max(d.temp$y)
      y.poly[,i] <- as.vector(c(0, as.vector(d.temp$y) / y.max * 0.8, 0) + y[i])
    }
    df.poly <- data.frame(x = as.vector(x.poly), y = as.vector(y.poly),
                          name = rep(params, each = nPoint.den + 2))
    p.poly <- geom_polygon(data = df.poly, aes(x = x, y = y, group = name, fill = y))
    p.col <- scale_fill_gradient(low = fill_color, high = fill_color, guide = "none")

    #point estimator
    if (color_by_rhat) {
      p.point <- geom_segment(aes(x = m, xend = m, y = y, yend = y + 0.25,
                                  color = rhat_id), linewidth = 1.5)
      p.all + p.poly + p.den + p.col + p.point + rhat_colors + rhat_lgnd
    } else {
      p.point <- geom_segment(aes(x = m, xend = m, y = y, yend = y + 0.25),
                              colour = est_color,
                              linewidth = 1.5)
      p.all + p.poly + p.den + p.col + p.point
    }

  } else {
    p.ci.2 <- geom_segment(aes(x = l, xend = h, y = y, yend = y),
                           colour = fill_color, linewidth = 2)
    if (color_by_rhat) {
      p.point <- geom_point(aes(x = m, y = y, fill = rhat_id), color = "black",
                            shape = 21, size = 4)
      p.all + p.ci.2 + p.point + rhat_colors + rhat_lgnd
    } else {
      p.point <- geom_point(aes(x = m, y = y), size = 4, color = fill_color,
                            fill = est_color, shape = 21)
      p.all + p.ci.2 + p.point
    }
  }
}


# histogram of rhat, n_eff/N or mcse/sd -----------------------------------
.rhat_neff_mcse_hist <- function(dat, which, N) {
  # samps: post-warmup samples
  xlab <- switch(which,
         rhat = "Rhat statistic",
         n_eff = "Effective sample size / iterations",
         mcse = "Monte Carlo se / posterior sd"
         )
  my_labs <- labs(y = "", x = xlab)
  base_fill
  graph <- qplot(x = x, data = dat, color = I(vline_base_clr),
                 fill = I(base_fill), linewidth = I(0.2))
  graph <- graph +
    my_labs +
    theme_classic() %+replace% (axis_color + axis_labs + fat_axis + no_yaxs + transparent)

  graph
}

# n_eff_warnings -----------------------------------------------------------
.n_eff_warnings <- function(summary, threshold = 10,
                            N_total = NULL) {
  n_eff <- summary[,"n_eff"]
  warn_params <- names(which(n_eff / N_total < threshold / 100))
  ll <- length(warn_params)
  if (ll == 0) "None"
  else paste0(warn_params, collapse = ", ")
}

# rhat_warnings -----------------------------------------------------------
.rhat_warnings <- function(summary, threshold = 1.10) {
  rhat <- summary[,"Rhat"]
  warn_params <- names(which(rhat > threshold))
  ll <- length(warn_params)
  if (ll == 0) "None"
  else paste0(warn_params, collapse = ", ")
}

# mcse_over_sd_warnings -----------------------------------------------------------
.mcse_over_sd_warnings <- function(summary, threshold = 10) {
  dat <- summary[,c("se_mean", "sd")]
  warn_params <- names(which(dat[,1] > (threshold/100) * dat[,2]))

  ll <- length(warn_params)
  if (ll == 0) "None"
  else paste0(warn_params, collapse = ", ")
}


# dynamic trace plot ------------------------------------------------------
.param_trace_dynamic <- function(param_samps, chain,
                                 warmup_val, warmup_shade = TRUE,
                                 stack = FALSE, grid = FALSE,
                                 x_lab = NULL, y_lab = NULL) {

  dim_samps <- dim(param_samps)
  if (is.null(dim_samps)) nChains <- 1
  else nChains <- dim_samps[2]
  if (nChains == 1) {
    param_chains <- xts::as.xts(ts(param_samps, start = 1))
  } else {
    if (chain != 0) {
      param_samps <- param_samps[, chain]
      param_chains <- xts::as.xts(ts(param_samps, start = 1))
    } else {
      param_chains <- xts::as.xts(ts(param_samps[,1], start = 1))
      for (i in 2:nChains) {
        param_chains <- cbind(param_chains, xts::as.xts(ts(param_samps[,i], start = 1)))
      }
      colnames(param_chains) <- paste0("Chain", 1:nChains)
    }
  }

  `%>%` <- dygraphs::`%>%`
  shade_to <- if (warmup_shade)
    paste0(warmup_val,"-01-01") else "0001-01-01"
  y_axis_label_remove <- if (stack)
    "white" else NULL
  clrs <- color_vector(nChains)
  if (chain != 0) clrs <- clrs[chain]
  dygraphs::dygraph(param_chains, xlab = x_lab, ylab = y_lab) %>%
    dygraphs::dyAxis("y", axisLabelColor = y_axis_label_remove) %>%
    dygraphs::dyAxis("x", axisLabelColor = "white") %>%
    dygraphs::dyOptions(colors = clrs, stackedGraph = stack, drawGrid = grid,
                        animatedZooms = TRUE, axisLineColor = axis_line_color) %>%
    dygraphs::dyLegend(show = "never") %>%
    dygraphs::dyHighlight(highlightCircleSize = 4,
                highlightSeriesBackgroundAlpha = 1/3,
                hideOnMouseOut = TRUE,
                highlightSeriesOpts = list(strokeWidth = 1.75)) %>%
    dygraphs::dyRoller(rollPeriod = 1) %>%
    dygraphs::dyShading(from = "0001-01-01", to = shade_to, color = "#EFEFEF", axis = "x") %>%
    dygraphs::dyCSS(css = "css/ShinyStan_dygraphs.css")
}

# trivariate_plot ---------------------------------------------------------
.param_trivariate <- function(samps, params,
                              transform_x = "identity", transform_y = "identity",
                              transform_z = "identity",
                              pt_size = 1, pt_color = "gray35", show_grid = TRUE,
                              flip_y = TRUE) {
  nParams <- 3
  dim_samps <- dim(samps)
  nIter <- dim_samps[1] * dim_samps[2]
  samps_use <- array(samps[,, params], c(nIter, nParams))
  colnames(samps_use) <- params

  t_x <- get(transform_x)
  t_y <- get(transform_y)
  t_z <- get(transform_z)

  if (transform_x != "identity") {
    samps_use[,1] <- t_x(samps_use[,1])
    colnames(samps_use)[1] <- paste0(transform_x, "(", params[1], ")")
  }
  if (transform_y != "identity") {
    samps_use[,2] <- t_y(samps_use[,2])
    colnames(samps_use)[2] <- paste0(transform_y, "(", params[2], ")")
  }
  if (transform_z != "identity") {
    samps_use[,3] <- t_z(samps_use[,3])
    colnames(samps_use)[3] <- paste0(transform_z, "(", params[3], ")")
  }
  threejs::scatterplot3js(samps_use, size = pt_size, color = pt_color,
                          grid = show_grid, flip.y = flip_y)
}


# bivariate plot ----------------------------------------------------------
.bivariate_plot <- function(samps, sp = NULL, max_td = NULL,
                            param, param2,
                            pt_alpha = 0.10,
                            pt_size = 2,
                            pt_shape = 10,
                            pt_color = "gray20",
                            ellipse_color = "black",
                            ellipse_lev = "None",
                            ellipse_lty = 1,
                            ellipse_lwd = 1,
                            ellipse_alpha = 1,
                            lines = "back",
                            lines_color = "gray",
                            lines_alpha,
                            points = TRUE,
                            transform_x = "identity",
                            transform_y = "identity"
){

  shape_translator <- function(x) {
    shape <- if (x >= 6) x + 9 else x
    shape
  }

  params <- c(param, param2)
  nParams <- 2
  nIter <- dim(samps)[1] * dim(samps)[2]
  samps_use <- array(samps[,,params], c(nIter, nParams))
  colnames(samps_use) <- params

  t_x <- get(transform_x)
  # t_x <- function(x) eval(parse(text = transform_x))
  t_y <- get(transform_y)
  x_lab <- if (transform_x != "identity")
    paste0(transform_x, "(", param, ")") else param
  y_lab <- if (transform_y != "identity")
    paste0(transform_y, "(", param2, ")") else param2
  param_labs <- labs(x = x_lab, y = y_lab)

  dat <- data.frame(
    x = if (transform_x == "identity")
      samps_use[,param] else t_x(samps_use[,param]),
    y = if (transform_y == "identity")
      samps_use[,param2] else t_y(samps_use[,param2]))
  if (!is.null(sp)) {
    dat$divergent <- c(sapply(sp, FUN = function(y) y[, "divergent__"]))
    dat$hit_max_td <- if (is.null(max_td)) 0 else
      c(sapply(sp, FUN = function(y) as.numeric(y[, "treedepth__"] == max_td)))
  } else {
    dat$divergent <- 0
    dat$hit_max_td <- 0
  }
  graph <- ggplot(dat, aes(x = x, y = y, xend=c(tail(x, n=-1), NA),
                           yend=c(tail(y, n=-1), NA)))

  if (lines == "hide") {
    graph <- graph + geom_point(alpha = pt_alpha, size = pt_size,
                                shape = shape_translator(pt_shape),
                                color = pt_color)
  } else { # if lines = "back" or "front"
    if (lines == "back") {
      graph <- graph +
        geom_path(alpha = lines_alpha, color = lines_color) +
        geom_point(alpha = pt_alpha, size = pt_size,
                   shape = shape_translator(pt_shape), color = pt_color)
    } else { # lines = "front"
      graph <- graph +
        geom_point(alpha = pt_alpha, size = pt_size,
                   shape = shape_translator(pt_shape), color = pt_color) +
        geom_path(alpha = lines_alpha, color = lines_color)
    }
  }
  if (ellipse_lev != "None")
    graph <- graph + stat_ellipse(level = as.numeric(ellipse_lev), color = ellipse_color,
                                  linetype = ellipse_lty, linewidth = ellipse_lwd, alpha = ellipse_alpha)
  if (!all(dat$divergent == 0))
    graph <- graph + geom_point(data = subset(dat, divergent == 1), aes(x,y),
                                size = pt_size + 0.5, shape = 21,
                                color = "#570000", fill = "#ae0001")
  if (!all(dat$hit_max_td == 0))
    graph <- graph + geom_point(data = subset(dat, hit_max_td == 1), aes(x,y),
                                size = pt_size + 0.5, shape = 21,
                                color = "#5f4a13", fill = "#eeba30")
  graph + param_labs +
    theme_classic() %+replace% (no_lgnd + axis_labs + fat_axis + axis_color + transparent)
}

fluidRow(
  column(
    width = 3, 
    uiOutput("ui_trivariate_select_x")
  ),
  column(
    width = 3,
    selectizeInput(
      "trivariate_param_y",
      label = strong_bl("y-axis"),
      choices = .param_list,
      selected = if (length(unlist(.param_list)) > 1) 
        unlist(.param_list)[2] else unlist(.param_list)[1],
      multiple = FALSE
    )
  ),
  column(
    width = 3,
    selectizeInput(
      "trivariate_param_z",
      label = strong_bl("z-axis"),
      choices = rev(.param_list),
      multiple = FALSE
    )
  )
)

# histogram
hist_transform_x <- eventReactive(input$hist_transform_x_go > 0,
                                  input$hist_transform_x)

histogram_plot <- reactive({
  validate(
    need(input$param, message = FALSE),
    need(!is.null(input$hist_chain), message = FALSE)
  )
  chain <- input$hist_chain
  if (is.na(chain))
    chain <- 0
  binwd <- input$hist_binwd
  if (is.na(binwd))
    binwd <- 0
  
  do.call(
    ".param_hist",
    args = list(
      param       = input$param,
      dat         = par_samps_post_warmup(),
      chain       = chain,
      binwd       = binwd,
      fill_color  = input$hist_fill_color,
      line_color  = input$hist_line_color,
      transform_x = hist_transform_x()
    )
  )
})

output$hist_plot_out <- renderPlot({
  x <- histogram_plot()
  suppress_and_print(x)
}, bg = "transparent")

# download plot
output$download_histogram <- downloadHandler(
  filename = 'shinystan-histogram-gg.RData',
  content = function(file) {
    shinystan_histogram_gg <- histogram_plot()
    save(shinystan_histogram_gg, file = file)
  }
)
output$save_pdf_histogram = downloadHandler(
  filename = "shinstan-histogram.pdf",
  content = function(file) {
    ggsave(file, plot = histogram_plot(), device = pdf)
  }
)

div(
  class = "help-glossary-div",
  br(), br(),
  div(
    class = "help-glossary-nav-container",
    navlistPanel(
      well = TRUE,
      id = "help_navlist",
      "Topics",
      tabPanel(
        "Questions, bugs, and new features",
        div(
          class = "glossary-entry",
          h4("Stan users group"),
          p(
            "To ask a question or suggest a new feature visit the",
            a(
              "Stan users message board.", 
              href = "http://discourse.mc-stan.org"
            )
          ),
          br(),
          h4("GitHub issue tracker"),
          p(
            "To report a bug  or suggest a new feature visit the",
            a(
              "GitHub issue tracker.", 
              href = "https://github.com/stan-dev/shinystan/issues"
            )
          )
        )
      ),
      tabPanel(
        "Saving plots",
        div(
          class = "glossary-entry",
          h4("Saving plots as ggplot2 objects"),
          p(
            "Clicking on a 'Save ggplot2 object' button will be save an .RData
            file that you can load into your Global Environment using the",
            code("load"),
            "function in R.
            You can then make changes to the plot using the functions in the
            ggplot2 package."
          ),
          p(
            "Any plot that can be saved as a ggplot2 object can also be saved
            as a PDF."
          )
      )), 
      tabPanel(
        "Large models and launch speed",
        div(
          class = "glossary-entry",
          h4("Launching ShinyStan faster"),
          p(
            "The", code("drop_parameters"), "function in the", 
            strong("shinystan"), "R package will allow you to reduce the size", 
            "of a shinystan object by removing parameters.", 
            "See", code("help('drop_parameters', 'shinystan')"), 
            "for the documentation."
          ),
          p(
            "Additionally, for large models, the", code("launch_shinystan"),
            "function will launch the app faster when used with a",
            "shinystan object rather than a stanfit object",
            "(because no conversion is required).",
            "If ShinyStan takes a long time to launch for your",
            "model then it can help to first create a",
            "shinystan object using the", code("as.shinystan"), "function.",
            "Alternatively, the first time you launch",
            "ShinyStan using a stanfit object, a shinystan",
            "object will be returned if you assign the value of",
            code("launch_shinystan"),
            "to a name, e.g."
          ),
          p(code("sso <- launch_shinystan(stanfit)")),
          p("rather than just"),
          p(code("launch_shinystan(stanfit)")),
          p(
            "The next time you launch ShinyStan for the same",
            "model you can launch it using", code("sso"), "rather than",
            code("stanfit"), "and it should be quicker to launch.",
            "If it is still too slow then dropping some large parameters", 
            "from the shinystan object is the best solution."
          )
        )
      )
    )
  ),
  br(), br()
)

shinyjs::hidden(div(
  id = "density_options",
  wellPanel(
    class = "optionswell",
    hr(class = "hroptions"),
    strongBig("Transformation"),
    transform_helpText("x"),
    fluidRow(
      column(width = 4, transformation_selectInput("dens_transform_x")),
      column(
        width = 2,
        actionButton("dens_transform_x_go", label = "Transform", class = "transform-go")
      )
    ),
    hr(class = "hroptions"),
    selectInput(
      "dens_options_display",
      label = strongBig("Control"),
      choices = c("Options", "Aesthetics", "Compare to function" = "Compare"),
      selected = "Options",
      width = "50%"
    ),
    conditionalPanel(
      condition = "input.dens_options_display == 'Options'",
      fluidRow(
        column(
          width = 3,
          numericInput(
            "dens_chain",
            label = strongMed("Chain"),
            min = 0,
            max = .nChains,
            step = 1,
            value = 0
          )
        ),
        column(
          width = 3,
          conditionalPanel(
            condition = "input.dens_chain == 0",
            radioButtons(
              "dens_chain_split",
              label = strongMed("All chains"),
              choices = c("Together", "Separate"),
              selected = "Together",
              inline = FALSE
            )
          )
        ),
        column(
          width = 3,
          selectInput(
            "dens_point_est",
            strongMed("Point est"),
            choices = c("None", "Mean", "Median", "MAP"),
            selected = "None"
          )
        ),
        column(
          width = 3,
          selectInput(
            "dens_ci",
            strongMed("CI %"),
            choices = c(
              "None" = "None",
              "50%" = 0.5,
              "80%" = 0.8,
              "95%" = 0.95
            ),
            selected = "None"
          )
        )
      )),
    conditionalPanel(
      condition = "input.dens_options_display == 'Aesthetics'",
      fluidRow(
        column(
          width = 3,
          selectInput(
            "dens_x_breaks",
            strongMed("x breaks"),
            choices = c("None", "Some", "Many"),
            selected = "Some"
          )
        ),
        column(
          width = 3,
          colourpicker::colourInput("dens_fill_color", strongMed("Fill"), base_fill)
        ),
        column(
          width = 3,
          colourpicker::colourInput("dens_line_color", strongMed("Line"), vline_base_clr)
        )
      )),
    conditionalPanel(
      condition = "input.dens_options_display == 'Compare'",
      fluidRow(
        column(
          width = 4,
          selectInput(
            "dens_prior",
            strongMed("Family"),
            choices = list(
              "None",
              "Normal",
              "t",
              "Cauchy",
              "Exponential",
              "Gamma",
              "Inverse Gamma",
              "Beta"
            )
          )),
        column(
          width = 2,
          condPanel_dens_prior(
            "Normal",
            numericInput(
              "dens_prior_normal_mu",
              "Location",
              value = 0,
              step = 0.1
            )
          ),
          condPanel_dens_prior(
            "t",
            numericInput(
              "dens_prior_t_df",
              "df",
              value = 1,
              min = 0,
              step = 0.1
            )
          ),
          condPanel_dens_prior(
            "Cauchy",
            numericInput(
              "dens_prior_cauchy_mu",
              "Location",
              value = 0,
              step = 0.1
            )
          ),
          condPanel_dens_prior(
            "Beta",
            numericInput(
              "dens_prior_beta_shape1",
              "Shape1",
              value = 1,
              min = 0,
              step = 0.1
            )
          ),
          condPanel_dens_prior(
            "Exponential",
            numericInput(
              "dens_prior_expo_rate",
              "Rate",
              value = 1,
              min = 0,
              step = 0.1
            )
          ),
          condPanel_dens_prior(
            "Gamma",
            numericInput(
              "dens_prior_gamma_shape",
              "Shape",
              value = 1,
              min = 0,
              step = 0.1
            )
          ),
          condPanel_dens_prior(
            "Inverse Gamma",
            numericInput(
              "dens_prior_inversegamma_shape",
              "Shape",
              value = 1,
              min = 0,
              step = 0.1
            )
          )
        ),
        column(
          width = 2,
          condPanel_dens_prior(
            "Normal",
            numericInput(
              "dens_prior_normal_sigma",
              "Scale",
              value = 1,
              min = 0,
              step = 0.1
            )
          ),
          condPanel_dens_prior(
            "t",
            numericInput(
              "dens_prior_t_mu",
              "Location",
              value = 0,
              step = 0.1
            )
          ),
          condPanel_dens_prior(
            "Cauchy",
            numericInput(
              "dens_prior_cauchy_sigma",
              "Scale",
              value = 1,
              min = 0,
              step = 0.1
            )
          ),
          condPanel_dens_prior(
            "Beta",
            numericInput(
              "dens_prior_beta_shape2",
              "Shape2",
              value = 1,
              min = 0,
              step = 0.1
            )
          ),
          condPanel_dens_prior(
            "Gamma",
            numericInput(
              "dens_prior_gamma_rate",
              "Rate",
              value = 1,
              min = 0,
              step = 0.1
            )
          ),
          condPanel_dens_prior(
            "Inverse Gamma",
            numericInput(
              "dens_prior_inversegamma_scale",
              "Scale",
              value = 1,
              min = 0,
              step = 0.1
            )
          )
        ),
        column(width = 2, condPanel_dens_prior(
          "t",
          numericInput(
            "dens_prior_t_sigma",
            "Scale",
            value = 1,
            min = 0,
            step = 0.1
          )
        ))
      ),
      condPanel_dens_together(
        textInput(
          "dens_xzoom",
          label = strongMed("x-axis limits"),
          value = "c(min, max)"
        )
      ),
      br()
    )
  )
))

div(
  style = "text-align: center; margin-top: 100px;",
  shinystan_version(),
  br(),br(),
  a(
    style = "font-size: 16px;",
    strong("Stan Development Team"),
    href = "http://mc-stan.org/"
  ),
  br(),
  a(
    style = "font-size: 14px;", 
    "mc-stan.org", 
    href = "http://mc-stan.org/"
  ),
  div(
    actionLink(
      inputId = "shinystan_citation_show",
      label = "Show Citation",
      style = "font-size: 12px;",
      class = "aoptions"
    ),
    div(shinyjs::hidden(
      wellPanel(
        id = "citation_div",
        style = "text-align: left;",
        includeHTML("html/citation.html")
      )
    ))
  ),
  br(),br(),
  h6("Author"),
  helpText(style = "font-size: 12px;", "Jonah Gabry"),
  br(),
  h6(style = "font-size: 12px;", "Contributors"),
  helpText(style = "font-size: 12px;", includeHTML("html/contribs.html")),
  br(),
  h6("Logo"),
  helpText(
    style = "font-size: 12px;", 
    "Michael Betancourt"
  ),
  br(),
  h6("Shiny"),
  helpText(
    style = "font-size: 12px;",
    "ShinyStan is powered by the",
    a(
      href = "http://shiny.rstudio.com", 
      "Shiny web application framework"
    ),
    "(RStudio)"
  ),
  br(),
  h6("Source code"),
  a(
    style = "color: #190201;",
    href = "http://github.com/stan-dev/shinystan",
    target = "_blank",
    tags$i(class = "fa fa-github fa-3x")
  )
)

div(
  br(),
  helpText(
    style = "font-size: 11px;",
    "Use your mouse to highlight areas in the traceplot to zoom into. Double-click to reset.",
    "The number in the small black box in the bottom left corner controls the",
    em("roll period."),
    "If you specify a roll period of N the resulting graph will be a moving average,",
    "with each plotted point representing the average of N points in the data."
  )
)

pp_hists_rep_vs_obs <- reactive({
  pp_tests()
  validate(need(input$pp_hists_rep_vs_obs_type, message = "Loading..."))
  y <- get_y()
  yrep <- get_yrep()
  sample_ids <- sample_ids_for_hist()
  yrep_samp <- yrep[sample_ids, ]
  rownames(yrep_samp) <- paste("yrep", sample_ids)
  geom <- input$pp_hists_rep_vs_obs_type 
  if (geom == "density" & input$pp_hists_rep_vs_obs_overlay == TRUE) {
    x_lim <- range(c(y, yrep))
    out <- do.call(".pp_dens_rep_vs_obs", args = list(
      y = y, 
      yrep_samp = yrep_samp,
      x_lim = x_lim
    ))
    return(out)
  }
  graphs <- .pp_hists_rep_vs_obs(y = y, yrep_samp = yrep_samp, geom = geom)
  suppressMessages(do.call(gridExtra::grid.arrange, c(graphs, ncol = 3)))
})

output$pp_hists_rep_vs_obs_out <- renderPlot({
  pp_hists_rep_vs_obs()
}, bg = "transparent")

# shinystan is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 3 of the License, or (at your option) any later
# version.
#
# shinystan is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, see <http://www.gnu.org/licenses/>.


# shinystan class definition ------------------------------------------
#' S4 \code{shinystan} objects
#'
#' @aliases shinystan-class
#' @description See \code{\link{as.shinystan}} for documentation on creating 
#'   \code{shinystan} objects and \code{\link{eight_schools}} for an example
#'   object.
#'  
#' @slot model_name (\code{"character"}) Model name.
#' @slot param_names (\code{"character"}) Parameter names.
#' @slot param_dims (\code{"list"}) Parameter dimensions.
#' @slot posterior_sample (\code{"array"}) MCMC sample.
#' @slot summary (\code{"matrix"}) Summary stats for \code{posterior_sample}.
#' @slot sampler_params (\code{"list"}) Sampler parameters (for certain Stan
#'   models only).
#' @slot n_chain (\code{"integer"}) Number of chains.
#' @slot n_iter (\code{"integer"}) Number of iterations per chain.
#' @slot n_warmup (\code{"integer"}) Number of warmup iterations per chain.
#' @slot user_model_info (\code{"character"}) Notes to display on the
#'   \strong{Notepad} page in the 'ShinyStan' GUI.
#' @slot model_code (\code{"character"}) Model code to display on the
#'   \strong{Model Code} page in the 'ShinyStan' GUI.
#' @slot misc (\code{"list"}) Miscellaneous, for internal use.
#' 
#' @template seealso-as.shinystan
#' @template seealso-drop_parameters
#' @template seealso-generate_quantity
#' @seealso \code{\link{shinystan-metadata}} to view or change metadata
#'   associated with a \code{shinystan} object.
#'   
#' @template reference-muth
#' 
shinystan <- setClass(
  Class = "shinystan",
  slots = list(
    model_name       = "character",
    param_names      = "character",
    param_dims       = "list",
    posterior_sample = "array",
    summary          = "matrix",
    sampler_params   = "list",
    n_chain          = "numeric",
    n_iter           = "numeric",
    n_warmup         = "numeric",
    user_model_info  = "character",
    model_code       = "character",
    misc             = "list"
  ),
  prototype = list(
    model_name = "No name",
    param_names = "",
    param_dims = list(),
    posterior_sample = array(NA, c(1, 1)),
    summary = matrix(NA, nr = 1, nc =1),
    sampler_params = list(NA),
    n_chain = 0,
    n_iter = 0,
    n_warmup = 0,
    user_model_info = "Use this space to store notes about your model",
    model_code = "Use this space to store your model code",
    misc = list(sso_version = utils::packageVersion("shinystan"))
  )
)



# create shinystan objects ------------------------------------------------
  
# as.shinystan (generic) --------------------------------------------------
#' Create and test \code{shinystan} objects
#' 
#' @description The \code{as.shinystan} function creates \code{shinystan}
#'   objects that can be used with \code{\link{launch_shinystan}} and various
#'   other functions in the \pkg{shinystan} package. \code{as.shinystan} is a
#'   generic for which the \pkg{shinystan} package provides several methods.
#'   Currently methods are provided for creating \code{shinystan} objects from
#'   arrays, lists of matrices, \code{stanfit} objects (\pkg{rstan}),
#'   \code{stanreg} objects (\pkg{rstanarm}), and \code{mcmc.list} objects
#'   (\pkg{coda}).
#'   
#'   \code{is.shinystan} tests if an object is a \code{shinystan} object.
#'
#' @name as.shinystan
#' @export
#' @param X For \code{as.shinystan}, an object to be converted to a
#'   \code{shinystan} object. See the Methods section below. For
#'   \code{is.shinystan}, an object to check.
#' @param ... Arguments passed to the individual methods.
#'   
#' @return \code{as.shinystan} returns a \code{shinystan} object, which is an
#'   instance of S4 class \code{"shinystan"}.
#'   
#'   \code{is.shinystan} returns \code{TRUE} if the tested object is a
#'   \code{shinystan} object and \code{FALSE} otherwise.
#'
#' @template seealso-launch
#' @template seealso-drop_parameters   
#' @template seealso-generate_quantity
#'
setGeneric("as.shinystan", function(X, ...) {
  if (inherits(X, "shinystan"))
    stop("Already a shinystan object.")
  standardGeneric("as.shinystan")
})

#' @export
#' @rdname as.shinystan
is.shinystan <- function(X) inherits(X, "shinystan")

# as.shinystan (array) ---------------------------------------------------
#' @describeIn as.shinystan Create a \code{shinystan} object from a 3-D
#'   \code{\link{array}} of simulations. The array should have dimensions
#'   corresponding to iterations, chains, and parameters, in that order.
#'
#' @param model_name A string giving a name for the model.
#' @param burnin Deprecated. Use \code{warmup} instead. The \code{burnin}
#'   argument will be removed in a future release.
#' @param warmup The number of iterations to treat as warmup. Should be
#'   \code{0} if warmup iterations are not included in \code{X}.
#' @param param_dims Rarely used and never necessary. A named list giving the 
#'   dimensions for all parameters. For scalar parameters use \code{0} as the 
#'   dimension. See Examples.
#' @param model_code Optionally, a character string with the code used to run
#'   the model. This can also be added to your \code{shinystan} object later
#'   using the \code{\link[shinystan]{model_code}} function. See 
#'   \code{\link[shinystan]{model_code}} for additional formatting instructions.
#'   After launching the app the code will be viewable in the \strong{Model 
#'   Code} tab. For \code{stanfit} (\pkg{rstan}) and \code{stanreg}
#'   (\pkg{rstanarm}) objects the model code is automatically retrieved from the
#'   object.
#' @param note Optionally, text to display on the \strong{Notepad} page in the 
#'   'ShinyStan' GUI (stored in \code{user_model_info} slot of the
#'   \code{shinystan} object).
#' @param sampler_params,algorithm,max_treedepth Rarely used and never 
#'   necessary. If using the \code{as.shinystan} method for arrays or lists, 
#'   these arguments can be used to manually provide information that is 
#'   automatically retrieved from a stanfit object when using the 
#'   \code{as.shinystan} method for stanfit objects. If specified, 
#'   \code{sampler_params} must have the same structure as an object returned by
#'   \code{\link[rstan]{get_sampler_params}} (\pkg{rstan}), which is a list of 
#'   matrices, with one matrix per chain. \code{algorithm}, if specified, must 
#'   be either \code{"NUTS"} or \code{"HMC"} (static HMC). If \code{algorithm}
#'   is \code{"NUTS"} then \code{max_treedepth} (an integer indicating the
#'   maximum allowed treedepth when the model was fit) must also be provided.
#'   
#' @examples  
#' \dontrun{
#' sso <- as.shinystan(X, ...) # replace ... with optional arguments or omit it
#' launch_shinystan(sso)
#' }
#'
setMethod(
  "as.shinystan",
  signature = "array",
  definition = function(X,
                        model_name = "unnamed model",
                        warmup = 0, burnin = 0, 
                        param_dims = list(),
                        model_code = NULL,
                        note = NULL,
                        sampler_params = NULL, 
                        algorithm = NULL,
                        max_treedepth = NULL,
                        ...) {
    validate_model_code(model_code)
    is3D <- isTRUE(length(dim(X)) == 3)
    if (!is3D)
      stop ("'X' must have 3 dimensions.")
    
    if (is.null(dimnames(X)[[3]]))
      dimnames(X)[[3]] <- paste0("V", seq_len(dim(X)[3]))
    param_names <- dimnames(X)[[3]]
    dimnames(X) <- list(
      iterations = seq_len(nrow(X)),
      chains = paste0("chain:", seq_len(ncol(X))),
      parameters = param_names
    )
    
    sp <- .validate_sampler_params(
      sampler_params, 
      n_chain = ncol(X), 
      n_iter = nrow(X), 
      algorithm = algorithm
    )
    
    n_warmup <- .deprecate_burnin(burnin, warmup)
    sso <- shinystan(
      model_name = model_name,
      param_names = param_names,
      param_dims = .set_param_dims(param_dims, param_names),
      posterior_sample = X,
      sampler_params = sp,
      summary = shinystan_monitor(X, warmup = n_warmup),
      n_chain = ncol(X),
      n_iter = nrow(X),
      n_warmup = n_warmup
    )
    
    if (!is.null(sampler_params)) {
      if (is.null(algorithm)) {
        stop("If 'sampler_params' is specified then 'algorithm' can't be NULL.")
      } else {
        algorithm <- match.arg(algorithm, choices = c("HMC", "NUTS"))
        if (algorithm == "NUTS" && is.null(max_treedepth))
          stop("If 'algorithm' is 'NUTS' then 'max_treedepth' must be provided.")
      }
      slot(sso, "misc") <- list(
        max_td = max_treedepth,
        stan_method = "sampling",
        stan_algorithm = algorithm,
        sso_version = utils::packageVersion("shinystan")
      )
    }
    if (!is.null(note))
      sso <- suppressMessages(notes(sso, note = note, replace = TRUE))
    if (!is.null(model_code))
      sso <- suppressMessages(model_code(sso, code = model_code))
    sso <- .rename_scalar(sso, oldname = "lp__", newname = "log-posterior")
    
    return(sso)
  }
)

# FIXME: remove this when 'burnin' arg is removed
.deprecate_burnin <- function(burnin = 0, warmup = 0) {
  if (warmup == 0) {
    if (burnin == 0) {
      return(0)
    } else {
      warning("The 'burnin' argument is deprecated and will be removed ", 
              "in a future release. Use the 'warmup' argument instead.", 
              call. = FALSE)
      return(burnin)
    }
  } else if (burnin == 0) {
    return(warmup)
  } else {
    stop("'burnin' and 'warmup' can't both be specified. ", 
         "'burnin' is deprecated. Please use 'warmup' instead.", 
         call. = FALSE)
  }
}

.validate_sampler_params <-
  function(x,
           n_chain,
           n_iter,
           algorithm = c("NUTS", "HMC")) {
    if (is.null(x))
      return(list(NA))
    
    if (!is.list(x) || length(x) != n_chain || !all(sapply(x, is.matrix)))
      stop("'sampler_params' must be a list of matrices with one matrix per chain.")
    if (!all(sapply(x, function(xj) nrow(xj) == n_iter)))
      stop("Each matrix in 'sampler_params' must have number of rows ",
           "equal to number of iterations in 'X'.")
    
    nms <- sapply(x, colnames)
    if (!is.character(nms))
      stop("Matrices in 'sampler_params' must have column names.")
    for (j in seq_along(x)) {
      if (!all.equal(nms[, 1], nms[, j]))
        stop("All matrices in 'sampler_params' must have the same column names.")
    }
    
    alg <- match.arg(algorithm)
    if (alg == "NUTS") {
      nuts_nms <-
        c(
          "accept_stat__",
          "stepsize__",
          "treedepth__",
          "n_leapfrog__",
          "divergent__",
          "energy__"
        )
      if (!all(nms[, 1] %in% nuts_nms))
        stop("For NUTS algorithm the following parameters must be included ", 
             "in 'sampler_params': ", paste(nuts_nms, collapse = ", "))
    }
    
    return(x)
  }

.set_param_dims <- function(param_dims = list(), 
                            param_names = character(length(param_dims))) {
  if (!length(param_dims)) {
    param_dims <- list()
    param_dims[seq_along(param_names)] <- NA
    names(param_dims) <- param_names
    for (i in seq_along(param_names))
      param_dims[[i]] <- numeric(0)
  } else {
    zeros <- sapply(seq_along(param_dims), function(i)
      0 %in% param_dims[[i]])
    for (i in which(zeros))
      param_dims[[i]] <- numeric(0)
  }
  param_dims 
}


# as.shinystan (list) ---------------------------------------------------
#' @describeIn as.shinystan Create a \code{shinystan} object from a
#'   \code{\link{list}} of matrices. Each \code{\link{matrix}} (or 2-D array)
#'   should contain the simulations for an individual chain and all of the
#'   matrices should have the same number of iterations (rows) and parameters
#'   (columns). Parameters should have the same names and be in the same order.
#'   
#' @examples 
#' \dontrun{   
#' ########################
#' ### list of matrices ###
#' ########################
#'
#' # Generate some fake data
#' chain1 <- cbind(beta1 = rnorm(100), beta2 = rnorm(100), sigma = rexp(100))
#' chain2 <- cbind(beta1 = rnorm(100), beta2 = rnorm(100), sigma = rexp(100))
#' sso <- as.shinystan(list(chain1, chain2))
#' launch_shinystan(sso)
#' 
#' # We can also specify some or all of the optional arguments
#' # note: in order to use param_dims we need to rename 'beta1' and 'beta2'
#' # to 'beta[1]' and 'beta[2]'
#' colnames(chain1) <- colnames(chain2) <- c(paste0("beta[",1:2,"]"), "sigma")
#' sso2 <- as.shinystan(list(chain1, chain2), 
#'                      model_name = "Example", warmup = 0, 
#'                      param_dims = list(beta = 2, sigma = 0))
#' launch_shinystan(sso2)
#' }
#'
setMethod(
  "as.shinystan",
  signature = "list",
  definition = function(X,
                        model_name = "unnamed model",
                        warmup = 0, burnin = 0,
                        param_dims = list(),
                        model_code = NULL,
                        note = NULL,
                        sampler_params = NULL, 
                        algorithm = NULL, 
                        max_treedepth = NULL,
                        ...) {
    validate_model_code(model_code)
    if (!length(X))
      stop('List is empty.')
    
    dims <- sapply(X, function(x) length(dim(x)))
    if (!isTRUE(all(dims == 2)))
      stop("All elements of X should be matrices / 2-D arrays.")
    
    nChain <- length(X)
    for (i in seq_len(nChain)) {
      nms <- colnames(as.matrix(X[[i]]))
      if (is.null(nms) || !all(nzchar(nms)))
        stop(
          "Some parameters are missing names. ",
          "Check the column names for the matrices in your list of chains."
        )
    }
    
    if (nChain == 1) {
      n_iter <- nrow(X[[1]])
      param_names <- colnames(X[[1]])
    } else {
      n_iter <- sapply(X, nrow)
      same_iters <- length(unique(n_iter)) == 1
      if (!same_iters)
        stop("Each chain should contain the same number of iterations.")
      cnames <- sapply(X, colnames)
      if (is.array(cnames)) {
        same_params <- identical(cnames[, 1], cnames[, 2])
        param_names <- cnames[, 1]
      } else {
        same_params <- length(unique(cnames)) == 1
        param_names <- cnames
      }
      if (!same_params)
        stop("The parameters for each chain should be in the same order ",
             "and have the same names.")
      n_iter <- n_iter[1]
    }
    param_names <- unique(param_names)
    nParam <- length(param_names)
    out <- array(NA, dim = c(n_iter, nChain, nParam))
    for (i in seq_len(nChain))
      out[, i,] <- X[[i]]

    dimnames(out) <- list(
      iterations = NULL,
      chains = paste0("chain:", seq_len(nChain)),
      parameters = param_names
    )
    as.shinystan(
      out,
      model_name = model_name,
      warmup = .deprecate_burnin(burnin, warmup),
      param_dims = param_dims,
      model_code = model_code,
      note = note,
      sampler_params = sampler_params,
      algorithm = algorithm,
      max_treedepth = max_treedepth,
      ...
    )
  }
)


# as.shinystan (mcmc.list) -----------------------------------------------
setOldClass("mcmc.list")
#' @describeIn as.shinystan Create a \code{shinystan} object from an 
#'   \code{mcmc.list} object (\pkg{coda}).
#' 
setMethod(
  "as.shinystan",
  signature = "mcmc.list",
  definition = function(X,
                        model_name = "unnamed model",
                        warmup = 0, burnin = 0,
                        param_dims = list(),
                        model_code = NULL,
                        note = NULL,
                        ...) {
    check_suggests("coda")
    validate_model_code(model_code)
    
    if (length(X) == 1) {
      return(
        as.shinystan(
          X = list(.mcmclist2matrix(X)),
          model_name = model_name,
          warmup = .deprecate_burnin(burnin, warmup),
          param_dims = param_dims,
          model_code = model_code,
          note = note,
          ...
        )
      )
    }
    
    posterior <- array(
      NA,
      dim = c(coda::niter(X), coda::nvar(X), coda::nchain(X)),
      dimnames = list(
        iter = time(X),
        var = coda::varnames(X),
        chain = coda::chanames(X)
      )
    )
    for (c in seq_len(coda::nchain(X)))
      posterior[, , c] <- X[[c]]
    
    posterior <- aperm(drop(posterior), c(1, 3, 2))
    dimnames(posterior) <- list(
      iterations = seq_len(nrow(posterior)),
      chains = paste0("chain:", seq_len(ncol(posterior))),
      parameters = dimnames(posterior)[[3]]
    )
    param_names <- dimnames(X[[1]])[[2]]
    
    sso <- shinystan(
      model_name = model_name,
      param_names = param_names,
      param_dims = .set_param_dims(param_dims, param_names),
      posterior_sample = posterior,
      summary = shinystan_monitor(posterior, warmup = burnin),
      n_chain = ncol(posterior),
      n_iter = nrow(posterior),
      n_warmup = burnin
    )
    if (!is.null(note))
      sso <- suppressMessages(notes(sso, note = note, replace = TRUE))
    if (!is.null(model_code))
      sso <- suppressMessages(model_code(sso, code = model_code))
    
    return(sso)
  }
)

.mcmclist2matrix <- function(x) {
  # adapted from Coda package
  out <- matrix(nrow = coda::niter(x) * coda::nchain(x), ncol = coda::nvar(x))
  cols <- seq_len(coda::nvar(x))
  for (i in seq_len(coda::nchain(x))) {
    rows <- (i-1)*coda::niter(x) + seq_len(coda::niter(x))
    out[rows, cols] <- x[[i]]
  }
  rownames <- character(ncol(out))
  rownames[cols] <- coda::varnames(x, allow.null = FALSE)
  dimnames(out) <- list(NULL, rownames)
  out
}


# as.shinystan (stanfit) -------------------------------------------------
setClass("stanfit", getClass("stanfit", where = getNamespace("rstan")))

#' @describeIn as.shinystan Create a \code{shinystan} object from a
#'   \code{stanfit} object (\pkg{\link[rstan]{rstan}}). Fewer optional arguments
#'   are available for this method because all important information can be
#'   taken automatically from the \code{stanfit} object.
#'   
#' @param pars For stanfit objects (\pkg{rstan}), an optional character vector 
#'   specifying which parameters should be included in the \code{shinystan}
#'   object.
#'
#' @examples
#' \dontrun{
#' ######################
#' ### stanfit object ###
#' ######################
#' library("rstan")
#' fit <- stan_demo("eight_schools")
#' sso <- as.shinystan(fit, model_name = "example")
#' }
#' 
setMethod(
  "as.shinystan",
  signature = "stanfit",
  definition = function(X,
                        pars,
                        model_name = X@model_name,
                        note = NULL,
                        ...) {
    check_suggests("rstan")
    if (!missing(pars)) {
      any_indiv_els <- any(grepl("[", pars, fixed = TRUE))
      if (any_indiv_els)
        stop("Individual elements of non-scalar parameters not allowed in 'pars'.")
      if (!"lp__" %in% pars)
        pars <- c(pars, "lp__")
    }
      
    posterior <-
      rstan::extract(X,
                     pars = pars,
                     permuted = FALSE,
                     inc_warmup = TRUE)
    
    param_dims <- X@sim$dims_oi
    if (!missing(pars)) {
      pd <- which(names(param_dims) %in% pars)
      if (length(pd))
        param_dims <- param_dims[pd]
    }
    
    sso <- shinystan(
      model_name = model_name,
      param_names = dimnames(posterior)[[3L]],
      param_dims = param_dims,
      posterior_sample = posterior,
      summary = .rstan_summary(X, pars = pars),
      sampler_params = .rstan_sampler_params(X),
      n_chain = ncol(X),
      n_iter = nrow(posterior),
      n_warmup = .rstan_warmup(X),
      model_code = rstan::get_stancode(X),
      misc = list(
        max_td = .rstan_max_treedepth(X),
        stan_method = .stan_method(X),
        stan_algorithm = .stan_algorithm(X),
        sso_version = utils::packageVersion("shinystan")
      )
    )
    sso <- .rename_scalar(sso, oldname = "lp__", newname = "log-posterior")
    if (!is.null(note))
      sso <- suppressMessages(notes(sso, note, replace = TRUE))

    return(sso)
  }
)

# rename a scalar parameter in a shinystan object
.rename_scalar <- function(sso, 
                           oldname = "lp__", 
                           newname = "log-posterior") {
  p <- which(sso@param_names == oldname)
  if (identical(integer(0), p)) 
    return(sso)
  
  sso@param_names[p] <- 
    dimnames(sso@posterior_sample)$parameters[p] <-
    names(sso@param_dims)[which(names(sso@param_dims) == oldname)] <- 
    rownames(sso@summary)[p] <- newname
  return(sso)
}

# Get stan_args from stanfit object
# @param x stanfit object
# @param which which of the entries in x@stan_args[[1]] is of interest? If NULL 
#   the full list x@stan_args is returned
.stan_args <- function(x, which = NULL) {
  stan_args <- x@stan_args[[1L]]
  if (!is.null(which)) 
    return(stan_args[[which]])
  stan_args
}

# Check if model was fit using cmdstan rather than rstan
# @param x stanfit object
.from_cmdstan <- function(x) {
  isTRUE("engine" %in% names(.stan_args(x)))
}

# Check if model was restored from a CSV file generated by rstan
# @param x stanfit object
.from_rstan_csv <- function(x) {
  isTRUE("sampler_t" %in% names(.stan_args(x)))
}

#Get the stan method (variatinal, sampling,optimization)
.stan_method <- function(x) {
  if(.from_rstan_csv(x)) {
    "sampling" #I assume there is no way to generate sample file withou sampling
  } else {
    .stan_args(x, "method")
  }
}


# Check if model fit using variational algorithm
# @param x stanfit object
.used_vb <- function(x) {
  isTRUE(.stan_args(x, "method") == "variational")
}

# Check which algorithm was used to fit model
# @param x stanfit object
.stan_algorithm <- function(x) {
  algo <- if (.from_cmdstan(x)) {
    toupper(.stan_args(x, "engine"))
  } else if (.from_rstan_csv(x)) {
    #When reading the CSV, the engine is of the form NUTS(diag_e) - getting rid of the brackets here
    gsub("\\([^)]*\\)","",.stan_args(x, "sampler_t"))
  } else {
    .stan_args(x, "algorithm")
  }
  
  if (.used_vb(x) || !(algo %in% c("NUTS", "HMC")))
    warning("Many features are only available for models fit using
            algorithm NUTS or algorithm HMC.", call. = FALSE)
  
  algo
}

# Get summary stats from a stanfit object
# @param x stanfit object
# @param pars optional vector of parameter names
.rstan_summary <- function(x, pars) {
  stan_summary <- rstan::summary(x, pars = pars)$summary
  if (!.used_vb(x))
    return(stan_summary)
  cbind(stan_summary, Rhat = NA, n_eff = NA, se_mean = NA)
}

# Get sampler params from a stanfit object
# @param x stanfit object
.rstan_sampler_params <- function(x) {
  if (.used_vb(x))
    return(list(NA))
  sp <- suppressWarnings(rstan::get_sampler_params(x))
  sp <- .rename_sampler_param(sp, 
                              oldname = "n_divergent__", 
                              newname = "divergent__")
  sp
}

# @param x list of sampler param arrays
.rename_sampler_param <- function(x, oldname, newname) {
  if (!identical(x, list(NA))) {
    for (j in seq_along(x)) {
      sel <- which(colnames(x[[j]]) == oldname)
      if (length(sel))
        colnames(x[[j]])[sel] <- newname
    }
  }
  return(x)
}

# Calculate correct value for number of warmup iterations
# @param x stanfit object
.rstan_warmup <- function(x) {
  warmup <- if (.from_cmdstan(x))
    x@sim$warmup2[1L] else x@sim$warmup
  
  saved <- .stan_args(x, "save_warmup")
  if (!is.null(saved) && !saved) 
    warmup <- 0
  
  if (.from_cmdstan(x)) 
    return(warmup)
  
  floor(warmup / x@sim$thin)
}

# Get value of max_treedepth parameter from stanfit object
# @param x stanfit object
.rstan_max_treedepth <- function(x) {
  cntrl <- .stan_args(x, "control")
  if (is.null(cntrl)) {
    max_td <- 10
  } else {
    max_td <- cntrl$max_treedepth
    if (is.null(max_td))
      max_td <- 10
  }
  max_td
}



# as.shinystan (stanreg) -------------------------------------------------
setOldClass("stanreg")
#' @describeIn as.shinystan Create a \code{shinystan} object from a
#'   \code{stanreg} object (\pkg{\link[rstanarm]{rstanarm}}).
#'   
#' @param ppd For \code{stanreg} objects (\pkg{rstanarm}), \code{ppd} 
#'   (logical) indicates whether to draw from the posterior predictive 
#'   distribution before launching the app. The default is \code{TRUE}, 
#'   although for very large objects it can be convenient to set it to 
#'   \code{FALSE} as drawing from the posterior predictive distribution can be 
#'   time consuming. If \code{ppd} is \code{TRUE} then graphical posterior
#'   predictive checks are available when 'ShinyStan' is launched.
#' @param seed Passed to \code{\link[rstanarm]{pp_check}} (\pkg{rstanarm}) if 
#'   \code{ppd} is \code{TRUE}.
#'   
#' @examples
#' \dontrun{
#' ######################
#' ### stanreg object ###
#' ######################
#' library("rstanarm")
#' example("example_model")
#' sso <- as.shinystan(example_model)
#' launch_shinystan(sso)
#' }
#'
setMethod(
  "as.shinystan",
  signature = "stanreg",
  definition = function(X,
                        ppd = TRUE,
                        seed = 1234,
                        model_name = NULL,
                        note = NULL,
                        ...) {
    check_suggests("rstanarm")
    sso <- as.shinystan(X$stanfit, ...)
    
    mname <- if (!is.null(model_name))
      model_name else paste0("rstanarm model (", sso@model_name, ")")
    sso <- suppressMessages(model_name(sso, mname))
    
    if (!is.null(note))
      sso <- suppressMessages(notes(sso, note, replace = TRUE))
    
    param_names <- slot(sso, "param_names")
    sel <- grep(":_NEW_", dimnames(slot(sso, "posterior_sample"))[[3L]], 
                fixed = TRUE)
    if (length(sel)) {
      param_names <- param_names[-sel]
      slot(sso, "posterior_sample") <- 
        slot(sso, "posterior_sample")[, , -sel, drop = FALSE]
      slot(sso, "summary")  <- 
        slot(sso, "summary")[-sel, , drop = FALSE]
    }
    param_dims <- rep(list(numeric(0)), length(param_names))
    names(param_dims) <- param_names
    
    slot(sso, "param_names") <- param_names
    slot(sso, "param_dims") <- param_dims
    slot(sso, "misc")[["stanreg"]] <- TRUE
    if (isTRUE(ppd))
      slot(sso, "misc")[["pp_check_plots"]] <- .rstanarm_pp_checks(X, seed)
    
    return(sso)
  }
)

.rstanarm_pp_checks <- function(X, seed, ...) {
  message(
    "\nHang on... preparing graphical posterior predictive checks for rstanarm model.",
    "\nSee help('shinystan', 'rstanarm') for how to disable this feature."
  )
  ppc <- rstanarm::pp_check
  pp_check_plots <- list()
  
  pp_check_plots[["pp_check_hist"]] <-
    do.call("ppc",
            list(
              object = X,
              plotfun = "hist",
              nreps = 8,
              seed = seed
            ))
  pp_check_plots[["pp_check_dens"]] <-
    do.call("ppc",
            list(
              object = X,
              plotfun = "dens_overlay",
              nreps = 50,
              seed = seed
            ))
  pp_check_plots[["pp_check_resid"]] <-
    do.call("ppc", 
            list(
              object = X,
              plotfun = "error_hist",
              nreps = 8,
              seed = seed
            ))
  pp_check_plots[["pp_check_scatter"]] <-
    do.call("ppc",
            list(
              object = X,
              plotfun = "scatter_avg",
              nreps = NULL,
              seed = seed
            ))
  pp_check_plots[["pp_check_stat_mean"]] <-
    do.call("ppc",
            list(
              object = X,
              plotfun = "stat",
              stat = "mean",
              seed = seed
            ))
  pp_check_plots[["pp_check_stat_sd"]] <-
    do.call("ppc", 
            list(
              object = X,
              plotfun = "stat",
              stat = "sd",
              seed = seed
            ))
  pp_check_plots[["pp_check_stat_min"]] <-
    do.call("ppc", 
            list(
              object = X,
              plotfun = "stat",
              stat = "min",
              seed = seed
            ))
  pp_check_plots[["pp_check_stat_max"]] <-
    do.call("ppc", 
            list(
              object = X,
              plotfun = "stat",
              stat = "max",
              seed = seed
            ))
  
  pp_check_plots
}


# as.shinystan (CmdStanMCMC) -----------------------------------------------
setOldClass("CmdStanMCMC")
#' @describeIn as.shinystan Create a \code{shinystan} object from a 
#'   \code{CmdStanMCMC} object (\pkg{cmdstanr}).
#' 
setMethod(
  "as.shinystan",
  signature = "CmdStanMCMC",
  definition = function(X,
                        pars = NULL,
                        model_name = NULL,
                        note = NULL,
                        ...) {
    check_suggests("cmdstanr")
    check_suggests("posterior")
    if (is.null(model_name)) {
      model_name <- X$runset$model_name()
    }
    
    if (X$metadata()$save_warmup == 0) {
      draws <- unclass(X$draws(pars))
      sampler_diagnostics <- X$sampler_diagnostics()
      n_warmup <- 0
    } else {
      draws <- unclass(X$draws(pars, inc_warmup = TRUE))
      sampler_diagnostics <- X$sampler_diagnostics(inc_warmup = TRUE)
      n_warmup <- X$metadata()$iter_warmup
    }
    
    sampler_params <- list()
    for (j in seq_len(dim(sampler_diagnostics)[2])) {
      sampler_params[[j]] <- posterior::as_draws_matrix(sampler_diagnostics[, j ,])
    }
    
    as.shinystan(
      draws,
      model_name = model_name,
      warmup = n_warmup,
      param_dims = X$metadata()$stan_variable_dims,
      model_code = NULL,
      note = note,
      sampler_params = sampler_params, 
      algorithm = "NUTS",
      max_treedepth = X$metadata()$max_treedepth
    )
  }
)

library(shinystan)
library(rstanarm)
context("Misc")

test_that("options set when package loads", {
  expect_false(getOption("shinystan.rstudio"))
  options(shinystan.rstudio = TRUE)
  expect_true(getOption("shinystan.rstudio"))
  options(shinystan.rstudio = FALSE)
})

test_that("Can work with CSV files created by RStan", {
  skip_if_not_installed("rstanarm")
  test_data <- data.frame(Y = rnorm(10))
  samples_tmp <- tempfile()
  fit <- rstanarm::stan_glm(Y ~ 1, data = test_data, sample_file = samples_tmp, chains = 1, iter = 10, sample_file = samples_tmp)
  fit_csv <- read_stan_csv(samples_tmp)

  expect_identical(.stan_algorithm(fit_csv), .stan_algorithm(fit$stanfit))
  expect_identical(.stan_method(fit_csv), .stan_method(fit$stanfit))
})
  

library(shinystan)
library(coda)
suppressPackageStartupMessages(library(rstan))

sso <- eight_schools
array1 <- array(rnorm(300), dim = c(25, 4, 3))
array2 <- array(rnorm(300), dim = c(100, 3))
chains1 <- list(chain1 = cbind(beta1 = rnorm(100), beta2 = rnorm(100), sigma = rexp(100)), 
                chain2 = cbind(beta1 = rnorm(100), beta2 = rnorm(100), sigma = rexp(100)))

data(line, package = "coda")
mcmc1 <- line
mcmc2 <- line[[1L]]

if (requireNamespace("rstanarm", quietly = TRUE)) {
  suppressPackageStartupMessages(library(rstanarm))
  suppressWarnings(capture.output(
    stanreg1 <- stan_glmer(mpg ~ wt + (1 + wt|cyl), data = mtcars, 
                           seed = 12345, iter = 200, chains = 2, refresh = 0)
  ))
  stanfit1 <- stanreg1$stanfit
}


# load 'old_sso', a shinystan object created by previous shinystan version
load("old_sso_for_tests.rda")

context("Checking shinystan objects")
# sso_check ---------------------------------------------------------------
test_that("sso_check throws errors", {
  expect_error(sso_check(array1))
  expect_error(sso_check(chain2))
  expect_error(sso_check(chains1))
  
  expect_true(sso_check(sso))
  expect_true(sso_check(as.shinystan(array1)))
  
  expect_error(sso_check(old_sso), 
               regexp = "use the 'update_sso' function to update your object")
})


# is.shinystan ------------------------------------------------------------
test_that("is.shinystan, is.stanfit, is.stanreg work", {
  skip_if_not_installed("rstanarm")
  expect_true(is.shinystan(sso))
  expect_false(is.shinystan(sso@posterior_sample))
  
  expect_true(is.stanfit(stanfit1))
  expect_false(is.stanfit(stanreg1))
  
  expect_true(is.stanreg(stanreg1))
  expect_false(is.stanreg(stanfit1))
})


context("Creating shinystan objects")
# as.shinystan helpers ----------------------------------------------------
test_that("as.shinystan stanfit helpers work", {
  skip_if_not_installed("rstanarm")
  expect_is(.rstan_max_treedepth(stanfit1), "integer")
  expect_equal(.rstan_warmup(stanfit1), 0)
  expect_equal(length(.rstan_sampler_params(stanfit1)), ncol(stanfit1))
  expect_is(.rstan_summary(stanfit1), "matrix")
  expect_identical(.stan_algorithm(stanfit1), "NUTS")
  expect_false(.used_vb(stanfit1))
  expect_false(.from_cmdstan(stanfit1))
  expect_is(.stan_args(stanfit1), "list")
  expect_true(all(c("iter", "seed", "warmup") %in% names(.stan_args(stanfit1))))
  
  stanfit1@stan_args[[1]]$method <- "variational"
  expect_true(.used_vb(stanfit1))
  expect_identical(.rstan_sampler_params(stanfit1), list(NA))
  
  stanfit1@stan_args[[1]]$control$max_treedepth <- NULL
  expect_equal(.rstan_max_treedepth(stanfit1), 10)
})


test_that("deprecation of burnin works properly", {
  expect_error(as.shinystan(array1, warmup = 2, burnin = 3), 
               "can't both be specified")
  expect_warning(x <- as.shinystan(array1, burnin = 8), 
                 "Use the 'warmup' argument instead")
  expect_equal(x@n_warmup, 8)
  expect_silent(x <- as.shinystan(array1, warmup = 7))
  expect_equal(x@n_warmup, 7)
})



# as.shinystan ------------------------------------------------------------
test_that("as.shinystan (array) creates sso", {
  expect_s4_class(x <- as.shinystan(array1, model_name = "test", note = "test"), "shinystan")
  expect_identical(sso_version(x), utils::packageVersion("shinystan"))
  expect_equal(x@model_name, "test")
  expect_equal(x@user_model_info, "test")
  
  # with sampler_params
  samp <- sso@posterior_sample
  sp <- sso@sampler_params
  x <- as.shinystan(samp, sampler_params = sp, warmup = 759, 
                    max_treedepth = 14, algorithm = "NUTS")
  expect_s4_class(x, "shinystan")
  expect_equal(x@n_warmup, 759)
  expect_equal(x@n_chain, dim(samp)[2])
  expect_equal(x@n_iter, dim(samp)[1])
  expect_equivalent(x@posterior_sample, samp)
  expect_equal(x@misc$max_td, 14)
  expect_equal(x@misc$stan_algorithm, "NUTS")
})
test_that("as.shinystan (mcmc.list) creates sso", {
  expect_is(x <- as.shinystan(mcmc1, model_name = "test", note = "test", model_code = "test"), "shinystan")
  expect_is(as.shinystan(mcmc1[1]), "shinystan")
  expect_identical(sso_version(x), utils::packageVersion("shinystan"))
})  
test_that("as.shinystan (list of matrices) creates sso", {
  expect_is(x <- as.shinystan(chains1, model_code = "test"), "shinystan")
  expect_is(as.shinystan(chains1[1]), "shinystan")
  colnames(chains1[[1]]) <- colnames(chains1[[2]]) <- c(paste0("beta[",1:2,"]"), "sigma")
  sso2 <- as.shinystan(chains1, param_dims = list(beta = 2, sigma = 0))
  expect_identical(sso2@param_dims, list(beta = 2, sigma = numeric(0)))
  expect_identical(sso_version(x), utils::packageVersion("shinystan"))
  
  # with sampler_params
  samp_list <- list()
  samp <- sso@posterior_sample
  for (j in 1:ncol(samp)) samp_list[[j]] <- samp[, j, ]
  sp <- sso@sampler_params
  x <- as.shinystan(samp_list, sampler_params = sp, warmup = 1000, 
                    max_treedepth = 11, algorithm = "NUTS")
  expect_s4_class(x, "shinystan")
})
test_that("as.shinystan (stanreg) creates sso", {
  skip_if_not_installed("rstanarm")
  expect_message(x <- as.shinystan(stanreg1, model_name = "test"), 
                 "preparing graphical posterior predictive checks")
  expect_is(x, "shinystan")
  
  # check that ppc plots created
  ppc <- x@misc$pp_check_plots
  expect_type(ppc, "list")
  expect_s3_class(ppc[[1]], "ggplot")
  
  # without ppd
  x <- as.shinystan(stanreg1, ppd = FALSE)
  expect_null(x@misc$pp_check_plots)
})
test_that("as.shinystan (stanfit) creates sso", {
  skip_if_not_installed("rstanarm")
  expect_is(x <- as.shinystan(stanfit1, model_name = "test", note = "test"), "shinystan")
  expect_identical(sso_version(x), utils::packageVersion("shinystan"))
})

test_that("as.shinystan throws errors", {
  expect_error(as.shinystan(array2))
  expect_error(as.shinystan(mcmc2))
})

test_that("as.shinystan arguments works with rstanarm example", {
  skip_if_not_installed("rstanarm")
  sso1 <- as.shinystan(stanreg1)
  sso2 <- as.shinystan(stanreg1, ppd = FALSE)
  expect_is(sso1, "shinystan")
  expect_is(sso2, "shinystan")
  expect_false(is.null(sso1@misc$pp_check_plots))
  expect_null(sso2@misc$pp_check_plots)
})

test_that("as.shinystan 'pars' argument works with rstan example", {
  # load 'stanfit2' saved stanfit object
  load("stanfit2_for_tests.rda")
  
  expect_error(as.shinystan(stanfit2, pars = c("alpha[1,1]", "lp__")), 
               "elements of non-scalar parameters not allowed")
  
  sso0 <- as.shinystan(stanfit2)
  sso1 <- as.shinystan(stanfit2, pars = "alpha")
  sso2 <- as.shinystan(stanfit2, pars = "beta")
  sso3 <- as.shinystan(stanfit2, pars = c("alpha", "beta"))
  
  expect_identical(sso0, sso3)

  sso1names <- c("alpha[1,1]", "alpha[2,1]", "alpha[1,2]", "alpha[2,2]",
                 "alpha[1,3]", "alpha[2,3]", "log-posterior")
  expect_identical(sso1@param_names, sso1names)
  expect_identical(rownames(sso1@summary), sort(sso1names))
  expect_identical(sso2@param_names, c("beta", "log-posterior"))
  expect_identical(rownames(sso2@summary), c("beta", "log-posterior"))
  
  expect_equal(dim(sso1@posterior_sample), c(200, 2, 7))
  expect_equal(dim(sso2@posterior_sample), c(200, 2, 2))
})


test_that("as.shinystan works with CmdStanMCMC objects", {
  skip_on_cran()
  skip_if_not_installed("cmdstanr")
  fit <- try(cmdstanr::cmdstanr_example("schools", save_warmup = TRUE, iter_warmup = 500, chains = 2))
  if (!inherits(fit, "try-error")) {
    sso <- as.shinystan(fit)
    expect_s4_class(sso, "shinystan")
    expect_equal(sso@model_name, "schools")
    expect_equal(sso@param_names, c("log-posterior", "mu", "tau", paste0("theta[", 1:8, "]")))
    expect_equal(sso@n_chain, 2)
    expect_equal(sso@n_warmup, 500)
  }
})


# update_sso ---------------------------------------------------------------
context("Updating shinystan objects")
test_that("update_sso errors and messages are correct", {
  expect_error(update_sso(1234))
  expect_message(sso2 <- update_sso(sso), "already up-to-date")
  expect_is(sso2, "shinystan")
  
  expect_message(sso3 <- update_sso(old_sso), "object updated")
  expect_is(sso3, "shinystan")
  expect_identical(sso_version(sso3), utils::packageVersion("shinystan"))
  
  sso3@misc[["sso_version"]] <- "2.9.5"
  expect_error(update_sso(sso3), 
               regexp = "was created using a more recent version of shinystan")
})

lp_name <- "log-posterior"
lp_lab <- "Log Posterior"
metrop_lab <- "Mean Metrop. Acceptance"
stepsize_lab <- "Sampled Step Size"
treedepth_lab <- "Treedepth"
ndivergent_lab <- "Divergent"

sp_nuts_check <- reactive({
  validate(
    need(STAN_ALGORITHM == "NUTS", message = "Only available for algorithm = NUTS"),
    need(input$diagnostic_chain, message = "Loading...")
  )
})
lp_check <- reactive({
  validate(
    need(lp_name %in% dimnames(SAMPS_post_warmup)[[3]], 
         message = "Plot not displayed\n(Draws for 'lp__' or 'log-posterior' not found)")
  )
})
diagnostic_chain <- reactive({
  validate(need(input$diagnostic_chain, message = "Waiting for chain (0 for all)"))
  input$diagnostic_chain
})
diagnostic_param <- reactive({
  validate(need(input$diagnostic_param, message = "Waiting for parameter"))
  input$diagnostic_param
})


diagnostic_param_transform <-
  eventReactive(input$diagnostic_param_transform_go > 0,
                input$diagnostic_param_transform)

selected_range <- debounce({
  panel <- input$diagnostics_navlist
  nm <- switch(
    panel,
    "By model parameter" = "parameter",
    "Sample information" = "lp",
    "Treedepth information" = "treedepth",
    "Step size information" = "stepsize",
    "Divergence information" = "divergent"
  )
  input_nm <- paste0("dynamic_trace_diagnostic_", nm, "_out_date_window")
  validate(need(input[[input_nm]], "Updating selected range"))
  sel <- input[[input_nm]]
  high <- as.integer(strsplit(sel[[2]], "[-]")[[1]][1])
  low <- as.integer(if (is.nan(sel[[1]])) "1" else strsplit(sel[[1]], "[-]")[[1]][1])
  low:high
}, millis = 125)

# stepsize ----------------------------------------------------------------
dynamic_trace_diagnostic_stepsize <- reactive({
  sp_nuts_check()
  chain <- diagnostic_chain()
  samps <- .stepsize_pw[,-1]
  lab <- "Sampled Step Size"
  stack <- FALSE
  `%>%` <- dygraphs::`%>%`
  graph <- do.call(
    ".dynamic_trace_diagnostics",
    args = list(
      param_samps = samps,
      param_name = lab,
      chain = chain,
      stack = stack,
      group = "stepsize_information"
    )
  )
  graph %>% dygraphs::dyAxis("y", pixelsPerLabel = 40)
})
stepsize_vs_lp <- reactive({
  sp_nuts_check()
  lp_check()
  chain <- diagnostic_chain()
  sel <- selected_range()
  stepsize <- .stepsize_pw[if (!is.null(sel)) sel,-1L, drop = FALSE] # drop iterations column
  lp <- SAMPS_post_warmup[if (!is.null(sel)) sel, , lp_name]
  .sampler_param_vs_param(
    p = lp,
    sp = stepsize,
    p_lab = lp_lab,
    sp_lab = stepsize_lab,
    chain = chain,
    violin = TRUE
  )
})
stepsize_vs_accept_stat <- reactive({
  sp_nuts_check()
  chain <- diagnostic_chain()
  sel <- selected_range()
  df_ss <- .stepsize_pw[if (!is.null(sel)) sel,-1L, drop = FALSE] # drop iterations column
  df_as <- .accept_stat_pw[if (!is.null(sel)) sel,-1L, drop = FALSE]
  .sampler_param_vs_sampler_param_violin(
    round(df_ss, 4),
    df_as,
    lab_x = stepsize_lab,
    lab_y = metrop_lab,
    chain = chain
  )
})


# sample (accept_stat, lp) ------------------------------------------------
dynamic_trace_diagnostic_lp <- reactive({
  sp_nuts_check()
  lp_check()
  chain <- diagnostic_chain()
  samps <- SAMPS_post_warmup[, , lp_name]
  lab <- "Log Posterior"
  stack <- FALSE
  do.call(
    ".dynamic_trace_diagnostics",
    args = list(
      param_samps = samps,
      param_name = lab,
      chain = chain,
      stack = stack,
      group = "sample_information"
    )
  )
})
dynamic_trace_diagnostic_accept_stat <- reactive({
  sp_nuts_check()
  chain <- diagnostic_chain()
  samps <- .accept_stat_pw[,-1]
  stack <- FALSE
  do.call(
    ".dynamic_trace_diagnostics",
    args = list(
      param_samps = samps,
      param_name = metrop_lab,
      chain = chain,
      stack = stack,
      group = "sample_information"
    )
  )
})
lp_hist <- reactive({
  sp_nuts_check()
  lp_check()
  chain <- diagnostic_chain()
  sel <- selected_range()
  lp <- SAMPS_post_warmup[if (!is.null(sel)) sel,, lp_name]
  df <- as.data.frame(cbind(iterations = 1:NROW(lp), lp))
  .p_hist(df, lab = lp_lab, chain)
})
accept_stat_hist <- reactive({
  sp_nuts_check()
  sel <- selected_range()
  df <- .accept_stat_pw[if (!is.null(sel)) sel,, drop=FALSE]
  chain <- diagnostic_chain()
  .p_hist(df, lab = metrop_lab, chain) + xlim(0,1)
})
accept_stat_vs_lp <- reactive({
  sp_nuts_check()
  lp_check()
  sel <- selected_range()
  metrop <- .accept_stat_pw[if (!is.null(sel)) sel, -1L, drop = FALSE] # drop iterations column
  lp <- SAMPS_post_warmup[if (!is.null(sel)) sel, , lp_name]
  chain <- input$diagnostic_chain
  divergent <- .ndivergent_pw[if (!is.null(sel)) sel, -1L, drop = FALSE]
  td <- .treedepth_pw[if (!is.null(sel)) sel, -1L, drop = FALSE]
  hit_max_td <- apply(td, 2L, function(y) as.numeric(y == MISC$max_td))
  .sampler_param_vs_param(
    p = lp,
    sp = metrop,
    divergent = divergent,
    hit_max_td = as.data.frame(hit_max_td),
    p_lab = lp_lab,
    sp_lab = metrop_lab,
    chain = chain
  )
})


# treedepth ---------------------------------------------------------------
dynamic_trace_diagnostic_treedepth <- reactive({
  sp_nuts_check()
  chain <- diagnostic_chain()
  samps <- .treedepth_pw[, -1]
  max_td <- MISC$max_td
  lab <- treedepth_lab
  stack <- FALSE
  graph <- do.call(
    ".dynamic_trace_diagnostics",
    args = list(
      param_samps = samps,
      param_name = lab,
      chain = chain,
      stack = stack,
      group = "treedepth_information"
    )
  )
  
  `%>%` <- dygraphs::`%>%`
  graph %>%
    dygraphs::dyLimit(
      limit = max_td,
      label = "max_treedepth",
      color = "black",
      labelLoc = "right",
      strokePattern = "solid"
    ) %>%
    dygraphs::dyAxis(
      "y",
      valueRange = c(0, max_td * 8 / 7),
      pixelsPerLabel = 20,
      drawGrid = FALSE
    )
})
treedepth_ndivergent_hist <- reactive({
  sp_nuts_check()
  chain <- diagnostic_chain()
  sel <- selected_range()
  df_td <- .treedepth_pw[if (!is.null(sel)) sel, , drop=FALSE]
  df_nd <- .ndivergent_pw[if (!is.null(sel)) sel, , drop=FALSE]
  .treedepth_ndivergent_hist(df_td, df_nd, chain = chain, divergent = "All")
})
treedepth_ndivergent0_hist <- reactive({
  sp_nuts_check()
  chain <- diagnostic_chain()
  sel <- selected_range()
  df_td <- .treedepth_pw[if (!is.null(sel)) sel, , drop=FALSE]
  df_nd <- .ndivergent_pw[if (!is.null(sel)) sel, , drop=FALSE]
  .treedepth_ndivergent_hist(df_td, df_nd, chain = chain, divergent = 0)
})
treedepth_ndivergent1_hist <- reactive({
  sp_nuts_check()
  chain <- diagnostic_chain()
  sel <- selected_range()
  df_td <- .treedepth_pw[if (!is.null(sel)) sel, , drop=FALSE]
  df_nd <- .ndivergent_pw[if (!is.null(sel)) sel, , drop=FALSE]
  .treedepth_ndivergent_hist(df_td, df_nd, chain = chain, divergent = 1)
})
treedepth_vs_lp <- reactive({
  sp_nuts_check()
  lp_check()
  chain <- diagnostic_chain()
  sel <- selected_range()
  treedepth <- .treedepth_pw[if (!is.null(sel)) sel,-1L, drop = FALSE] # drop iterations column
  lp <- SAMPS_post_warmup[if (!is.null(sel)) sel, , lp_name]
  .sampler_param_vs_param(
    p = lp,
    sp = treedepth,
    p_lab = lp_lab,
    sp_lab = treedepth_lab,
    chain = chain,
    violin = TRUE
  )
})
treedepth_vs_accept_stat <- reactive({
  sp_nuts_check()
  chain <- diagnostic_chain()
  sel <- selected_range()
  df_td <- .treedepth_pw[if (!is.null(sel)) sel,-1L, drop = FALSE] # drop iterations column
  df_as <- .accept_stat_pw[if (!is.null(sel)) sel,-1L, drop = FALSE]
  .sampler_param_vs_sampler_param_violin(
    df_td,
    df_as,
    lab_x = treedepth_lab,
    lab_y = metrop_lab,
    chain = chain
  )
})


# N divergent -------------------------------------------------------------
dynamic_trace_diagnostic_ndivergent <- reactive({
  sp_nuts_check()
  chain <- diagnostic_chain()
  samps <- .ndivergent_pw[,-1]
  stack <- FALSE
  graph <- do.call(
    ".dynamic_trace_diagnostics",
    args = list(
      param_samps = samps,
      param_name = ndivergent_lab,
      chain = chain,
      stack = stack,
      group = "ndivergent_information"
    )
  )
  
  `%>%` <- dygraphs::`%>%`
  graph %>% dygraphs::dyAxis(
    "y",
    valueRange = c(0, 1.1),
    pixelsPerLabel = 1e4,
    drawGrid = FALSE
  )
})
ndivergent_vs_lp <- reactive({
  sp_nuts_check()
  lp_check()
  chain <- diagnostic_chain()
  sel <- selected_range()
  ndivergent <- .ndivergent_pw[if (!is.null(sel)) sel,-1L, drop = FALSE] # drop iterations column
  lp <- SAMPS_post_warmup[if (!is.null(sel)) sel, , lp_name]
  .sampler_param_vs_param(
    p = lp,
    sp = ndivergent,
    p_lab = lp_lab,
    sp_lab = ndivergent_lab,
    chain = chain,
    violin = TRUE
  )
})
ndivergent_vs_accept_stat <- reactive({
  sp_nuts_check()
  chain <- diagnostic_chain()
  sel <- selected_range()
  df_nd <- .ndivergent_pw[if (!is.null(sel)) sel,-1L, drop = FALSE] # drop iterations column
  df_as <- .accept_stat_pw[if (!is.null(sel)) sel,-1L, drop = FALSE]
  .sampler_param_vs_sampler_param_violin(
    df_nd,
    df_as,
    lab_x = ndivergent_lab,
    lab_y = metrop_lab,
    chain = chain
  )
})



# energy ------------------------------------------------------------------
energy_hist <- reactive({
  sp_nuts_check()
  chain <- diagnostic_chain()
  np <- bayesplot::nuts_params(SAMPLER_PARAMS_post_warmup, pars = "energy__")
  if (chain != 0)
    np <- subset(np, Chain == chain)
  
  schm <- unlist(bayesplot::color_scheme_get("brightblue"))
  schm["light"] <- base_fill
  schm["mid"] <- overlay_fill
  schm["light_highlight"] <- vline_base_clr
  schm["mid_highlight"] <- pt_outline_clr
  bayesplot::color_scheme_set(unname(schm))
  
  bayesplot::mcmc_nuts_energy(np, merge_chains = isTRUE(chain != 0)) + 
    ggplot2::facet_wrap(~ Chain, labeller = "label_both") + 
    thm_no_yaxs + 
    bayesplot::facet_bg(FALSE) + 
    bayesplot::facet_text(size = rel(1)) + 
    bayesplot::legend_move("right") + 
    theme(legend.text.align = 0, legend.text = element_text(size = rel(1.5)))
})


# model parameter ---------------------------------------------------------
dynamic_trace_diagnostic_parameter <- reactive({
  sp_nuts_check()
  chain <- diagnostic_chain()
  param <- diagnostic_param()
  transform_x <- diagnostic_param_transform()
  samps <- SAMPS_post_warmup[, , param]
  lab <- param
  if (transform_x != "identity") {
    t_x <- get(transform_x)
    samps <- t_x(samps)
    lab <- paste0(transform_x, "(", param, ")")
  }
  stack <- FALSE
  do.call(
    ".dynamic_trace_diagnostics",
    args = list(
      param_samps = samps,
      param_name = paste("Parameter:", lab),
      chain = chain,
      stack = stack
    )
  )
})
param_vs_lp <- reactive({
  sp_nuts_check()
  lp_check()
  param <- diagnostic_param()
  chain <- diagnostic_chain()
  sel <- selected_range()
  lp <- SAMPS_post_warmup[if (!is.null(sel)) sel, , lp_name]
  transform_x <- diagnostic_param_transform()
  samps <- SAMPS_post_warmup[if (!is.null(sel)) sel, , param, drop = FALSE]
  divergent <- .ndivergent_pw[if (!is.null(sel)) sel,-1L, drop = FALSE]
  td <- .treedepth_pw[if (!is.null(sel)) sel,-1L, drop = FALSE]
  hit_max_td <- apply(td, 2L, function(y) as.numeric(y == MISC$max_td))
  lab <- param
  if (transform_x != "identity") {
    t_x <- get(transform_x)
    samps <- t_x(samps)
    lab <- paste0(transform_x, "(", param, ")")
  }
  samps <- as.data.frame(samps)
  .sampler_param_vs_param(
    p = lp,
    sp = samps,
    divergent = divergent,
    hit_max_td = as.data.frame(hit_max_td),
    p_lab = lp_lab,
    sp_lab = lab,
    chain = chain,
    violin = FALSE
  )
})
param_vs_accept_stat <- reactive({
  sp_nuts_check()
  chain <- diagnostic_chain()
  param <- diagnostic_param()
  sel <- selected_range()
  metrop <- .accept_stat_pw[if (!is.null(sel)) sel, -1L, drop=FALSE] # drop iterations column
  transform_x <- diagnostic_param_transform()
  samps <- SAMPS_post_warmup[if (!is.null(sel)) sel,, param]
  divergent <- .ndivergent_pw[if (!is.null(sel)) sel, -1L, drop=FALSE]
  td <- .treedepth_pw[if (!is.null(sel)) sel, -1L, drop=FALSE]
  hit_max_td <- apply(td, 2L, function(y) as.numeric(y == MISC$max_td))
  lab <- param
  if (transform_x != "identity") {
    t_x <- get(transform_x)
    samps <- t_x(samps)
    lab <- paste0(transform_x, "(", param, ")")
  }
  graph <- .sampler_param_vs_param(
    p = samps,
    sp = metrop,
    divergent = divergent,
    hit_max_td = as.data.frame(hit_max_td),
    chain = chain,
    p_lab = lab,
    sp_lab = metrop_lab
  )
  graph + coord_flip()
})
param_vs_stepsize <- reactive({
  sp_nuts_check()
  chain <- diagnostic_chain()
  param <- diagnostic_param()
  sel <- selected_range()
  stepsize <- .stepsize_pw[if (!is.null(sel)) sel, -1L, drop=FALSE] # drop iterations column
  transform_x <- diagnostic_param_transform()
  samps <- SAMPS_post_warmup[if (!is.null(sel)) sel, , param]
  lab <- param
  if (transform_x != "identity") {
    t_x <- get(transform_x)
    samps <- t_x(samps)
    lab <- paste0(transform_x, "(", param, ")")
  }
  .sampler_param_vs_param(
    p = samps,
    sp = stepsize,
    p_lab = lab,
    sp_lab = stepsize_lab,
    chain = chain,
    violin = TRUE
  )
})
param_vs_treedepth <- reactive({
  sp_nuts_check()
  chain <- diagnostic_chain()
  param <- diagnostic_param()
  sel <- selected_range()
  treedepth <- .treedepth_pw[if (!is.null(sel)) sel, -1L, drop=FALSE] # drop iterations column
  transform_x <- diagnostic_param_transform()
  samps <- SAMPS_post_warmup[if (!is.null(sel)) sel, , param]
  lab <- param
  if (transform_x != "identity") {
    t_x <- get(transform_x)
    samps <- t_x(samps)
    lab <- paste0(transform_x, "(", param, ")")
  }
  .sampler_param_vs_param(
    p = samps,
    sp = treedepth,
    p_lab = lab,
    sp_lab = treedepth_lab,
    chain = chain,
    violin = TRUE
  )
})
p_hist <- reactive({
  sp_nuts_check()
  chain <- diagnostic_chain()
  param <- diagnostic_param()
  sel <- selected_range()
  transform_x <- diagnostic_param_transform()
  samps <- SAMPS_post_warmup[if (!is.null(sel)) sel,, param]
  lab <- param
  if (transform_x != "identity") {
    t_x <- get(transform_x)
    samps <- t_x(samps)
    lab <- paste0(transform_x, "(", param, ")")
  }
  df <- as.data.frame(cbind(iterations = 1:NROW(samps), samps))
  .p_hist(df, lab = lab, chain = chain)
})

# outputs ---------------------------------------------------
trace_nms <- c("parameter", "lp", "accept_stat", 
               "treedepth", "stepsize", "ndivergent")
hmc_plots <- c("accept_stat_trace", "accept_stat_hist","accept_stat_vs_lp",
               "lp_trace", "lp_hist", "ndivergent_trace", "treedepth_trace",
               "treedepth_ndivergent_hist","treedepth_ndivergent0_hist", 
               "treedepth_ndivergent1_hist", "treedepth_vs_lp", "ndivergent_vs_lp", 
               "treedepth_vs_accept_stat", "ndivergent_vs_accept_stat", 
               "stepsize_vs_lp", "stepsize_vs_accept_stat", "stepsize_trace", 
               "param_vs_lp", "param_vs_accept_stat", "param_vs_stepsize", 
               "param_vs_treedepth", "p_trace", "p_hist", 
               "energy_hist")
for (j in seq_along(trace_nms)) {
  local({
    fn <- paste0("dynamic_trace_diagnostic_", trace_nms[j]) 
    output[[paste0(fn,"_out")]] <- dygraphs::renderDygraph(do.call(fn, list()))
  })
}
for (i in seq_along(hmc_plots)) {
  local({
    fn <- hmc_plots[i]
    output[[paste0(fn, "_out")]] <- renderPlot({
      x <- suppressMessages(do.call(fn, list()))
      suppress_and_print(x)
    })
  })
}
output$diagnostic_chain_text <- renderText({
  chain <- diagnostic_chain()
  if (chain == 0)
    return("All chains")
  paste("Chain", chain)
})

output$diagnostics_warnings_text <- renderText({
  sp_nuts_check()
  divs <- sum(.ndivergent_pw[, -1])
  hits <- sum(.treedepth_pw[, -1] == MISC$max_td)
  d <- divs > 0
  h <- hits > 0
  if (d && h) {
    msg <- paste(
      "WARNINGS -- Diverging error:", divs, "iterations.",
      "Maximum treedepth reached:", hits, "iterations."
    )
  } else if (d && !h) {
    msg <- paste("WARNINGS -- Diverging error:", divs, "iterations.")
  } else if (!d && h) {
    msg <- paste("WARNINGS -- Maximum treedepth reached:", hits, "iterations.")
  } else {
    msg <- NULL
  }
  msg
})

pp_y_vs_avg_rep <- reactive({
  pp_tests()
  y <- get_y()
  yrep <- get_yrep()
  zoom <- input$pp_zoom_to_zero
  do.call(".pp_y_vs_avg_rep", args = list(
    y = y, 
    colMeans_yrep = colMeans(yrep),
    zoom_to_zero = zoom
  ))
})

output$pp_y_vs_avg_rep_out <- renderPlot({
  pp_y_vs_avg_rep()
}, bg = "transparent")

thm <- theme_classic() %+replace% (no_lgnd + fat_axis + axis_labs + transparent)
thm_no_yaxs <- thm + no_yaxs

.sampler_param_pw <- function(sp, which = "accept_stat__", warmup_val) {
  if (!which %in% colnames(sp[[1]]))
    return(NULL)
  sp_pw <- lapply(1:length(sp), function(i) {
    out <- sp[[i]][, which]
  })
  sp_mat <- do.call("cbind", sp_pw)
  colnames(sp_mat) <- paste0("chain:", 1:ncol(sp_mat))
  sp_mat <- cbind(iterations = seq(from = warmup_val + 1, to = warmup_val + nrow(sp_mat)), 
                  sp_mat)
  as.data.frame(sp_mat)
}


.sampler_param_vs_param <- function(p, sp, divergent = NULL, hit_max_td = NULL, 
                                    p_lab, sp_lab, chain = 0, violin = FALSE, 
                                    smoother = FALSE) {
  xy_labs <- labs(
    y = if (missing(p_lab)) NULL else p_lab, 
    x = if (missing(sp_lab)) NULL else sp_lab
    )
  df <- data.frame(sp = do.call("c", sp), p = c(p))
  if (violin)
    df$sp <- as.factor(round(df$sp, 4))
  if (!is.null(divergent))
    df$divergent <- do.call("c", divergent)
  if (!is.null(hit_max_td))
    df$hit_max_td <- do.call("c", hit_max_td)
  
  base <- ggplot(df, aes(sp,p)) + 
    xy_labs + 
    thm 
  
  if (chain == 0) {
    if (violin) graph <- base + geom_violin(color = vline_base_clr, fill = base_fill) 
    else {
      graph <- base + geom_point(alpha = 1/3, color = pt_outline_clr, fill = base_fill, shape = 19) 
      if (smoother) 
        graph <- graph + stat_smooth(color = overlay_fill, se = FALSE)
      if (!is.null(divergent))
        graph <- graph + geom_point(data = subset(df, divergent == 1), aes(sp,p), 
                                    color = divergent_clr, fill = divergent_fill,
                                    size = 3, shape = div_and_hit_shape)
      if (!is.null(hit_max_td))
        graph <- graph + geom_point(data = subset(df, hit_max_td == 1), aes(sp,p), 
                                    color = hit_max_td_clr, fill = hit_max_td_fill, 
                                    size = 3, shape = div_and_hit_shape)
    }
    return(graph)
  }
  chain_data <- data.frame(sp = sp[, chain], p = p[, chain])
  if (!is.null(divergent))
    chain_data$div <- divergent[, chain]
  if (!is.null(hit_max_td))
    chain_data$hit <- hit_max_td[, chain]
  chain_clr <- color_vector_chain(ncol(sp))[chain]
  chain_fill <- chain_clr
  if (violin) {
    chain_data$sp <- as.factor(round(chain_data$sp, 4))
    graph <- base + 
      geom_violin(color = vline_base_clr, fill = base_fill) + 
      geom_violin(data = chain_data, aes(sp, p), color = chain_clr, 
                  fill = chain_fill, alpha = 0.5)
    return(graph)
  }
  graph <- base + geom_point(alpha = 1/3, color = pt_outline_clr, fill = base_fill, shape = 19)
  if (smoother) 
    graph <- graph + stat_smooth(color = overlay_fill, se = FALSE)
  graph <- graph + geom_point(data = chain_data, aes(sp,p), color = chain_fill, alpha = 0.5)
  if (smoother) 
    graph <- graph + stat_smooth(data = chain_data, aes(sp,p), color = chain_fill, se = FALSE)
  if (!is.null(divergent))
    graph <- graph + geom_point(data = subset(chain_data, div == 1), aes(sp,p), 
                                color = divergent_clr, fill = divergent_fill,
                                size = 3, shape = div_and_hit_shape)
  if (!is.null(hit_max_td))
    graph <- graph + geom_point(data = subset(chain_data, hit == 1), aes(sp,p), 
                                color = hit_max_td_clr, fill = hit_max_td_fill, 
                                size = 3, shape = div_and_hit_shape)
  graph
}

.sampler_param_vs_sampler_param_violin <- function(df_x, df_y, lab_x, lab_y, 
                                                   chain = 0) {
  xy_labs <- labs(y = lab_y, x = lab_x)
  df <- data.frame(x = do.call("c", df_x), y = do.call("c", df_y))
  df$x <- as.factor(df$x)
  
  base <- ggplot(df, aes(x,y)) + 
    xy_labs + 
    thm
  
  graph <- base + geom_violin(color = vline_base_clr, fill = base_fill) 
  if (chain == 0) 
    return(graph)
  
  chain_clr <- color_vector_chain(ncol(df_x))[chain]
  chain_fill <- chain_clr
  chain_data <- data.frame(x = as.factor(df_x[, chain]), y = df_y[, chain])
  graph + geom_violin(data = chain_data, aes(x,y), color = chain_clr, 
                      fill = chain_fill, alpha = 0.5)
}

.p_hist <- function(df, lab, chain = 0) {
  thm <- thm_no_yaxs
  mdf <- reshape2::melt(df, id.vars = "iterations")
  base <- ggplot(mdf, aes(x = value)) + 
    geom_histogram(aes_string(y="..density.."),
                   binwidth = diff(range(mdf$value))/30, fill = base_fill, 
                   color = vline_base_clr, linewidth = 0.2) + 
    labs(x = if (missing(lab)) NULL else lab, y = "") + 
    thm
  
  if (chain == 0) {
    graph <- base + 
      geom_vline(xintercept = mean(mdf$value), color = vline_base_clr, linewidth = .8) + 
      geom_vline(xintercept = median(mdf$value), color = vline_base_clr, lty = 2, linewidth = 1)
    return(graph)
  }
  chain_data <- subset(mdf, variable == paste0("chain:",chain))
  chain_clr <- color_vector_chain(ncol(df) - 1)[chain]
  chain_fill <- chain_clr
  base + thm + geom_histogram(data = chain_data,
                              aes_string(y="..density.."),
                              binwidth = diff(range(chain_data$value))/30,
                              fill = chain_fill, alpha = 0.5) +
    geom_vline(xintercept = mean(chain_data$value), color = chain_clr, linewidth = .8) + 
    geom_vline(xintercept = median(chain_data$value), 
               color = chain_clr, lty = 2, linewidth = 1)
}

.treedepth_ndivergent_hist <- function(df_td, df_nd, chain = 0, divergent = c("All", 0, 1)) {
  plot_title <- theme(plot.title = element_text(size = 11, hjust = 0))
  plot_theme <- thm_no_yaxs + plot_title
  x_lab <- if (divergent == "All") "Treedepth (All)" else paste0("Treedepth (Divergent = ", divergent,")")
  plot_labs <- labs(x = x_lab, y = "") 
  
  mdf_td <- reshape2::melt(df_td, id.vars = "iterations")
  mdf_nd <- reshape2::melt(df_nd, id.vars = "iterations")
  mdf <- cbind(mdf_td, div = mdf_nd$value)
  plot_data <- if (divergent == "All") 
    mdf else subset(mdf, div == divergent)
  if (nrow(plot_data) == 0) 
    return(NULL)
  
  graph <- ggplot(plot_data, aes(x = factor(value)), na.rm = TRUE) + 
    geom_bar(aes(y=..count../sum(..count..)), width=1, fill = base_fill,
             color = vline_base_clr, linewidth = 0.2) + 
    plot_labs + 
    plot_theme
  if (chain == 0) 
    return(graph)
  
  chain_clr <- color_vector_chain(ncol(df_td) - 1)[chain]
  chain_fill <- chain_clr
  chain_data <- subset(plot_data, variable == paste0("chain:",chain))
  graph + geom_bar(data = chain_data, aes(y=..count../sum(..count..)), 
                   fill = chain_fill, alpha = 0.5, width = 1)
}

.dynamic_trace_diagnostics <- function(param_samps, param_name, chain = 0,
                                       stack = FALSE, grid = FALSE, 
                                       group = NULL) {
  dim_samps <- dim(param_samps)
  if (is.null(dim_samps)) {
    nChains <- 1
  } else {
    nChains <- dim_samps[2]
  }
  if (nChains == 1) {
    param_chains <- xts::as.xts(ts(param_samps, start = 1))
  } else {
    if (chain != 0) {
      param_samps <- param_samps[, chain]
      param_chains <- xts::as.xts(ts(param_samps, start = 1))
    } else {
      param_chains <- xts::as.xts(ts(param_samps[,1], start = 1))
      for (i in 2:nChains) {
        param_chains <- cbind(param_chains,
                              xts::as.xts(ts(param_samps[,i], start = 1)))
      }
      colnames(param_chains) <- paste0("Chain", 1:nChains)
    }
  }
  `%>%` <- dygraphs::`%>%`
  y_axis_label_remove <- if (stack) "white" else NULL
  step_plot <- param_name %in% c("Treedepth", "Divergent")
  fill_graph <- param_name == "Divergent"
  stroke_width <- if (step_plot) 0.33 else 0.75
  clrs <- color_vector(nChains) 
  if (chain != 0) clrs <- clrs[chain]
  dygraphs::dygraph(param_chains, xlab = param_name, ylab = NULL, 
                    group = group) %>%
    dygraphs::dyOptions(colors = clrs, stackedGraph = stack, drawGrid = grid,
                        stepPlot = step_plot, #axisLabelFontSize = 11,
                        fillGraph = fill_graph, fillAlpha = 0.5,
                        strokeWidth = 0.75, animatedZooms = TRUE, 
                        drawXAxis = TRUE, drawYAxis = !fill_graph, 
                        drawAxesAtZero = TRUE, axisLineColor = "black", 
                        retainDateWindow = TRUE) %>%
    dygraphs::dyAxis("x", pixelsPerLabel = 1e7, axisLineWidth = 3) %>%
    dygraphs::dyAxis("y", pixelsPerLabel = 30, axisLabelWidth = 30) %>%
    dygraphs::dyLegend(show = "never") %>%
    dygraphs::dyHighlight(highlightCircleSize = 2,
                          highlightSeriesBackgroundAlpha = 1/3,
                          hideOnMouseOut = TRUE,
                          highlightSeriesOpts = list(strokeWidth = 1.5)) %>%
    dygraphs::dyCSS(css = "css/ShinyStan_dygraphs.css")
}

tagList(
  fluidRow(
    column(
      width = 3,
      selectizeInput(
        inputId = "param",
        label = h4("Select parameter"),
        choices = .param_list,
        selected = .param_list[1],
        multiple = FALSE
      )
    ),
    column(
      width = 7, 
      offset = 1, 
      DT::dataTableOutput("parameter_summary_out")
    )
  ),
  navlistPanel(
    well = FALSE,
    widths = c(3, 9),
    
    #### multiview ####
    tabPanel(
      title = "Multiview",
      icon = icon("th-large", lib = "glyphicon"),
      checkboxInput(
        "multiview_warmup",
        label = strong("Include warmup"),
        value = FALSE
      ),
      splitLayout(h5("Kernel Density Estimate"), h5("Autocorrelation")),
      splitLayout(
        plotOutput("multiview_density_out", height = "150"),
        plotOutput("multiview_autocorr_out", height = "150"),
        cellArgs = list(class = "plot_hover_shadow")
      ),
      h5("Trace"),
      dygraphs::dygraphOutput("multiview_trace_out", height = "200px"),
      source_ui("dynamic_trace_helptext.R")
    ),
    
    #### bivariate #####
    tabPanel(
      title = "Bivariate",
      selectizeInput(
        "bivariate_param_y",
        label = strong(style = "color: #006DCC;", "y-axis"),
        choices = rev(.param_list),
        selected = rev(.param_list)[1],
        multiple = FALSE
      ),
      a_options("bivariate"),
      source_ui("bivariate_customize.R"),
      plotOutput("bivariate_plot_out", height = "350px"),
      helpText(
        style = "font-size: 11px",
        "For Stan models using the NUTS algorithm, red points indicate iterations that encountered a divergent transition.",
        "Yellow points indicate a transition that hit the maximum treedepth",
        "rather than terminated its evolution normally."
      ),
      hr(),
      downloadButton("download_bivariate", "ggplot2",  class = "plot-download"),
      downloadButton('save_pdf_bivariate', "pdf", class = "plot-download pdf-download")
    ),
    
    #### trivariate #####
    tabPanel(
      title = "Trivariate",
      source_ui("trivariate_select.R"),
      a_options("trivariate"),
      source_ui("trivariate_customize.R"),
      br(),
      threejs::scatterplotThreeOutput("trivariate_plot_out", height = "400px"),
      helpText(
        style = "font-size: 12px;",
        "Use your mouse and trackpad to rotate the plot and zoom in or out."
      )
    ),
    
    #### density #####
    tabPanel(
      title = "Density",
      a_options("density"),
      source_ui("density_customize.R"),
      plotOutput("density_plot_out", height = "250px"),
      hr(),
      downloadButton("download_density", "ggplot2",  class = "plot-download"),
      downloadButton('save_pdf_density', "pdf", class = "plot-download pdf-download")
    ),
    
    #### histogram #####
    tabPanel(
      title = "Histogram",
      a_options("hist"),
      source_ui("hist_customize.R"),
      plotOutput("hist_plot_out", height = "250px"),
      hr(),
      downloadButton("download_histogram", "ggplot2",  class = "plot-download"),
      downloadButton('save_pdf_histogram', "pdf", class = "plot-download pdf-download")
    )
  )
)

tabsetPanel(
  id = "diagnose_tabset",
  
  #### hmc/nuts plots ####
  tabPanel(
    title = "NUTS (plots)",
    source_ui("diagnostics_customize.R"),
    navlistPanel(
      id = "diagnostics_navlist",
      tabPanel(
        "By model parameter", 
        source_ui("diagnostics_by_parameter.R")
      ),
      tabPanel(
        "Sample information", 
        source_ui("diagnostics_sample.R")
      ),
      tabPanel(
        "Divergence information", 
        source_ui("diagnostics_ndivergent.R")
      ),
      tabPanel(
        "Energy information", 
        source_ui("diagnostics_energy.R")
      ),
      tabPanel(
        "Treedepth information", 
        source_ui("diagnostics_treedepth.R")
      ),
      tabPanel(
        "Step size information", 
        source_ui("diagnostics_stepsize.R")
      ),
      tabPanel(
        "Help", 
        source_ui("diagnostics_help.R")
      ),
      well = FALSE,
      widths = c(2, 10)
    )
  ),
  
  #### hmc/nuts stats ####
  tabPanel(
    title = "HMC/NUTS (stats)",
    h2("Summary of sampler parameters"),
    a_glossary("open_glossary_from_nuts_table"),
    br(),
    source_ui("sampler_stats_customize.R"),
    DT::dataTableOutput("sampler_summary"),
    br()
  ),
  
  #### rhat, n_eff, mcse ####
  tabPanel(
    title = "\\(\\hat{R}, n_{eff}, \\text{se}_{mean}\\)",
    source_ui("rhat_neff_mcse_layout.R")
  ),
  
  #### autocorrelation ####
  tabPanel(
    title = "Autocorrelation",
    source_ui("autocorr_customize.R"),
    wellPanel(fluidRow(
      column(
        width = 8,
        selectizeInput(
          "ac_params",
          width = "100%",
          label = h5("Select or enter parameter names"),
          choices = .param_list_with_groups,
          multiple = TRUE
        )
      ),
      column(
        width = 3, 
        offset = 1, 
        a_options("autocorr")
      )
    )),
    plotOutput("autocorr_plot_out")
  ),
  
  #### ppcheck ####
  tabPanel(
    title = "PPcheck",
    h2("Graphical posterior predictive checks"),
    h6("Experimental feature"),
    source_ui(if (.has_rstanarm_ppcs)
      "pp_navlist_rstanarm.R" else "pp_navlist.R"),
    br()
  )
)

tabsetPanel(
  id = "estimate_tabset",
  
  #### multiparameter plot ####
  tabPanel(
    title = "Parameters plot",
    wellPanel(
      fluidRow(
        column(width = 6,
               uiOutput("ui_multiparam_selectize")),
        column(
          width = 3,
          offset = 1,
          sliderInput(
            "param_plot_ci_level",
            h5("Posterior interval"),
            width = "75%",
            ticks = FALSE,
            min = 50,
            max = 95,
            value = 50,
            step = 5,
            post = "%"
          )
        ),
        column(width = 2,
               a_options("multiparam"))
      ),
      fluidRow(column(
        width = 3,
        offset = 1,
        span(id = "params_to_plot_regex_label",
             "Add parameters by regex search")
      )),
      fluidRow(
        column(
          width = 1,
          actionButton("param_plot_regex", label = "Search", class = "regex-go")
        ),
        column(
          width = 3,
          textInput("params_to_plot_regex", label = NULL, value = "")
        ),
        column(width = 5,
               textOutput("invalid_regex"))
      )
    ),
    source_ui("multiparam_customize.R"),
    plotOutput("multiparam_plot_out", width = "90%"),
    br()
  ),
  
  #### posterior summary statistics ####
  tabPanel(
    "Posterior summary statistics",
    source_ui("table_customize.R"),
    div(DT::dataTableOutput("all_summary_out"),
        style = "overflow-x: auto")
  ),
  
  #### LaTex tables ####
  tabPanel(
    "Generate LaTeX table",
    br(),
    sidebarLayout(
      mainPanel = source_ui("table_latex_main.R"),
      sidebarPanel = source_ui("table_latex_sidebar.R")
    )
  )
)

# shinystan is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 3 of the License, or (at your option) any later
# version.
#
# shinystan is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, see <http://www.gnu.org/licenses/>.


#' \pkg{shinystan} R package ('ShinyStan' graphical user interface)
#'
#' @docType package
#' @name shinystan-package
#'
#' @description
#' \if{html}{
#'   \figure{stanlogo.png}{options: width="50" alt="mc-stan.org"}
#'   \emph{Stan Development Team}
#' }
#'
#' Applied Bayesian data analysis is primarily implemented through the Markov
#' chain Monte Carlo (MCMC) algorithms offered by various software packages.
#' When analyzing a posterior sample obtained by one of these algorithms the
#' first step is to check for signs that the chains have converged to the target
#' distribution and and also for signs that the algorithm might require tuning
#' or might be ill-suited for the given model. There may also be theoretical
#' problems or practical inefficiencies with the specification of the model. The
#' ShinyStan app provides interactive plots and tables helpful for analyzing a
#' posterior sample, with particular attention to identifying potential problems
#' with the performance of the MCMC algorithm or the specification of the model.
#' ShinyStan is powered by the Shiny web application framework by RStudio
#' (\url{https://shiny.rstudio.com/}) and works with the output of MCMC programs
#' written in any programming language (and has extended functionality for
#' models fit using the \pkg{rstan} package and the No-U-Turn sampler).
#'
#' @section ShinyStan has extended functionality for Stan models:
#'
#' Stan (\url{https://mc-stan.org/}) models can be run in \R using the
#' \pkg{\link[rstan]{rstan}} package. Other packages like
#' \pkg{\link[rstanarm]{rstanarm}} and \pkg{brms} provide
#' higher-level interfaces to Stan that use \pkg{rstan} internally.
#'
#' @section Saving and sharing:
#'
#'   The \pkg{shinystan} package allows you to store the basic components of an
#'   entire project (code, posterior samples, graphs, tables, notes) in a single
#'   object, a \code{\link[=as.shinystan]{shinystan object}} (sso, for short).
#'   Users can save many of the plots as \pkg{ggplot2} objects for further
#'   customization and easy integration in reports or post-processing for
#'   publication.
#'
#'   The \code{\link{deploy_shinystan}} function lets you easily deploy your own
#'   ShinyStan apps online for any of your models using the shinyapps.io 
#'   service from 'RStudio'. Each of your apps (each of your models) will have a
#'   unique url and will be compatible with most web browsers.
#'
#' @section License:
#'
#'  The \pkg{shinystan} package is open source licensed under the GNU Public
#'  License, version 3 (GPLv3).
#'
#' @section Demo:
#'
#'  Check out the demo using \code{\link{launch_shinystan_demo}} or try it with
#'  one of your own models using \code{\link{launch_shinystan}}.
#'
#' @section Resources:
#' \itemize{
#'  \item Web page with online documentation
#'    (\url{https://mc-stan.org/shinystan/})
#'  \item Stan Forums on Discourse (\url{https://discourse.mc-stan.org})
#'  \item GitHub issue tracker (\url{https://github.com/stan-dev/shinystan/issues})
#' }
#'
#' @template seealso-as.shinystan
#' @template seealso-demo
#' @template seealso-launch
#' 
#' @template reference-muth
#' @template reference-bayesvis
#'
#' @import ggplot2
#' @import methods
#' @import shiny
#' @import dygraphs
#' @import bayesplot
#' @import threejs
#' @import utils
#' @import xts
#' @importFrom stats acf quantile time var na.omit
#' @importFrom shinythemes shinytheme
#' @importFrom colourpicker colourInput
#' @importFrom shinyjs useShinyjs
#' @importFrom DT datatable
#' @importFrom gridExtra arrangeGrob grid.arrange
#' @importFrom gtools mixedsort
#' @importFrom markdown markdownToHTML
#' @importFrom reshape2 melt
#' @importFrom xtable xtable print.xtable
#'
NULL

fluidRow(
  # column(
  #   width = 4,
  #   helpText(style = "margin-bottom: 2px;", "Table tips: Drag column names to rearrange the table columns.")
  #   # helpText(style = "margin-top: 2px; font-size: 11px;", "Drag column names to rearrange the table columns.")
  # ),
  style = "margin-left: 4px; margin-bottom: 6px;",
  column(
    width = 2,
    div(
      strong(id = "table_digits_txt", "Digits"),
      numericInput(
        "table_digits",
        label = NULL,
        value = 1,
        min = 0,
        max = 7,
        step = 1
      )
    )
  ),
  column(
    width = 2, 
    offset = 8,
    a_glossary("open_glossary_from_table")
  )
)

# posterior summary statistics for a single parameter
parameter_summary <- reactive({
  validate(need(input$param != "", message = FALSE))
  do.call(
    ".param_summary", 
    args = list(
      param = input$param,
      summary = SUMMARY
    )
  )
})

output$param_name <- renderText({
  input$param
})
output$parameter_summary_out <- DT::renderDataTable({
  DT::datatable({
    as.data.frame(round(parameter_summary(), 2))
  }, 
  rownames = FALSE,
  options = list(
    paging = FALSE, 
    searching = FALSE, 
    info = FALSE, 
    ordering = FALSE,
    autoWidth = TRUE,
    columnDefs = list(list(sClass="alignRight", targets ="_all")),
    initComplete = htmlwidgets::JS( # change background color of table header
      'function(settings, json) {
      $(this.api().table().header()).css({"background-color": "transparent", "color": "black"});
      }')
  ))
})

pp_avg_rep_vs_avg_resid_rep <- reactive({
  pp_tests()
  y <- get_y()
  yrep <- get_yrep()
  rowMeans_resids <- rowMeans(y - yrep)
  rowMeans_yrep <- rowMeans(yrep)
  do.call(".pp_avg_rep_vs_avg_resid_rep", args = list(
    rowMeans_yrep = rowMeans_yrep,
    rowMeans_resids = rowMeans_resids
  ))
})

output$pp_avg_rep_vs_avg_resid_rep_out <- renderPlot({
  pp_avg_rep_vs_avg_resid_rep()
}, bg = "transparent")

# summary statistics for sampler parameters -------------------------------
summary_stats_sampler <- reactive({
  validate(
    need(STAN_ALGORITHM %in% c("NUTS", "HMC"), message = "Only available for algorithm = NUTS or HMC"),
    need(input$sampler_warmup, message = "Loading...")
  )
  sp <- if (input$sampler_warmup == "include")
    SAMPLER_PARAMS else SAMPLER_PARAMS_post_warmup
  
  do.call(
    ".sampler_summary",
    args = list(
      sampler_params  = sp,
      warmup_val      = N_WARMUP,
      report          = input$sampler_report,
      digits          = input$sampler_digits
    )
  )
})

output$sampler_summary <- DT::renderDataTable({
  DT::datatable({
    summary_stats_sampler()
  }, options = list(
    # rownames = FALSE,
    processing = TRUE,
    deferRender = TRUE,
    scrollX = TRUE,
    scrollY = "200px",
    scrollCollapse = TRUE,
    paging = FALSE,
    searching = FALSE,
    info = FALSE
  ))
})

absolutePanel(
  id = "controls_autocorr",
  class = "draggable_controls",
  fixed = TRUE,
  top = 185,
  right = 20,
  width = 200,
  draggable = TRUE,
  shinyjs::hidden(div(
    id = "autocorr_options",
    wellPanel(
      class = "optionswell",
      strongBig("Autocorrelation"),
      hr(class = "hroptions"),
      br(),
      sliderInput(
        "ac_lags",
        label = NULL,
        post = " lags",
        min = 0,
        max = .nIter - .nWarmup - 5,
        step = 5,
        value = min(25, round((.nIter - .nWarmup) / 2))
      ),
      checkboxInput("ac_partial", label = "Partial autocorrelation", value = FALSE),
      checkboxInput("ac_warmup", label = "Include warmup", FALSE),
      checkboxInput("ac_combine", label = "Combine chains", FALSE),
      checkboxInput("ac_flip", label = "Flip facets", FALSE),
      hr(class = "hroptions"),
      downloadButton("download_autocorr", "ggplot2", class = "plot-download"),
      downloadButton('save_pdf_autocorr', "pdf", class = "plot-download pdf-download")
    )
  ))
)

# trivariate scatterplot
trivariate_transform_x <-
  eventReactive(input$trivariate_transform_go > 0,
                input$trivariate_transform_x)
trivariate_transform_y <-
  eventReactive(input$trivariate_transform_go > 0,
                input$trivariate_transform_y)
trivariate_transform_z <-
  eventReactive(input$trivariate_transform_go > 0,
                input$trivariate_transform_z)

trivariate_plot <- reactive({
  validate(
    need(input$trivariate_flip, message = "Loading..."),
    need(input$trivariate_param_x, message = "Waiting for x ..."),
    need(input$trivariate_param_y, message = "Waiting for y ..."),
    need(input$trivariate_param_z, message = "Waiting for z ...")
  )
  x <- input$trivariate_param_x
  y <- input$trivariate_param_y
  z <- input$trivariate_param_z
  samps <- SAMPS_post_warmup
  do.call(
    ".param_trivariate",
    args = list(
      params = c(x, y, z),
      samps = samps,
      pt_color = input$trivariate_pt_color,
      pt_size = input$trivariate_pt_size,
      show_grid = input$trivariate_grid == "show",
      flip_y = input$trivariate_flip == "flip",
      transform_x = trivariate_transform_x(),
      transform_y = trivariate_transform_y(),
      transform_z = trivariate_transform_z()
    )
  )
})

output$trivariate_plot_out <- threejs::renderScatterplotThree({
  trivariate_plot()
})

tagList(
  logo_and_name(),
  div(class = "home-links",
      div(id = "model-name",
          br(),
          h2("Model:"),
          h4(.model_name))),
  br(), br(), br(), br(),
  includeHTML("html/home_page_links.html")
)

thm <- theme_classic() %+replace% (no_lgnd + fat_axis + axis_labs + transparent)
thm_no_yaxs <- thm + no_yaxs

.sampler_param_pw <- function(sp, which = "accept_stat__", warmup_val) {
  if (!which %in% colnames(sp[[1]]))
    return(NULL)
  sp_pw <- lapply(1:length(sp), function(i) {
    out <- sp[[i]][, which]
  })
  sp_mat <- do.call("cbind", sp_pw)
  colnames(sp_mat) <- paste0("chain:", 1:ncol(sp_mat))
  sp_mat <- cbind(iterations = seq(from = warmup_val + 1, to = warmup_val + nrow(sp_mat)), 
                  sp_mat)
  as.data.frame(sp_mat)
}


.sampler_param_vs_param <- function(p, sp, divergent = NULL, hit_max_td = NULL, 
                                    p_lab, sp_lab, chain = 0, violin = FALSE, 
                                    smoother = FALSE) {
  xy_labs <- labs(
    y = if (missing(p_lab)) NULL else p_lab, 
    x = if (missing(sp_lab)) NULL else sp_lab
    )
  df <- data.frame(sp = do.call("c", sp), p = c(p))
  if (violin)
    df$sp <- as.factor(round(df$sp, 4))
  if (!is.null(divergent))
    df$divergent <- do.call("c", divergent)
  if (!is.null(hit_max_td))
    df$hit_max_td <- do.call("c", hit_max_td)
  
  base <- ggplot(df, aes(sp,p)) + 
    xy_labs + 
    thm 
  
  if (chain == 0) {
    if (violin) graph <- base + geom_violin(color = vline_base_clr, fill = base_fill) 
    else {
      graph <- base + geom_point(alpha = 1/3, color = pt_outline_clr, fill = base_fill, shape = 19) 
      if (smoother) 
        graph <- graph + stat_smooth(color = overlay_fill, se = FALSE)
      if (!is.null(divergent))
        graph <- graph + geom_point(data = subset(df, divergent == 1), aes(sp,p), 
                                    color = divergent_clr, fill = divergent_fill,
                                    size = 3, shape = div_and_hit_shape)
      if (!is.null(hit_max_td))
        graph <- graph + geom_point(data = subset(df, hit_max_td == 1), aes(sp,p), 
                                    color = hit_max_td_clr, fill = hit_max_td_fill, 
                                    size = 3, shape = div_and_hit_shape)
    }
    return(graph)
  }
  chain_data <- data.frame(sp = sp[, chain], p = p[, chain])
  if (!is.null(divergent))
    chain_data$div <- divergent[, chain]
  if (!is.null(hit_max_td))
    chain_data$hit <- hit_max_td[, chain]
  chain_clr <- color_vector_chain(ncol(sp))[chain]
  chain_fill <- chain_clr
  if (violin) {
    chain_data$sp <- as.factor(round(chain_data$sp, 4))
    graph <- base + 
      geom_violin(color = vline_base_clr, fill = base_fill) + 
      geom_violin(data = chain_data, aes(sp, p), color = chain_clr, 
                  fill = chain_fill, alpha = 0.5)
    return(graph)
  }
  graph <- base + geom_point(alpha = 1/3, color = pt_outline_clr, fill = base_fill, shape = 19)
  if (smoother) 
    graph <- graph + stat_smooth(color = overlay_fill, se = FALSE)
  graph <- graph + geom_point(data = chain_data, aes(sp,p), color = chain_fill, alpha = 0.5)
  if (smoother) 
    graph <- graph + stat_smooth(data = chain_data, aes(sp,p), color = chain_fill, se = FALSE)
  if (!is.null(divergent))
    graph <- graph + geom_point(data = subset(chain_data, div == 1), aes(sp,p), 
                                color = divergent_clr, fill = divergent_fill,
                                size = 3, shape = div_and_hit_shape)
  if (!is.null(hit_max_td))
    graph <- graph + geom_point(data = subset(chain_data, hit == 1), aes(sp,p), 
                                color = hit_max_td_clr, fill = hit_max_td_fill, 
                                size = 3, shape = div_and_hit_shape)
  graph
}

.sampler_param_vs_sampler_param_violin <- function(df_x, df_y, lab_x, lab_y, 
                                                   chain = 0) {
  xy_labs <- labs(y = lab_y, x = lab_x)
  df <- data.frame(x = do.call("c", df_x), y = do.call("c", df_y))
  df$x <- as.factor(df$x)
  
  base <- ggplot(df, aes(x,y)) + 
    xy_labs + 
    thm
  
  graph <- base + geom_violin(color = vline_base_clr, fill = base_fill) 
  if (chain == 0) 
    return(graph)
  
  chain_clr <- color_vector_chain(ncol(df_x))[chain]
  chain_fill <- chain_clr
  chain_data <- data.frame(x = as.factor(df_x[, chain]), y = df_y[, chain])
  graph + geom_violin(data = chain_data, aes(x,y), color = chain_clr, 
                      fill = chain_fill, alpha = 0.5)
}

.p_hist <- function(df, lab, chain = 0) {
  thm <- thm_no_yaxs
  mdf <- reshape2::melt(df, id.vars = "iterations")
  base <- ggplot(mdf, aes(x = value)) + 
    geom_histogram(aes_string(y="..density.."),
                   binwidth = diff(range(mdf$value))/30, fill = base_fill, 
                   color = vline_base_clr, size = 0.2) + 
    labs(x = if (missing(lab)) NULL else lab, y = "") + 
    thm
  
  if (chain == 0) {
    graph <- base + 
      geom_vline(xintercept = mean(mdf$value), color = vline_base_clr, size = .8) + 
      geom_vline(xintercept = median(mdf$value), color = vline_base_clr, lty = 2, size = 1)
    return(graph)
  }
  chain_data <- subset(mdf, variable == paste0("chain:",chain))
  chain_clr <- color_vector_chain(ncol(df) - 1)[chain]
  chain_fill <- chain_clr
  base + thm + geom_histogram(data = chain_data,
                              aes_string(y="..density.."),
                              binwidth = diff(range(chain_data$value))/30,
                              fill = chain_fill, alpha = 0.5) +
    geom_vline(xintercept = mean(chain_data$value), color = chain_clr, size = .8) + 
    geom_vline(xintercept = median(chain_data$value), 
               color = chain_clr, lty = 2, size = 1)
}

.treedepth_ndivergent_hist <- function(df_td, df_nd, chain = 0, divergent = c("All", 0, 1)) {
  plot_title <- theme(plot.title = element_text(size = 11, hjust = 0))
  plot_theme <- thm_no_yaxs + plot_title
  x_lab <- if (divergent == "All") "Treedepth (All)" else paste0("Treedepth (Divergent = ", divergent,")")
  plot_labs <- labs(x = x_lab, y = "") 
  
  mdf_td <- reshape2::melt(df_td, id.vars = "iterations")
  mdf_nd <- reshape2::melt(df_nd, id.vars = "iterations")
  mdf <- cbind(mdf_td, div = mdf_nd$value)
  plot_data <- if (divergent == "All") 
    mdf else subset(mdf, div == divergent)
  if (nrow(plot_data) == 0) 
    return(NULL)
  
  graph <- ggplot(plot_data, aes(x = factor(value)), na.rm = TRUE) + 
    geom_bar(aes(y=..count../sum(..count..)), width=1, fill = base_fill,
             color = vline_base_clr, size = 0.2) + 
    plot_labs + 
    plot_theme
  if (chain == 0) 
    return(graph)
  
  chain_clr <- color_vector_chain(ncol(df_td) - 1)[chain]
  chain_fill <- chain_clr
  chain_data <- subset(plot_data, variable == paste0("chain:",chain))
  graph + geom_bar(data = chain_data, aes(y=..count../sum(..count..)), 
                   fill = chain_fill, alpha = 0.5, width = 1)
}

.dynamic_trace_diagnostics <- function(param_samps, param_name, chain = 0,
                                       stack = FALSE, grid = FALSE, 
                                       group = NULL) {
  dim_samps <- dim(param_samps)
  if (is.null(dim_samps)) {
    nChains <- 1
  } else {
    nChains <- dim_samps[2]
  }
  if (nChains == 1) {
    param_chains <- xts::as.xts(ts(param_samps, start = 1))
  } else {
    if (chain != 0) {
      param_samps <- param_samps[, chain]
      param_chains <- xts::as.xts(ts(param_samps, start = 1))
    } else {
      param_chains <- xts::as.xts(ts(param_samps[,1], start = 1))
      for (i in 2:nChains) {
        param_chains <- cbind(param_chains,
                              xts::as.xts(ts(param_samps[,i], start = 1)))
      }
      colnames(param_chains) <- paste0("Chain", 1:nChains)
    }
  }
  `%>%` <- dygraphs::`%>%`
  y_axis_label_remove <- if (stack) "white" else NULL
  step_plot <- param_name %in% c("Treedepth", "Divergent")
  fill_graph <- param_name == "Divergent"
  stroke_width <- if (step_plot) 0.33 else 0.75
  clrs <- color_vector(nChains) 
  if (chain != 0) clrs <- clrs[chain]
  dygraphs::dygraph(param_chains, xlab = param_name, ylab = NULL, 
                    group = group) %>%
    dygraphs::dyOptions(colors = clrs, stackedGraph = stack, drawGrid = grid,
                        stepPlot = step_plot, #axisLabelFontSize = 11,
                        fillGraph = fill_graph, fillAlpha = 0.5,
                        strokeWidth = 0.75, animatedZooms = TRUE, 
                        drawXAxis = TRUE, drawYAxis = !fill_graph, 
                        drawAxesAtZero = TRUE, axisLineColor = "black", 
                        retainDateWindow = TRUE) %>%
    dygraphs::dyAxis("x", pixelsPerLabel = 1e7, axisLineWidth = 3) %>%
    dygraphs::dyAxis("y", pixelsPerLabel = 30, axisLabelWidth = 30) %>%
    dygraphs::dyLegend(show = "never") %>%
    dygraphs::dyHighlight(highlightCircleSize = 2,
                          highlightSeriesBackgroundAlpha = 1/3,
                          hideOnMouseOut = TRUE,
                          highlightSeriesOpts = list(strokeWidth = 1.5)) %>%
    dygraphs::dyCSS(css = "css/ShinyStan_dygraphs.css")
}

# ggplot theme elements to be used as needed  


# transparent background --------------------------------------------------
transparent <- theme(
  panel.background = element_blank(), 
  plot.background = element_blank()
  )


# axes and titles ---------------------------------------------------------
axis_line_color <- "gray20"
axis_color <- theme(
  axis.line = element_line(color = axis_line_color)
  )
axis_labs <- theme(
  axis.title = element_text(face = "bold", size = 13)
  )
title_txt <- theme(
  plot.title = element_text(face = "bold", size = 14)
  )
fat_axis <- theme(
  axis.line.x = element_line(size = 3, color = axis_line_color), 
  axis.line.y = element_line(size = 0.5, color = axis_line_color)
  )
no_yaxs <- theme(
  axis.line.y = element_blank(),
  axis.ticks.y = element_blank(),
  axis.text.y = element_blank()
  )

# facet strips ------------------------------------------------------------
strip_txt <- theme(
  strip.text = element_text(size = 12, face = "bold", color = "white"),
  strip.background = element_rect(color = axis_line_color, fill = axis_line_color)
  )


# horizontal and vertical lines -------------------------------------------
h_lines <- theme(
  panel.grid.major = element_line(size = 0.10, linetype = 3, color = "turquoise4"),
  panel.grid.major.x = element_blank()
  )
v_lines <- theme(
  panel.grid.major = element_line(size = 0.25, linetype = 3, color = "turquoise4"),
  panel.grid.major.y = element_blank()
  )


# legends -----------------------------------------------------------------
no_lgnd <- theme(
  legend.position = "none"
  )
lgnd_bot <- theme(
  legend.position = "bottom", 
  legend.background = element_blank()
  )
lgnd_top <- theme(
  legend.position = "top", 
  legend.background = element_blank()
  )
lgnd_left <- theme(
  legend.position = "left", 
  legend.background = element_blank()
  )
lgnd_right <- theme(
  legend.position = "right", 
  legend.background = element_blank()
  )


# colors and shapes -------------------------------------------------------
base_fill <- "#66a7e0" 
overlay_fill <- "#006dcc" 
vline_base_clr <- "#006dcc"
pt_outline_clr <- "#328ad6"
divergent_fill <- "#ae0001" 
hit_max_td_fill <- "#eeba30"
divergent_clr <-  "black" 
hit_max_td_clr <- "black" 
div_and_hit_shape <- 21

blue_color <- "#006dcc"

---
title: 'Deploying to shinyapps.io'
date: "09/17/2015"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{ShinyStan: Deploying to shinyapps.io}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

Create a ShinyStan app unique to your model and host it online with RStudio's ShinyApps service (shinyapps.io). Each app you deploy will have a unique url.  

*Note: R users who don't use RStudio's IDE can still deploy ShinyStan apps to shinyapps.io.* 

## Step 1: ShinyApps account

**Signup**

To deploy your app to RStudio's shinyapps.io you will need a ShinyApps account.
If you don't already have one you can sign up at https://www.shinyapps.io/.

The only limit to the number of different ShinyStan apps you can deploy is the
limit set by RStudio for the [type of ShinyApps
account](https://www.shinyapps.io/#pricing) you sign up for.

**Setup**

When you finish signing up for your ShinyApps account there will be instructions
for setting up your account on your local system using `rsconnect::setAccountInfo()` (this requires first installing the **rsconnect** package)

```r
rsconnect::setAccountInfo(name, token, secret)
```

where `name` is your ShinyApps account name, and `token` and `secret` can be
found from your ShinyApps account web page.

## Step 2: Use `deploy_shinystan` to deploy your app to shinyapps.io

The `deploy_shinystan` function will deploy a ShinyStan app unique to your model
to RStudio's ShinyApps service.

For the example below assume that 

- `my_sso` is the shinystan object you want to use
- the name you want to use for the app is `MyModel` 
- the username for your ShinyApps account (from Step 1) is `username`.

To deploy the app use the command

```r
deploy_shinystan(my_sso, appName = "MyModel", account = "username")
```


**When to specify the `account` argument**

If you have multiple ShinyApps accounts configured on your local system then the `account` argument is required. If you only have a single account then you can omit `account`

```r
deploy_shinystan(my_sso, appName = "MyModel")
```

**Graphical posterior predictive checks: the optional `ppcheck_data` and `ppcheck_yrep` arguments**

If you have a vector of observations `y` that you want to use for ShinyStan's graphical posterior predictive checks then you can also specify the optional `ppcheck_data` argument

```r
deploy_shinystan(my_sso, appName = "MyModel", 
                 ppcheck_data = y)
```

If you've specified the `ppcheck_data` argument you can also use the `ppcheck_yrep` argument to tell ShinyStan the name of the parameter/quantity in your model containing the posterior predictive simulations/replications so that it is preselected as the parameter to use for doing the graphical posterior predictive checks. This is never required because you can also set this manually while using the app, but it is helpful if you want other people to view your model online because it avoids the need to communicate to them which parameter they need to select. If this parameter/quantity containing the posterior predictive replications is named `yRep` then you would use

```r
deploy_shinystan(my_sso, appName = "MyModel", 
                 ppcheck_data = y, ppcheck_yrep = "yRep")
```


If the deployment process is successful the url for your app will be printed to the console and it should open in your web browser. You can also view your app by going to https://www.shinyapps.io and logging into your account.

---
title: 'Getting Started'
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    toc: true
vignette: >
  %\VignetteIndexEntry{Getting Started}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Using the ShinyStan app with different types of objects

## stanfit objects

If `my_stanfit` is a stanfit object (the result of fitting a model with
[**rstan**](https://mc-stan.org/)), then to launch the ShinyStan app simply use

```r
library(shinystan)
my_sso <- launch_shinystan(my_stanfit)
```

and ShinyStan will launch. Here `my_sso` is the name you want to use for the
shinystan object that will be returned. If you simply run

```r
launch_shinystan(my_stanfit)
```

then ShinyStan will launch but no shinystan object will be saved upon quitting
the app. 

Sometimes keeping only a subset of parameters before can improve performance.
This can be done by creating an object with `as.shinystan` and specifying the
`pars` argument. The resulting shinystan object can then be passed to
`launch_shinystan()`.


### stanreg and brmsfit objects

The **rstanarm** and **brms** packages provide `launch_shinystan` methods for stanreg and brmsfit objects, 
respectively. For example, the method for stanreg objects is 
documented at 

http://mc-stan.org/rstanarm/reference/launch_shinystan.stanreg.html

## mcmc.list objects

If you have an `mcmc.list` object called `my_mcmc` then you can use the
`as.shinystan` function to convert `my_mcmc` into a shinystan object that can
then be used with `launch_shinystan`:

```r
my_sso <- launch_shinystan(as.shinystan(my_mcmc, model_name = "my_model"))
```

If, for example, the first 100 iterations in each chain in `my_mcmc` are 
warmup iterations, you should add the `warmup` argument when you 
call `as.shinystan`:

```r
my_sso <- launch_shinystan(as.shinystan(my_mcmc, model_name = "my_model", warmup = 100))
```

However, you should only use the `warmup` argument if the warmup iterations have 
been saved and included in `my_mcmc`.   


## Other types of objects

### 3-D array

To convert a 3-D array to a shinystan object make sure that the three dimensions 
of the array correspond to the 
*number of iterations* **x** *number of chains*  **x** *number of parameters*. 
You can then `as.shinystan` exactly how it's used in the examples for `mcmc.list`
objects above (but you don't need to convert your array  to an `mcmc.list`).

### List of matrices

If you have separate chains that are each a matrix (with iterations as rows and
parameters as columns) you can combine them in a list to pass to `as.shinystan`

```r
# Generate some fake data
chain1 <- cbind(beta1 = rnorm(100), beta2 = rnorm(100), sigma = rexp(100))
chain2 <- cbind(beta1 = rnorm(100), beta2 = rnorm(100), sigma = rexp(100))
chain_list <- list(chain1, chain2)
my_sso <- launch_shinystan(as.shinystan(X = list(chain1, chain2), model_name = "my_model"))
```


******

# Other functions in the shinystan package

## Generating new quantities

You can add a new parameter/quantity as a function of one or two existing 
parameters to your shinystan object by using the `generate_quantity` function. 
For example, assume `sso` is a shinystan object and two of the parameters are
```alpha``` and ```beta```. We could add a parameter ```gamma``` that is the 
inverse logit of ```beta``` using the code

```r
inv_logit <- function(x) 1/(1 + exp(-x))
sso <- generate_quantity(sso, fun = inv_logit, param1 = "beta", new_name = "gamma")
```

Here, `fun` is the function we want to use, `param1` is the name of the parameter 
to apply the function to, and `new_name` is the name to give the new parameter. 

Adding a parameter as a function of two parameters just requires specifying the
`param2` argument and providing a function of two variables. For example, we can 
add a parameter `delta` to `sso` that is the squared difference of `alpha` and 
`beta` like this

```r
sso <- generate_quantity(sso, fun = function(x,y) (x-y)^2,
                       param1 = "alpha", param2 = "beta", new_name = "delta")
```


## Storing your model code in a shinystan object

For models fit using **rstan** the model code will 
automatically be stored in the `model_code` slot of your shinystan object. 
When ShinyStan is open you can view your model code in the *Model Code* tab. 

If you did not use **rstan** fit your model then you can add your model code 
by using the `model_code()` function. For example, you may have used Bugs 
or JAGS or some other software and want to add the following code

```
  for (i in 1:length(Y)) {
     Y[i] ~ dpois(lambda[i])
     log(lambda[i]) <- inprod(X[i,], theta[])
   }
   for (j in 1:J) {
     theta[j] ~ dt(0.0, 1.0, 1.0)
   }
 }
```

to your shinystan object. To add that code you can simply include it as the 
`code` argument to the `model_code` function 

```r
my_code <- "
 model {
  for (i in 1:length(Y)) {
     Y[i] ~ dpois(lambda[i])
     log(lambda[i]) <- inprod(X[i,], theta[])
   }
   for (j in 1:J) {
     theta[j] ~ dt(0.0, 1.0, 1.0)
   }
 }
"

# Add the code to a shinystan object sso
sso <- model_code(sso, code = my_code)
```

## Renaming a model

On the home page ShinyStan will display the model name associated with the 
shinystan object being used. This name can be set by adding the `model_name`
argument to `as.shinystan` when creating a shinystan object. For an existing
shinystan object you can use the `model_name` function like this:

```r
sso <- model_name(sso, "new_model_name")
```

where `"new_model_name"` is the new name you want to give your model.

## Using Stan and ShinyStan for posterior predictive checking

In this tutorial we do the following:

1. Generate some fake data to play with
2. Write code for a simple Stan model
3. Fit the model using **RStan** 
4. Use **ShinyStan** for graphical posterior predictive checks

### Data

First we'll generate some fake data in R to use for this example

    # Number of observations 
    N <- 100

    # Model matrix (with column of 1s for intercept and one covariate)
    X <- cbind(Const = 1, X1 = rnorm(N))
    K <- ncol(X)

    # Generate fake outcome y
    beta <- c(2, 1/2) # pick intercept and coefficient
    sigma <- 1 # standard deviation
    y <- rnorm(N, mean = X %*% beta, sd = sigma) # generate data


### Stan code

Now we can write Stan code for a simple linear regression model.

    data {
      int           N ; # integer, number of observations
      int           K ; # integer, number of columns in model matrix
      matrix[N,K]   X ; # N by K model matrix
      vector[N]     y ; # vector of N observations
    }
    
    parameters {
      real<lower=0> sigma ; # real number > 0, standard deviation
      vector[K]     beta ;  # K-vector of regression coefficients
    }
    
    model {
      beta ~ normal(0, 5) ;       # prior for betas
      sigma ~ cauchy(0, 2.5) ;    # prior for sigma
      y ~ normal(X*beta, sigma) ; # vectorized likelihood
    }
    
    generated quantities {
    # Here we do the simulations from the posterior predictive distribution
      vector[N] y_rep ; # vector of same length as the data y
      for (n in 1:N) 
        y_rep[n] <- normal_rng(X[n]*beta, sigma) ;
    }

In this case the posterior predictive distribution we want to simulate from is the normal distribution with mean and standard deviation updated to reflect the posterior draws of `beta` and `sigma`. 

The code in the `generated quantities` block will be evaluated for each posterior draw of the parameters. For example, if we have 100 post-warmup iterations then we will have 100 `y_rep` vectors, each of length `N`. 

### Fit the model

If we've saved our Stan code in a file called `stan_code.stan` then we can run this model with **RStan** and then launch **ShinyStan** like this:

    library(rstan)
    library(ShinyStan)
    
    # Prepare the data we'll need as a list
    stan_data <- list(y = y, X = X, N = N, K = K)
    
    # Fit the model
    stanfit <- stan(file = "stan_code.stan", data = stan_data)
    
    # Launch ShinyStan
    launch_shinystan(stanfit)


### Graphical posterior predictive checks with ShinyStan

Once we've launched **ShinyStan** we can navigate to the page for posterior predictive checking. In the dropdown menus it will ask us to select the object containing our data from our R global environment and the name of the paramter from our model containing the posterior predictive replications. So we enter `y` and `y_rep`, respectively. 

**ShinyStan** will then generate graphics that will aid in checking the fit of our model including comparisons of the distribution of the observed data to the distributions of the posterior predictive replications, distributions of test statistics, and residual plots.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_quantity.R
\name{generate_quantity}
\alias{generate_quantity}
\title{Add new quantity to shinystan object}
\usage{
generate_quantity(sso, param1, param2, fun, new_name)
}
\arguments{
\item{sso}{A \code{\link[=as.shinystan]{shinystan object}}.}

\item{param1}{Name of first parameter as character string.}

\item{param2}{Optional. Name of second parameter as character string.}

\item{fun}{Function to call, i.e. \code{function(param1)} or 
\code{function(param1,param2)}. See Examples, below.}

\item{new_name}{Name for the new parameter as character string.}
}
\value{
sso, updated. See Examples.
}
\description{
Add to shinystan object a new parameter as a function of one or two existing
parameters.
}
\examples{
# Using example shinystan object 'eight_schools'
sso <- eight_schools
sso <- generate_quantity(sso, fun = function(x) x^2, 
                         param1 = "tau", new_name = "tau_sq")
sso <- generate_quantity(sso, fun = "-", 
                         param1 = "theta[1]", param2 = "theta[2]", 
                         new_name = "theta1minus2")
                         
}
\seealso{
\code{\link{drop_parameters}} to remove parameters from a
  \code{shinystan} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shinystan-objects.R
\name{as.shinystan}
\alias{as.shinystan}
\alias{is.shinystan}
\alias{as.shinystan,array-method}
\alias{as.shinystan,list-method}
\alias{as.shinystan,mcmc.list-method}
\alias{as.shinystan,stanfit-method}
\alias{as.shinystan,stanreg-method}
\alias{as.shinystan,CmdStanMCMC-method}
\title{Create and test \code{shinystan} objects}
\usage{
as.shinystan(X, ...)

is.shinystan(X)

\S4method{as.shinystan}{array}(
  X,
  model_name = "unnamed model",
  warmup = 0,
  burnin = 0,
  param_dims = list(),
  model_code = NULL,
  note = NULL,
  sampler_params = NULL,
  algorithm = NULL,
  max_treedepth = NULL,
  ...
)

\S4method{as.shinystan}{list}(
  X,
  model_name = "unnamed model",
  warmup = 0,
  burnin = 0,
  param_dims = list(),
  model_code = NULL,
  note = NULL,
  sampler_params = NULL,
  algorithm = NULL,
  max_treedepth = NULL,
  ...
)

\S4method{as.shinystan}{mcmc.list}(
  X,
  model_name = "unnamed model",
  warmup = 0,
  burnin = 0,
  param_dims = list(),
  model_code = NULL,
  note = NULL,
  ...
)

\S4method{as.shinystan}{stanfit}(X, pars, model_name = X@model_name, note = NULL, ...)

\S4method{as.shinystan}{stanreg}(X, ppd = TRUE, seed = 1234, model_name = NULL, note = NULL, ...)

\S4method{as.shinystan}{CmdStanMCMC}(X, pars = NULL, model_name = NULL, note = NULL, ...)
}
\arguments{
\item{X}{For \code{as.shinystan}, an object to be converted to a
\code{shinystan} object. See the Methods section below. For
\code{is.shinystan}, an object to check.}

\item{...}{Arguments passed to the individual methods.}

\item{model_name}{A string giving a name for the model.}

\item{warmup}{The number of iterations to treat as warmup. Should be
\code{0} if warmup iterations are not included in \code{X}.}

\item{burnin}{Deprecated. Use \code{warmup} instead. The \code{burnin}
argument will be removed in a future release.}

\item{param_dims}{Rarely used and never necessary. A named list giving the 
dimensions for all parameters. For scalar parameters use \code{0} as the 
dimension. See Examples.}

\item{model_code}{Optionally, a character string with the code used to run
the model. This can also be added to your \code{shinystan} object later
using the \code{\link[shinystan]{model_code}} function. See 
\code{\link[shinystan]{model_code}} for additional formatting instructions.
After launching the app the code will be viewable in the \strong{Model 
Code} tab. For \code{stanfit} (\pkg{rstan}) and \code{stanreg}
(\pkg{rstanarm}) objects the model code is automatically retrieved from the
object.}

\item{note}{Optionally, text to display on the \strong{Notepad} page in the 
'ShinyStan' GUI (stored in \code{user_model_info} slot of the
\code{shinystan} object).}

\item{sampler_params, algorithm, max_treedepth}{Rarely used and never 
necessary. If using the \code{as.shinystan} method for arrays or lists, 
these arguments can be used to manually provide information that is 
automatically retrieved from a stanfit object when using the 
\code{as.shinystan} method for stanfit objects. If specified, 
\code{sampler_params} must have the same structure as an object returned by
\code{\link[rstan]{get_sampler_params}} (\pkg{rstan}), which is a list of 
matrices, with one matrix per chain. \code{algorithm}, if specified, must 
be either \code{"NUTS"} or \code{"HMC"} (static HMC). If \code{algorithm}
is \code{"NUTS"} then \code{max_treedepth} (an integer indicating the
maximum allowed treedepth when the model was fit) must also be provided.}

\item{pars}{For stanfit objects (\pkg{rstan}), an optional character vector 
specifying which parameters should be included in the \code{shinystan}
object.}

\item{ppd}{For \code{stanreg} objects (\pkg{rstanarm}), \code{ppd} 
(logical) indicates whether to draw from the posterior predictive 
distribution before launching the app. The default is \code{TRUE}, 
although for very large objects it can be convenient to set it to 
\code{FALSE} as drawing from the posterior predictive distribution can be 
time consuming. If \code{ppd} is \code{TRUE} then graphical posterior
predictive checks are available when 'ShinyStan' is launched.}

\item{seed}{Passed to \code{\link[rstanarm]{pp_check}} (\pkg{rstanarm}) if 
\code{ppd} is \code{TRUE}.}
}
\value{
\code{as.shinystan} returns a \code{shinystan} object, which is an
  instance of S4 class \code{"shinystan"}.
  
  \code{is.shinystan} returns \code{TRUE} if the tested object is a
  \code{shinystan} object and \code{FALSE} otherwise.
}
\description{
The \code{as.shinystan} function creates \code{shinystan}
  objects that can be used with \code{\link{launch_shinystan}} and various
  other functions in the \pkg{shinystan} package. \code{as.shinystan} is a
  generic for which the \pkg{shinystan} package provides several methods.
  Currently methods are provided for creating \code{shinystan} objects from
  arrays, lists of matrices, \code{stanfit} objects (\pkg{rstan}),
  \code{stanreg} objects (\pkg{rstanarm}), and \code{mcmc.list} objects
  (\pkg{coda}).
  
  \code{is.shinystan} tests if an object is a \code{shinystan} object.
}
\section{Functions}{
\itemize{
\item \code{as.shinystan,array-method}: Create a \code{shinystan} object from a 3-D
\code{\link{array}} of simulations. The array should have dimensions
corresponding to iterations, chains, and parameters, in that order.

\item \code{as.shinystan,list-method}: Create a \code{shinystan} object from a
\code{\link{list}} of matrices. Each \code{\link{matrix}} (or 2-D array)
should contain the simulations for an individual chain and all of the
matrices should have the same number of iterations (rows) and parameters
(columns). Parameters should have the same names and be in the same order.

\item \code{as.shinystan,mcmc.list-method}: Create a \code{shinystan} object from an 
\code{mcmc.list} object (\pkg{coda}).

\item \code{as.shinystan,stanfit-method}: Create a \code{shinystan} object from a
\code{stanfit} object (\pkg{\link[rstan]{rstan}}). Fewer optional arguments
are available for this method because all important information can be
taken automatically from the \code{stanfit} object.

\item \code{as.shinystan,stanreg-method}: Create a \code{shinystan} object from a
\code{stanreg} object (\pkg{\link[rstanarm]{rstanarm}}).

\item \code{as.shinystan,CmdStanMCMC-method}: Create a \code{shinystan} object from a 
\code{CmdStanMCMC} object (\pkg{cmdstanr}).
}}

\examples{
 
\dontrun{
sso <- as.shinystan(X, ...) # replace ... with optional arguments or omit it
launch_shinystan(sso)
}

\dontrun{   
########################
### list of matrices ###
########################

# Generate some fake data
chain1 <- cbind(beta1 = rnorm(100), beta2 = rnorm(100), sigma = rexp(100))
chain2 <- cbind(beta1 = rnorm(100), beta2 = rnorm(100), sigma = rexp(100))
sso <- as.shinystan(list(chain1, chain2))
launch_shinystan(sso)

# We can also specify some or all of the optional arguments
# note: in order to use param_dims we need to rename 'beta1' and 'beta2'
# to 'beta[1]' and 'beta[2]'
colnames(chain1) <- colnames(chain2) <- c(paste0("beta[",1:2,"]"), "sigma")
sso2 <- as.shinystan(list(chain1, chain2), 
                     model_name = "Example", warmup = 0, 
                     param_dims = list(beta = 2, sigma = 0))
launch_shinystan(sso2)
}

\dontrun{
######################
### stanfit object ###
######################
library("rstan")
fit <- stan_demo("eight_schools")
sso <- as.shinystan(fit, model_name = "example")
}

\dontrun{
######################
### stanreg object ###
######################
library("rstanarm")
example("example_model")
sso <- as.shinystan(example_model)
launch_shinystan(sso)
}

}
\seealso{
\code{\link{launch_shinystan}} to launch the 'ShinyStan' interface 
  using a particular \code{shinystan} object.

\code{\link{drop_parameters}} to remove parameters from a
  \code{shinystan} object.

\code{\link{generate_quantity}} to add a new quantity to a
  \code{shinystan} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shinystan-objects.R
\docType{class}
\name{shinystan-class}
\alias{shinystan-class}
\alias{shinystan}
\title{S4 \code{shinystan} objects}
\description{
See \code{\link{as.shinystan}} for documentation on creating 
  \code{shinystan} objects and \code{\link{eight_schools}} for an example
  object.
}
\section{Slots}{

\describe{
\item{\code{model_name}}{(\code{"character"}) Model name.}

\item{\code{param_names}}{(\code{"character"}) Parameter names.}

\item{\code{param_dims}}{(\code{"list"}) Parameter dimensions.}

\item{\code{posterior_sample}}{(\code{"array"}) MCMC sample.}

\item{\code{summary}}{(\code{"matrix"}) Summary stats for \code{posterior_sample}.}

\item{\code{sampler_params}}{(\code{"list"}) Sampler parameters (for certain Stan
models only).}

\item{\code{n_chain}}{(\code{"integer"}) Number of chains.}

\item{\code{n_iter}}{(\code{"integer"}) Number of iterations per chain.}

\item{\code{n_warmup}}{(\code{"integer"}) Number of warmup iterations per chain.}

\item{\code{user_model_info}}{(\code{"character"}) Notes to display on the
\strong{Notepad} page in the 'ShinyStan' GUI.}

\item{\code{model_code}}{(\code{"character"}) Model code to display on the
\strong{Model Code} page in the 'ShinyStan' GUI.}

\item{\code{misc}}{(\code{"list"}) Miscellaneous, for internal use.}
}}

\references{
Muth, C., Oravecz, Z., and Gabry, J. (2018)
User-friendly Bayesian regression modeling: A tutorial with rstanarm and shinystan.
\emph{The Quantitative Methods for Psychology}. 14(2), 99--119.
\url{https://www.tqmp.org/RegularArticles/vol14-2/p099/p099.pdf}
}
\seealso{
\code{\link{as.shinystan}} for creating \code{shinystan} objects.

\code{\link{drop_parameters}} to remove parameters from a
  \code{shinystan} object.

\code{\link{generate_quantity}} to add a new quantity to a
  \code{shinystan} object.

\code{\link{shinystan-metadata}} to view or change metadata
  associated with a \code{shinystan} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sso-metadata.R
\name{shinystan-metadata}
\alias{shinystan-metadata}
\alias{sso_info}
\alias{model_code}
\alias{notes}
\alias{model_name}
\title{View or change metadata associated with a \code{shinystan} object}
\usage{
sso_info(sso)

model_code(sso, code = NULL)

notes(sso, note = NULL, replace = FALSE)

model_name(sso, name = NULL)
}
\arguments{
\item{sso}{A \code{\link[=as.shinystan]{shinystan object}}.}

\item{code}{A string, containing model code to be added, that can be
used as an argument to \code{\link{cat}}. See \strong{Examples}.}

\item{note}{A string containing a note to add to any existing notes
or replace existing notes, depending on the value of \code{replace}.}

\item{replace}{If \code{TRUE} the existing notes are overwritten by 
\code{note} if \code{note} is specified. If \code{FALSE} (the default) 
if \code{note} is specified then its content is appended to the existing
notes.}

\item{name}{A string giving the new model name to use.}
}
\value{
\code{sso_info} prints basic metadata including number of parameters, 
  chains, iterations, warmup iterations, etc. It does not return anything.

\code{model_code} returns or replaces model code stored in a 
  \code{shinystan} object. If \code{code} is \code{NULL} then any existing 
  model code stored in \code{sso} is returned as a character string. If 
  \code{code} is specified then an updated \code{shinystan} object is 
  returned with \code{code} added. For \code{shinystan} objects created from 
  stanfit (\pkg{rstan}) and stanreg (\pkg{rstanarm}) objects, model code is 
  automatically taken from that object and does not need to be added 
  manually. From within the 'ShinyStan' interface model code can be viewed on
  the \strong{Model Code} page.

\code{notes} returns, amends, or replaces notes stored in a
  \code{shinystan} object. If \code{note} is \code{NULL} then any existing
  notes stored in \code{sso} are returned as a character string. If
  \code{note} is specified then an updated \code{shinystan} object is
  returned with either \code{note} added to the previous notes (if
  \code{replace=FALSE}) or overwritten by \code{note} (if \code{replace =
  TRUE}). From within the 'ShinyStan' interface, notes are viewable on the
  \strong{Notepad} page.

\code{model_name} returns or replaces the model name associated with 
  a \code{shinystan} object. If \code{name} is \code{NULL} then the current
  model name is returned. If \code{name} is specified then \code{sso} is
  returned with an updated model name.
}
\description{
View or change metadata associated with a \code{shinystan} object
}
\examples{
# use eight_schools example object
sso <- eight_schools

################
### sso_info ###
################

sso_info(sso)

##################
### model_code ###
##################

# view model code in example shinystan object 'eight_schools'
cat(model_code(sso))

# change the model code in sso 
# some jags style code
my_code <- "
 model {
   for (i in 1:length(Y)) {
     Y[i] ~ dpois(lambda[i])
     log(lambda[i]) <- inprod(X[i,], theta[])
   }
   for (j in 1:J) {
     theta[j] ~ dt(0.0, 1.0, 1.0)
   }
 }
"
sso <- model_code(sso, my_code)
cat(model_code(sso))

#############
### notes ###
#############

# view existing notes
notes(sso)

# add a note to the existing notes
sso <- notes(sso, "New note")
notes(sso)
cat(notes(sso))

# replace existing notes
sso <- notes(sso, "replacement note", replace = TRUE)
notes(sso)
 
##################
### model_name ###
##################

# view model name
model_name(sso)

# change model name
sso <- model_name(sso, "some other name")
identical(model_name(sso), "some other name")

}
\seealso{
\code{\link{as.shinystan}} for creating \code{shinystan} objects.

\code{\link{drop_parameters}} to remove parameters from a
  \code{shinystan} object.

\code{\link{generate_quantity}} to add a new quantity to a
  \code{shinystan} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/launch_shinystan.R
\name{launch_shinystan}
\alias{launch_shinystan}
\alias{launch_shinystan.default}
\alias{launch_shinystan.shinystan}
\title{Launch the 'ShinyStan' app}
\usage{
launch_shinystan(object, ...)

\method{launch_shinystan}{default}(object, ..., rstudio = getOption("shinystan.rstudio"))

\method{launch_shinystan}{shinystan}(object, ..., rstudio = getOption("shinystan.rstudio"))
}
\arguments{
\item{object}{The object to use. For the default method this can be an object
of class \code{"shinystan"}, \code{"stanfit"}, or \code{"stanreg"}. To use
other types of objects first create a shinystan object using
\code{\link{as.shinystan}}.}

\item{...}{Optional arguments passed to \code{\link[shiny]{runApp}}.}

\item{rstudio}{Only relevant for 'RStudio' users. The default (\code{FALSE})
is to launch the app in the user's default web browser rather than the 
pop-up Viewer provided by 'RStudio'. Users can change the default to
\code{TRUE} by setting the global option \code{options(shinystan.rstudio =
TRUE)}.}
}
\value{
The \code{launch_shinystan} function is used for the side effect of 
  starting the 'ShinyStan' app, but it also returns a \code{shinystan}
  object, an instance of S4 class \code{"shinystan"}.
}
\description{
Launch the 'ShinyStan' app in the default web browser. 'RStudio' users also
have the option of launching the app in the pop-up Viewer.
}
\examples{
\dontrun{
#######################################
# Example 1: 'sso' is a shinystan object
#######################################

# Just launch shinystan
launch_shinystan(sso)

# Launch shinystan and replace sso with an updated version of itself
# if any changes are made to sso while using the app
sso <- launch_shinystan(sso)

# Launch shinystan but save any changes made to sso while running the app
# in a new shinystan object sso2. sso will remained unchanged. 
sso2 <- launch_shinystan(sso) 

#######################################
# Example 2: 'sf' is a stanfit object
#######################################

# Just launch shinystan
launch_shinystan(sf)

# Launch shinystan and save the resulting shinystan object
sf_sso <- launch_shinystan(sf)

# Now sf_sso is a shinystan object and so Example 1 (above) applies when
# using sf_sso. 

#######################################
# Example 3: 'fit' is an mcmc.list, array or list of matrices
#######################################

# First create shinystan object (see ?as.shinystan) for full details)
}

}
\references{
Muth, C., Oravecz, Z., and Gabry, J. (2018)
User-friendly Bayesian regression modeling: A tutorial with rstanarm and shinystan.
\emph{The Quantitative Methods for Psychology}. 14(2), 99--119.
\url{https://www.tqmp.org/RegularArticles/vol14-2/p099/p099.pdf}

Gabry, J. , Simpson, D. , Vehtari, A. , Betancourt, M. and
  Gelman, A. (2019), Visualization in Bayesian workflow.
  *J. R. Stat. Soc. A*, 182: 389-402. doi:10.1111/rssa.12378
  ([journal version](https://rss.onlinelibrary.wiley.com/doi/full/10.1111/rssa.12378),
   [preprint arXiv:1709.01449](https://arxiv.org/abs/1709.01449),
   [code on GitHub](https://github.com/jgabry/bayes-vis-paper))
}
\seealso{
\code{\link{as.shinystan}} for creating \code{shinystan} objects.

\code{\link{update_sso}} to update a \code{shinystan} object created
  by a previous version of the package.

\code{\link{launch_shinystan_demo}} to try a demo.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deploy_shinystan.R
\name{deploy_shinystan}
\alias{deploy_shinystan}
\title{Deploy a 'ShinyStan' app on the web using 'shinyapps.io' by 'RStudio'}
\usage{
deploy_shinystan(sso, appName, account = NULL, ..., deploy = TRUE)
}
\arguments{
\item{sso}{A \code{\link[=as.shinystan]{shinystan object}}.}

\item{appName}{The name to use for the application. Application names must be
at least four characters long and may only contain letters, numbers, dashes
and underscores.}

\item{account}{shinyapps.io account username. Only required if more than one
account is configured on the system.}

\item{...}{Optional arguments. See Details.}

\item{deploy}{Should the app be deployed? The only reason for this to be
\code{FALSE} is if you just want to check that the preprocessing before
deployment is successful.}
}
\value{
\link[=invisible]{Invisibly}, \code{TRUE} if deployment succeeded
  (did not encounter an error) or, if \code{deploy} argument is set to
  \code{FALSE}, the path to the temporary directory containing the app ready
  for deployment (also invisibly).
}
\description{
Requires a (free or paid) 'ShinyApps' account. Visit
\url{https://www.shinyapps.io/} to sign up.
}
\details{
In \code{...}, the arguments \code{ppcheck_data} and
  \code{ppcheck_yrep} can be specified. \code{ppcheck_data} should be a
  vector of observations to use for graphical posterior predictive checking
  and \code{ppcheck_yrep} should be a character string naming the parameter
  in \code{sso} containing the posterior predictive simulations/replications.
  The value of \code{ppcheck_yrep} is only used to preselect the appropriate
  parameter/generated quantity to use for the posterior predictive checking.
  \code{ppcheck_yrep} (but not \code{ppcheck_data}) can also be set
  interactively on shinyapps.io when using the app.
}
\examples{
\dontrun{
# For this example assume sso is the name of the \code{shinystan} object for
# the model you want to use. Assume also that you want to name your app
# 'my-model' and that your shinyapps.io username is 'username'.

deploy_shinystan(sso, appName = "my-model", account = "username")

# If you only have one ShinyApps account configured then you can also omit
# the 'account' argument.

deploy_shinystan(sso, appName = "my-model")
}

}
\seealso{
The example in the \emph{Deploying to shinyapps.io} vignette that
  comes with this package.

  \url{https://www.shinyapps.io/} to sign up for a free or paid 'ShinyApps'
  account and for details on how to configure your account on your local
  system using the \pkg{rsconnect} package from 'RStudio'.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_sso.R
\name{update_sso}
\alias{update_sso}
\title{Update an object created by the previous version of shinystan}
\usage{
update_sso(sso)
}
\arguments{
\item{sso}{A \code{\link[=as.shinystan]{shinystan object}}.}
}
\value{
If \code{sso} is already compatible with your version of
  \pkg{shinystan} then \code{sso} itself is returned and a message is printed
  indicating that \code{sso} is already up-to-date. Otherwise an updated
  version of \code{sso} is returned unless an error is encountered.
}
\description{
If you encounter any errors when using a shinystan object (\code{sso}) 
created by a previous version of \pkg{shinystan}, you might need to run
\code{update_sso}. If \code{update_sso} does not resolve the problem and 
you still have the object (e.g. stanfit,  stanreg, mcmc.list) from which 
\code{sso} was originally created, you can create a new shinystan object 
using \code{\link{as.shinystan}}.
}
\examples{
\dontrun{
sso_new <- update_sso(sso)
}

}
\seealso{
\code{\link{as.shinystan}} for creating \code{shinystan} objects.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sso-metadata.R
\name{rename_model}
\alias{rename_model}
\title{rename_model (deprecated)}
\usage{
rename_model(sso, new_model_name)
}
\arguments{
\item{sso, new_model_name}{Use the \code{\link{model_name}} function instead.}
}
\description{
This function is deprecated and will be removed in a future release. Please 
use the \code{\link{model_name}} function instead.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/launch_shinystan.R
\name{launch_shinystan_demo}
\alias{launch_shinystan_demo}
\alias{eight_schools}
\title{'ShinyStan' demo}
\usage{
launch_shinystan_demo(
  demo_name = "eight_schools",
  rstudio = getOption("shinystan.rstudio"),
  ...
)
}
\arguments{
\item{demo_name}{The name of the demo. Currently \code{"eight_schools"} is 
the only option, but additional demos may be available in future releases.
\describe{
\item{\code{eight_schools}}{Hierarchical meta-analysis model. See 
 \emph{Meta Analysis} chapter of the 'Stan' manual 
 \url{https://mc-stan.org/users/documentation/}.}
}}

\item{rstudio}{Only relevant for 'RStudio' users. The default (\code{FALSE})
is to launch the app in the user's default web browser rather than the 
pop-up Viewer provided by 'RStudio'. Users can change the default to
\code{TRUE} by setting the global option \code{options(shinystan.rstudio =
TRUE)}.}

\item{...}{Optional arguments passed to \code{\link[shiny]{runApp}}.}
}
\value{
An S4 shinystan object.
}
\description{
'ShinyStan' demo
}
\examples{
\dontrun{
# launch demo but don't save a shinystan object
launch_shinystan_demo() 

# launch demo and save the shinystan object for the demo 
sso_demo <- launch_shinystan_demo()
}

}
\seealso{
\code{\link{launch_shinystan}} to launch the 'ShinyStan' interface 
  using a particular \code{shinystan} object.

\code{\link{as.shinystan}} for creating \code{shinystan} objects.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shinystan-package.R
\docType{package}
\name{shinystan-package}
\alias{shinystan-package}
\title{\pkg{shinystan} R package ('ShinyStan' graphical user interface)}
\description{
\if{html}{
  \figure{stanlogo.png}{options: width="50" alt="mc-stan.org"}
  \emph{Stan Development Team}
}

Applied Bayesian data analysis is primarily implemented through the Markov
chain Monte Carlo (MCMC) algorithms offered by various software packages.
When analyzing a posterior sample obtained by one of these algorithms the
first step is to check for signs that the chains have converged to the target
distribution and and also for signs that the algorithm might require tuning
or might be ill-suited for the given model. There may also be theoretical
problems or practical inefficiencies with the specification of the model. The
ShinyStan app provides interactive plots and tables helpful for analyzing a
posterior sample, with particular attention to identifying potential problems
with the performance of the MCMC algorithm or the specification of the model.
ShinyStan is powered by the Shiny web application framework by RStudio
(\url{https://shiny.rstudio.com/}) and works with the output of MCMC programs
written in any programming language (and has extended functionality for
models fit using the \pkg{rstan} package and the No-U-Turn sampler).
}
\section{ShinyStan has extended functionality for Stan models}{


Stan (\url{https://mc-stan.org/}) models can be run in \R using the
\pkg{\link[rstan]{rstan}} package. Other packages like
\pkg{\link[rstanarm]{rstanarm}} and \pkg{brms} provide
higher-level interfaces to Stan that use \pkg{rstan} internally.
}

\section{Saving and sharing}{


  The \pkg{shinystan} package allows you to store the basic components of an
  entire project (code, posterior samples, graphs, tables, notes) in a single
  object, a \code{\link[=as.shinystan]{shinystan object}} (sso, for short).
  Users can save many of the plots as \pkg{ggplot2} objects for further
  customization and easy integration in reports or post-processing for
  publication.

  The \code{\link{deploy_shinystan}} function lets you easily deploy your own
  ShinyStan apps online for any of your models using the shinyapps.io 
  service from 'RStudio'. Each of your apps (each of your models) will have a
  unique url and will be compatible with most web browsers.
}

\section{License}{


 The \pkg{shinystan} package is open source licensed under the GNU Public
 License, version 3 (GPLv3).
}

\section{Demo}{


 Check out the demo using \code{\link{launch_shinystan_demo}} or try it with
 one of your own models using \code{\link{launch_shinystan}}.
}

\section{Resources}{

\itemize{
 \item Web page with online documentation
   (\url{https://mc-stan.org/shinystan/})
 \item Stan Forums on Discourse (\url{https://discourse.mc-stan.org})
 \item GitHub issue tracker (\url{https://github.com/stan-dev/shinystan/issues})
}
}

\references{
Muth, C., Oravecz, Z., and Gabry, J. (2018)
User-friendly Bayesian regression modeling: A tutorial with rstanarm and shinystan.
\emph{The Quantitative Methods for Psychology}. 14(2), 99--119.
\url{https://www.tqmp.org/RegularArticles/vol14-2/p099/p099.pdf}

Gabry, J. , Simpson, D. , Vehtari, A. , Betancourt, M. and
  Gelman, A. (2019), Visualization in Bayesian workflow.
  *J. R. Stat. Soc. A*, 182: 389-402. doi:10.1111/rssa.12378
  ([journal version](https://rss.onlinelibrary.wiley.com/doi/full/10.1111/rssa.12378),
   [preprint arXiv:1709.01449](https://arxiv.org/abs/1709.01449),
   [code on GitHub](https://github.com/jgabry/bayes-vis-paper))
}
\seealso{
\code{\link{as.shinystan}} for creating \code{shinystan} objects.

\code{\link{launch_shinystan_demo}} to try a demo.

\code{\link{launch_shinystan}} to launch the 'ShinyStan' interface 
  using a particular \code{shinystan} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/retrieve.R
\name{retrieve}
\alias{retrieve}
\title{Get summary statistics from shinystan object}
\usage{
retrieve(sso, what, ...)
}
\arguments{
\item{sso}{A \code{\link[=as.shinystan]{shinystan object}}.}

\item{what}{What do you want to get? See Details, below.}

\item{...}{Optional arguments, in particular \code{pars} to specify parameter
names (by default all parameters will be used). For NUTS sampler parameters
only (e.g. stepsize, treedepth) \code{inc_warmup} can also be specified to 
include/exclude warmup iterations (the default is \code{FALSE}). See
Details, below.}
}
\description{
From a shinystan object get rhat, effective sample size, posterior
quantiles, means, standard deviations, sampler diagnostics, etc.
}
\details{
The argument \code{what} can take on the values below. 'Args:
  \code{arg}' means that \code{arg} can be specified in \code{...} for this
  value of \code{what}.
\describe{
  \item{\code{"rhat"}, \code{"Rhat"}, \code{"r_hat"}, or \code{"R_hat"}}{returns: Rhat statistics. Args: \code{pars}}
  \item{\code{"N_eff"}, \code{"n_eff"}, \code{"neff"}, \code{"Neff"}, \code{"ess"}, or \code{"ESS"}}{returns: Effective sample sizes. Args: \code{pars}}
  \item{\code{"mean"}}{returns: Posterior means. Args: \code{pars}}
  \item{\code{"sd"}}{returns: Posterior standard deviations. Args: \code{pars}}
  \item{\code{"se_mean"} or \code{"mcse"}}{returns: Monte Carlo standard error. Args: \code{pars}}
  \item{\code{"median"}}{returns: Posterior medians. Args: \code{pars}.}
  \item{\code{"quantiles"} or any string with \code{"quant"} in it (not case sensitive)}{returns: 2.5\%, 25\%, 50\%, 75\%, 97.5\% posterior quantiles. Args: \code{pars}.}
  \item{\code{"avg_accept_stat"} or any string with \code{"accept"} in it (not case sensitive)}{returns: Average value of "accept_stat" (which itself is the average acceptance probability over the NUTS subtree). Args: \code{inc_warmup}}
  \item{\code{"prop_divergent"} or any string with \code{"diverg"} in it (not case sensitive)}{returns: Proportion of divergent iterations for each chain. Args: \code{inc_warmup}}
  \item{\code{"max_treedepth"} or any string with \code{"tree"} or \code{"depth"} in it (not case sensitive)}{returns: Maximum treedepth for each chain. Args: \code{inc_warmup}}
  \item{\code{"avg_stepsize"} or any string with \code{"step"} in it (not case sensitive)}{returns: Average stepsize for each chain. Args: \code{inc_warmup}}
}
}
\note{
Sampler diagnostics (e.g. \code{"avg_accept_stat"}) only available for
  models originally fit using Stan.
}
\examples{
# Using example shinystan object 'eight_schools'
sso <- eight_schools
retrieve(sso, "rhat")
retrieve(sso, "mean", pars = c('theta[1]', 'mu'))
retrieve(sso, "quantiles")
retrieve(sso, "max_treedepth")  # equivalent to retrieve(sso, "depth"), retrieve(sso, "tree"), etc.
retrieve(sso, "prop_divergent")
retrieve(sso, "prop_divergent", inc_warmup = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/drop_parameters.R
\name{drop_parameters}
\alias{drop_parameters}
\title{Drop parameters from a \code{shinystan} object}
\usage{
drop_parameters(sso, pars)
}
\arguments{
\item{sso}{A \code{\link[=as.shinystan]{shinystan object}}.}

\item{pars}{A character vector of parameter names. If the name of a 
non-scalar (e.g. vector, matrix) parameter is included in \code{pars} all 
of its elements will be removed. Currently it is not possible to remove 
only a subset of the elements of a non-scalar parameter.}
}
\value{
\code{sso}, with \code{pars} dropped.
}
\description{
Remove selected parameters from a \code{shinystan} object. This is useful if 
you have a very large \code{shinystan} object when you only want to look at a
subset of parameters. With a smaller \code{shinystan} object, 
\code{\link{launch_shinystan}} will be faster and you should experience 
better performance (responsiveness) after launching when using the
'ShinyStan' app.
}
\examples{
# Using example shinystan object 'eight_schools'
print(eight_schools@param_names)

# Remove the scalar parameters mu and tau
sso <- drop_parameters(eight_schools, pars = c("mu", "tau"))
print(sso@param_names)

# Remove all elements of the parameter vector theta
sso <- drop_parameters(sso, pars = "theta")
print(sso@param_names)

}
\seealso{
\code{\link{generate_quantity}} to add a new quantity to a
  \code{shinystan} object.
}
